/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/acces.h"               */
/*  Constants and definitions for the access control.                        */
/*                                                                           */
/*  Author : RG                         Last modification : 21/10/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ACCES_H                        /* If this file is not already        */
#define ACCES_H                        /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Dimensionnement generaux .................................................*/

#define NB_CARD_SLOTS         2        /* Nombre de slots SMC.               */
#define NB_CARD_SLOTS         2        /* Nombre de slots SMC.               */
#define MAX_CAS               6        /* Nombre max de slot CA.             */
#define MAX_SYSID            16        /* Max CA_system_id par CA.           */
#define MAX_CA_SRC            8        /* Nombre max de sources.             */
#define MAX_CA_STM            6        /* Flux: aud, vid, sub, ttx.          */
#define MAX_DSC             250        /* Max 250 descripteurs CA.           */

#define MAX_CA_VL (MAX_CA_SRC*MAX_CA_STM) /* Nombre de vl AUT_CA_API.        */


/* Changements a notifier avec "ca_desc[i].not" .............................*/

#define CAT_OFF      (IWORD)  0        /* Fin du flux EMM.                   */
#define CAT_ON       (IWORD)  1        /* Debut du flux EMM.                 */
#define CAT_STILL    (IWORD)  2        /* Chgt CAT sans chgt flux.           */
#define CAT_UPD      (IWORD)  3        /* Modif du flux EMM.                 */
#define PRG_OFF      (IWORD)  4        /* Fin prog (arret pour  CA)          */
#define PRG_ON       (IWORD)  5        /* Debut du programme.                */
#define STM_OFF      (IWORD)  6        /* Fin stream elementaire.            */
#define STM_ON       (IWORD)  7        /* Debut stream elementaire.          */
#define PAT_UPD      (IWORD)  8        /* Modif de la PAT.                   */
#define PRG_UPD      (IWORD)  9        /* Chgt PMT prg en cours.             */
#define STM_UPD      (IWORD) 10        /* Chgt PMT stream en cours.          */
#define STM_KEEP     (IWORD) 11        /* Laisse au, malgre No Righ          */
#define TRP_CHANGED  (IWORD) 12        /* Changement transpondeur.           */
#define SRC_ON       (IWORD) 13        /* Demarrage/def source.              */
#define SRC_OFF      (IWORD) 14        /* Arret source de donnee.            */
#define PRG_END      (IWORD) 15        /* Plus progr tous CA                 */
#define SMC_CLOSED   (IWORD) 16        /* Fermeture slot carte               */
#define SRC_START    (IWORD) 17        /* Demarrer dechif source             */
#define SRC_STOP     (IWORD) 18        /* Arreter dechif  source.            */
#define SDT_UPD      (IWORD) 19        /* Chargement/Changement SDT          */
#define NIT_UPD      (IWORD) 20        /* Chargement/Changement NIT          */
#define EXT_RETRY    (IWORD) 21        /* Nouvel essai Pb sur DVBCI          */
#define PAT_SAME     (IWORD) 22        /* Rechargement meme PAT              */
#define CAT_SAME     (IWORD) 23        /* Rechargement mem CAT               */
#define NIT_SAME     (IWORD) 24        /* Rechargement mem NIT               */
#define SDT_SAME     (IWORD) 25        /* Rechargement mem SDT               */


/* Numero de carte concernee ................................................*/
 
#define NO_CARD_SLOT   (INT)  0        /* Aucun slot de carte.               */
#define CARD_SLOT0     (INT)  0        /* 1er slot de carte.                 */
#define CARD_SLOT1     (INT)  1        /* 2nd slot de carte.                 */
#define USE_CARD_SLOT0 (INT)  1        /* 1er slot de carte.                 */
#define USE_CARD_SLOT1 (INT)  2        /* 2nd slot de carte.                 */
#define NB_CARD_SLOTS         2        /* Nombre de slots.                   */
#define MSK_CARD_SLOTS 0xFFFFFFFC      /* Error mask (logical OR on SLOTSx). */


/* Statut carte a puce, selon slot CA .......................................*/

#define CARD_OK        (INT)  0        /* Carte reconnue.                    */
#define CARD_REFUSED   (INT)  1        /* Carte reconnue et refusee          */
#define CARD_ERROR     (INT)  2        /* Erreur avec la carte.              */
#define NO_CARD        (INT)  3        /* Pas de carte dans le slot          */
#define CARD_BAD_INS   (INT)  4        /* Carte mal inseree.                 */


/* Modes de souscriptions PPV ...............................................*/

#define PPV_SUBSCRIBE         (INT) 0  /* Souscrire.                         */
#define PPV_UNSUBSCRIBE       (INT) 1  /* Annuler une souscription.          */
#define PPV_PREBOOK           (INT) 2  /* Faire un pre-achat.                */
#define PPV_SUB_NBMOD         (INT) 3  /* Nb de modes souscription.          */
#define BUY_PPV_CREDIT        (INT) 0  /* Achat de credits.                  */
#define BUY_PPV_PROGRAM       (INT) 1  /* Achat par numero de prog.          */
#define BUY_PPV_CLASS         (INT) 2  /* Achat par classe.                  */
#define BUY_PPV_THEME_AND_LVL (INT) 3  /* Achat par theme et niveau          */
#define BUY_PPV_PRICE         (INT) 4  /* Achat par prix.                    */
#define BUY_PPV_CEILING       (INT) 5  /* Achat par plafond de prix          */
#define PREBOOK_SURVEY        (INT) 6  /* Declenche appel operateur          */
#define PPV_SUB_NBTYP         (INT) 7  /* Nb types de souscription.          */


/* Reponse a l'offre d'achat PPV au zapping .................................*/
 
#define PPV_ACCEPT   (INT)    0        /* On accepte l'achat.                */
#define PPV_REFUSE   (INT)    1        /* On refuse l'achat.                 */
#define PPV_CANCEL   (INT)    2        /* On arrete l'achat PPV/T.           */


/* Modes de fonctionnement ..................................................*/

#define PPV                   (INT)0x0000001     /* Pay-Per-View.            */
#define PREBOOKED_PPV         (INT)0x0000002     /* Prebooked PPV.           */
#define SUBSCRIBE_PPV         (INT)0x0000004     /* PPV par voie de retour   */
#define IMPULSE_PPV           (INT)0x0000008     /* PPV au zapping.          */
#define AUTOMATIC_PREVIEW     (INT)0x0000010     /* Lancement de previe      */
#define PREVIEW_ON_REQUEST    (INT)0x0000020     /* Demande de preview       */
#define PINCODE_PROTECTION    (INT)0x0000040     /* Protection par PIN.      */
#define MATURITY_RATING       (INT)0x0000080     /* Code parental.           */
#define NEED_PAT_TABLE        (INT)0x0000100     /* Necessite la PAT.        */
#define NEED_PMT_TABLE        (INT)0x0000200     /* Necessite la PAT.        */
#define NEED_CAT_TABLE        (INT)0x0000400     /* Necessite la PAT.        */
#define NEED_NIT_TABLE        (INT)0x0000800     /* Necessite la NIT.        */
#define NEED_SDT_TABLE        (INT)0x0001000     /* Necessite la SDT.        */
#define NEED_TDT_TABLE        (INT)0x0002000     /* Necessite la TDT.        */
#define NEED_EIT_TABLE        (INT)0x0004000     /* Necessite la EIT.        */
#define PMT_WITHOUT_SYSID     (INT)0x0010000     /* PMT meme qd pas de sid   */
#define ONE_DSC_PER_SYSID     (INT)0x0020000     /* Un seul descripteur CA de*/
                                                 /* la PMT par CA_System_Id  */
                                                 /* reference par le CA.     */
#define ONE_DSC_PER_SLOT      (INT)0x0040000     /* Un seul descripteur CA de*/
                                                 /* la PMT par slot du CA.   */
#define SLOT_TIED_TO_SRC      (INT)0x0080000     /* Chaque slot est lie a un */
                                                 /* hardware (DVBCI).        */
#define CHANNEL_LOCK          (INT)0x0100000     /* Verrouillage de chaines. */
#define NEED_EXTERNAL_EMM     (INT)0x0200000     /* EMM a recuperer hors DVB */
#define NEED_EXTERNAL_ECM     (INT)0x0400000     /* ECM a recuperer hors DVB */


/*
    2    Structures and typedefs
    _____________________________
*/


typedef struct                         /* Reference a un CA:                 */
  BEGIN                                /* ---------------------------------- */
    USHORT          ecm              ; /* PID des ECM.                       */
    USHORT          sid              ; /* CA_system_id.                      */
    UCHAR           niv              ; /* Niveau: stream/programme 0/1.      */
    UCHAR           ref              ; /* Reference a passer au slot CA.     */
    UCHAR           slot             ; /* Slot CA associe ou 0xFF.           */
    UCHAR           use              ; /* inutilisable(0) / a utiliser(1) /  */
                                       /* deja utilise(2).                   */
  END CA_dsc                         ; /* ---------------------------------- */

typedef struct                         /* Contexte d'une source:             */
  BEGIN                                /* ---------------------------------- */

/* 4 octets .................................................................*/

    UINT       iod                   ; /* IOD demux/pvr associe a la source. */
    UCHAR     *nxt_cat_des           ; /* Memo pointeur prochain descr CAT.  */
    UCHAR     *nxt_pmt_des           ; /* Memo pointeur prochain descr PMT.  */
    INT        dec_rteid[MAX_CA_STM] ; /* Id route pour IND_REPORT decodeur. */

/* 2 octets .................................................................*/

    USHORT     elm_pid  [MAX_CA_STM] ; /* PID des flux elementaires.         */
    USHORT     lg_pmt                ; /* Longueur de la PMT courante.       */
    USHORT     lg_pat                ; /* Longueur de la CAT courante.       */
    USHORT     lg_cat                ; /* Longueur de la CAT courante.       */
    USHORT     lg_sdt                ; /* Longueur de la SDT courante.       */
    USHORT     lg_nit                ; /* Longueur de la NIT courante.       */
    USHORT     prog_nb               ; /* Program number courant.            */
    USHORT     tsid                  ; /* Transport Stream Id courant.       */
    USHORT     tsid_sdt              ; /* Transport Stream Id de la SDT.     */
    USHORT     onid                  ; /* Original Network Id courant.       */
    USHORT     lgr_pmt               ; /* Memo lgr restant a parser dans PMT.*/
    USHORT     lgr_cat               ; /* Memo lgr restant a parser dans CAT.*/
    USHORT     lgr_sec               ; /* Memo lgr restant a parser dans la  */
                                       /* section courante de la CAT.        */
/* 1 octet  .................................................................*/

    UCHAR      elm_state[MAX_CA_STM] ; /* Etat des flux elementaires.        */
    UCHAR      pmt      [      1024] ; /* PMT courante.                      */
    UCHAR      pat      [      4096] ; /* PAT courante.                      */
    UCHAR      cat      [      4096] ; /* CAT courante.                      */
    UCHAR      sdt      [      4096] ; /* SDT courante.                      */
    UCHAR      nit      [      4096] ; /* NIT courante.                      */
    UCHAR      patvers               ; /* Numero de version PAT              */
    UCHAR      catvers               ; /* Numero de version CAT              */
    UCHAR      sdtvers               ; /* Numero de version SDT              */
    UCHAR      nitvers               ; /* Numero de version NIT              */
    UCHAR      state                 ; /* Arretee / demaree : 0 / 1          */
    UCHAR      type                  ; /* Type de source: $DMX_SRC ou $HD_SRC*/
    UCHAR      nbstm                 ; /* Nombre de streams utilisables.     */
    UCHAR      prg_state             ; /* Etat ON/OFF du programme courant.  */
    CHAR       pmt_parsed_loop       ; /* Memo derniere boucle parsee de PMT.*/
  END CA_src                         ; /* ---------------------------------- */

typedef struct                         /* Contexte de stream:                */
  BEGIN                                /* ---------------------------------- */

/* 4 octets .................................................................*/

    UINT            ca_id            ; /* Iod du slot CA concerne.           */
    INT             flt_iod          ; /* Iod du filtre concernes (cles).    */
    INT             dec_iod          ; /* Iod du decodeur concerne ou -1.    */
    UINT            slot_ad          ; /* Info complementaire propre slot CA */
    INT             tid              ; /* Id du timer de poling.             */
    INT             wait_to          ; /* Id timer de rappel sur Wait Status.*/
    INT             dec_to           ; /* Id timer d'attente Status decodeur.*/
    UINT            slot_more        ; /* Statut specifique slot CA courant  */
    CA_dsc         *dsc              ; /* Donnees CA pour stream.            */
    CA_dsc         *bdsc             ; /* Pointe sur zone memoire allouee aux*/
                                       /* donnees CA de PMT pour stream.     */
/* 2 octets .................................................................*/

    USHORT          pid              ; /* PID du stream.                     */

/* 1 octet ..................................................................*/

    UCHAR           mpeg             ; /* Type mpeg du stream.               */
    UCHAR           type             ; /* Type du stream ($VIDEO_STREAM etc.)*/
    UCHAR           stat             ; /* Status: clear/crypte/decrypte/     */
                                       /*   attente du choix de l'utilisateur*/
    UCHAR           st               ; /* Dernier status remonte par le CA.  */
    UCHAR           idx              ; /* Index CA courant dans dsc[].       */
    UCHAR           nb_dsc           ; /* Nombre de descripteurs dans dsc[]. */
    UCHAR           sz               ; /* Taille de dsc en nb de descripteurs*/
    UCHAR           nb_bdsc          ; /* Nombre de descripteurs dans bdsc[].*/
    UCHAR           szb              ; /* Taille de bdsc (nb de descripteurs)*/
    UCHAR           slot_st          ; /* Status generique du slot CA courant*/
    UCHAR           dec_sta          ; /* Statut courant du decodeur.        */
  END CA_stm                         ; /* ---------------------------------- */

typedef struct                         /* Description d'un controle d'acces: */
  BEGIN                                /* ---------------------------------- */

/* 4 octets ................................................................ */

    INT             modes            ; /* Modes de fonctionnement.           */
    IWORD           (*inf)()         ; /* Adr. procedure info sur les slots. */
    IWORD           (*ini)()         ; /* Adr. procedure init specifique.    */
    IWORD           (*end)()         ; /* Adr. procedure end specifique.     */
    IWORD           (*not)()         ; /* Adr. proc. notifier chgt / streams.*/
    IWORD           (*set)()         ; /* Adr. procedure setval specifique.  */
    IWORD           (*get)()         ; /* Adr. procedure getval specifique.  */
    IWORD           (*pin)()         ; /* Adr. procedure enter_pincode specif*/
    IWORD           (*tim)()         ; /* Adr. procedure trt CA_TIMER.       */
    IWORD           (*sch)()         ; /* Adr. procedure start channel       */
    IWORD           (*sfl)()         ; /* Adr. procedure start filter        */
    IWORD           (*lch)()         ; /* Adr. procedure lock channel        */
    IWORD           (*dat)()         ; /* Adr. procedure msg received        */

/* 2 octets ................................................................ */

    USHORT          sid   [      2]  ; /* CA system Id rsv selon norme: [a;b[*/

/* 1 octet ................................................................. */

    CHAR            name  [STDNAME]  ; /* Nom (en majuscules).               */
    CHAR            emb              ; /* 0/1 slots externes/CA embarque.    */
    CHAR            nb_slots         ; /* Nombre de slots (1 pour Embedded). */
    CHAR            nb_src           ; /* Nombre de sources supportees.      */
  END CA_desc                        ; /* ---------------------------------- */

typedef struct                         /* Contexte d'un slot de CA:          */
  BEGIN                                /* ---------------------------------- */

/* 4 octets ................................................................ */

    IWORD           ca_id            ; /* Iod CA du slot de controle d'acces.*/

/* 2 octets ................................................................ */

    USHORT          sysid[MAX_SYSID] ; /* Systemes ids du CA.                */
    USHORT          emm_pid          ; /* PID des EMM pour le CA.            */
    USHORT          o_emm_pid        ; /* Ancien PID des EMM avant CAT_UPDATE*/

/* 1 octet ................................................................. */

    UCHAR           fsysid[MAX_SYSID]; /* Flag indiquant tests a effectuer.  */
    UCHAR           sysref[MAX_SYSID]; /* Reference CA a rappeler / sysid.   */
    UCHAR           cat_ref          ; /* Reference CA a rapeler pour les EMM*/
    UCHAR           o_cat_ref        ; /* Ancienne ref CA a rappeler pour EMM*/
    UCHAR           nbsys            ; /* Nb de Systeme id du CA.            */
    UCHAR           state            ; /* Etat du controle d'acces.          */
    UCHAR           smc              ; /* Accessibilite des smartcards.      */
    UCHAR           activation       ; /* Gestion notifications % activations*/
    UCHAR           src  [MAX_CA_SRC]; /* Numeros sources confiees au slot.  */
    UCHAR           nb_active_src    ; /* Nombre de sources confiees au slot.*/
  END CA_ctx                         ; /* Fin de la structure.               */

typedef struct                         /* Structure d'une notification:      */
  BEGIN                                /* ---------------------------------- */

/* 4 octets ................................................................ */

    IWORD           code             ; /* Code de la notification.           */
    IWORD           tsid             ; /* Transport stream id.               */
    INT             ca_id            ; /* Identifiant du slot CA.            */
    INT             iod              ; /* IOD associe le cas echeant.        */

/* 2 octets ................................................................ */

    USHORT          prog_nb          ; /* Numero du programe (service id.)   */
    USHORT          pid              ; /* PID du flux.                       */
    USHORT          ecm              ; /* PID des ECMs lies au flux.         */
    USHORT          emm              ; /* PID des EMMs lies au programme.    */
    USHORT          o_emm            ; /* Ancien PID des ECMs lies au prog.  */

/* 1 octet ................................................................. */

    UCHAR           src              ; /* Source concernee.                  */
    UCHAR           stm              ; /* Stream concerne.                   */
    UCHAR           ref              ; /* Reference a passer au CA.          */
    UCHAR           o_ref            ; /* Reference precedente passee au CA. */
    UCHAR           mpeg             ; /* Type du stream MPEG.               */
    UCHAR           niv              ; /* Niveau de cryptage: flux(0)/prog(1)*/
    UCHAR           flags            ; /* Flag indiquant ce qu'on attend en  */
                                       /* retour de la part du slot CA:      */
                                       /*  b0: si 1, evt CA_STREAM_STATUS    */
                                       /*  b1: si 1, on est en polling sur le*/
                                       /*  slot, pas de messages tant que le */
                                       /*  status ne change pas (DVBCI).     */
 END CA_not                          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    _____________________________________
*/


/* Procedure d'initialisation ...............................................*/

  IWORD init_ca_api         (void                                            );


/* Procedures de l'API controle d'acces .....................................*/

  IWORD CA_getslotlist      (CHAR*,INT*,INT*                                 );
  IWORD CA_getslotinfo      (CHAR*,INT*,CHAR*,CHAR*,CHAR*,INT*, INT*,INT*    );
  IWORD CA_getsysidlist     (INT,INT*,INT[][2],INT*                          );
  IWORD CA_getslotsysid     (INT,INT*,INT*,INT*                              );
  IWORD CA_init             (INT,INT,INT,INT,INT,INT*                        );
  IWORD CA_end              (INT,INT*                                        );
  IWORD CA_table_changed    (INT,INT,UCHAR*,INT,INT*                         );
  IWORD CA_stream_changed   (INT,INT,INT,INT,INT,INT,INT,INT,INT*            );
  IWORD CA_source_changed   (INT,INT,INT,INT,INT,INT*                        );
  IWORD CA_set_source       (INT,INT,INT,INT*                                );
  IWORD CA_setval           (INT,INT,INT,INT,INT,CHAR*,INT,INT*              );
  IWORD CA_getval           (INT,INT,INT,INT,INT,CHAR*,INT,INT*              );
  IWORD CA_enter_pincode    (INT,INT,INT,INT,INT,INT*                        );
  IWORD CA_get_stm_ca       (INT,INT,INT,UCHAR**,INT*,INT*                   );
  IWORD CA_get_prg_ca       (INT,INT,UCHAR**,INT*,INT*                       );


/* Sous-programmes publics ..................................................*/

  IWORD maj_ca_sysid        (IWORD,IWORD,IWORD,IWORD,USHORT*, UCHAR*         );
  IWORD ca_event            (IWORD,INT,INT,UCHAR*,INT,UINT,UINT, UINT        );
  void  parse_pmt           (IWORD,IWORD,IWORD,IWORD,IWORD, UCHAR**           ,
                             IWORD*,IWORD*,IWORD*,IWORD*                     );
  void  parse_cat           (IWORD,IWORD,IWORD,UCHAR**,IWORD*, IWORD*,IWORD*  ,
                             IWORD*                                          );
  void  get_pmt_pid         (IWORD,IWORD,IWORD*,IWORD*,IWORD*, UCHAR**       );
  void  get_dvb_table       (IWORD,IWORD,UCHAR**,IWORD*                      );
  void  get_dvb_triplet     (IWORD,USHORT*,USHORT*,USHORT*                   );
  IWORD ca_get_stm_iod      (IWORD,UINT,UINT*                                );


#endif                                 /* End of ACCES_H.                    */
