/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/adc_stm32.h"           */
/*  Declaration of the procedures of the DACs'.                              */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/23/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ADC_STM32_H                    /* If this file is not already        */
#define ADC_STM32_H                    /* included, include it.              */

/*
    1    Defines
    ____________
*/



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parametres du device ADC           */
  BEGIN                                /* ---------------------------------- */
    UCHAR         adc_num            ; /* Timer number for accurate channel  */
    UCHAR         tim_num            ; /* Timer number for accurate channel  */
    UCHAR         dma_num            ; /* DMA reference number               */
    UCHAR         jext_trig          ; /* Trigger extern for injected channel*/
    UCHAR         accu_ch            ; /* Number of the accurate channel     */
    UINT          dma_ch             ; /* DMA reference channel              */
    UCHAR         dma_st             ; /* DMA reference stream               */
  END             Adc_stm32_def      ; /* ---------------------------------- */

typedef struct                         /* Low level Context                  */
  BEGIN                                /* ---------------------------------- */
    UCHAR         dev                ; /* Device index for ADC driver        */
    UCHAR         adc_num            ; /* ADC number                         */
    UCHAR         tim_num            ; /* TIM number                         */
    UCHAR         dma_num            ; /* DMA number                         */
    UCHAR         jext_trigg         ; /* external trig for injected channel */
    UCHAR         accu_ch            ; /* accurate channel number            */
    UINT          width              ; /* Width                              */
    UINT          wmask              ; /* mask associated to the width       */
    volatile Tim *timaddr            ; /* Timer address                      */
    UINT          freq               ; /* Timer frequency                    */
  END             Adc_stm32          ; /* ---------------------------------- */




/*
    3    Public variables
    _____________________
*/

extern const Adc_desc adc_desc_stm32        ; /* ADC  low level interface    */


extern       Adc_stm32 adc_stm32[]   ; /* Context table of tim_stm32.c       */

/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD handlerADC_stm32                 (IWORD,IWORD                          );

IWORD handlerDAC_stm32                 (IWORD,IWORD                          );
IWORD handlerdmaDAC_stm32              (IWORD,IWORD                          );


#endif                                 /* End of ADC_STM32_H                 */
