/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/classe.h"              */
/*  Constants and definitions for PanTalk objects and class.                 */
/*                                                                           */
/*  Author : JYA                        Last modification : 22/10/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef CLASSE_H                       /* If this file is not already        */
#define CLASSE_H                       /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Acces aux champs des variable...........................................*/

#define V_TAB       (USHORT)0x1000     /* Bit "tableau" (champ Hcla_ctx.att) */
#define VTYP(pvar)  (pvar)->type       /* Type de la variable.               */
#define VLGV(pvar)  (pvar)->lg         /* Longueur conceptuelle de variable. */
#define VOFB(pvar)  (pvar)->offset     /* Offset de la valeur dans le buffer.*/
#define VPAR(pvar)  (pvar)->par        /* Valeur du parametre.               */


/* La variable est-elle un tableau ? ........................................*/

#define VTAB(pvar)  (((pvar)->att AND V_TAB) NE 0)


/*****************************************************************************
 * Tableaux : VNBE1 nombre d'elements dans la dimension 1                    *
 *            VNBE2 nombre d'elements dans la dimension 2                    *
 *            VNBE  nombre d'elements au total                               *
 *            VOFE  offset entre deux elements                               *
 *****************************************************************************/

#define VDIM(pvar)  (VTAB(pvar)                                              ?\
                     (IWORD)((Hcla_arr*)(pvar))->dim                         :\
                     0                                                       )
#define VNBE1(pvar) (VTAB(pvar)                                              ?\
                     (IWORD)((Hcla_arr*)(pvar))->dim1                        :\
                     1                                                       )
#define VNBE2(pvar) (VTAB(pvar)                                              ?\
                     (IWORD)((Hcla_arr*)(pvar))->dim2                        :\
                     1                                                       )
#define VNBE(pvar)  (VTAB(pvar)                                              ?\
                     (IWORD)((Hcla_arr*)(pvar))->dim1 *                       \
                     (IWORD)((Hcla_arr*)(pvar))->dim2                        :\
                     1                                                       )
#define VOFE(pvar)  (VTAB(pvar)                                              ?\
                     (IWORD)((Hcla_arr*)(pvar))->offset                      :\
                     (pvar)->lg                                              )


/* Conversion des types de donnees ..........................................*/

#define EXT_CON               1        /* Externe -> conceptuelle            */
#define CON_EXT               2        /* Externe <- conceptuelle            */
#define EXTERNE               1        /* Externe                            */
#define CONCEPTUEL            2        /* Conceptuelle                       */


/* Definition des types de donnees ..........................................*/

#define T_ALPHA               0        /* Chaine alphabetique.               */
#define T_ALPHANU             1        /* Chaine alphanumerique.             */
#define T_TEXT                2        /* Texte ASCII.                       */
#define T_FORMAT              3        /* Format.                            */
#define T_MEMORY              4        /* Zone memoire.                      */
#define T_ADDRESS             5        /* Adresse.                           */
#define T_DATE                6        /* Date.                              */
#define T_TIME                7        /* Heure.                             */
#define T_BYTE                8        /* Entier 8 bits signe.               */
#define T_BOOLEAN             9        /* Entier 8 bits 1/0 (O/N).           */
#define T_SHORT              10        /* Entier 16 bits signe.              */
#define T_CHAR               11        /* Entier 16 bits non signe (Unicode) */
#define T_INT                12        /* Entier 32 bits signe.              */
#define T_OBJ                13        /* Reference : entier 32 bits ns.     */
#define T_FLOAT              14        /* Flottant 32 bits.                  */
#define T_DOUBLE             15        /* Flottant 64 bits.                  */
#define T_LONG               16        /* Entier 64 bits signe.              */
#define T_LONGITUDE          17        /* Longitude.                         */
#define T_LATITUDE           18        /* Latitude.                          */
#define T_WAP                19        /* WAP                                */
#define MAX_TYPE             19        /* Type de donnee maximum.            */


/* Genres des variable(exactement les types CPI utilises ....................*/

#define G_BYT                 0        /* Entier.                            */
#define G_SHT                 1        /* Entier.                            */
#define G_INT                 2        /* Entier.                            */
#define G_LNG                 3        /* Entier.                            */
#define G_FLT                 4        /* Flottant.                          */
#define G_DBL                 5        /* Flottant.                          */
#define G_OBJ                 6        /* Reference.                         */
#define G_CHR                 7        /* Caractere.                         */
#define G_STR                 8        /* Chaine.                            */
#define G_ADR                 9        /* Adresse de zone memoire.           */
#define G_MEM                10        /* Zone Memoire.                      */
#define G_WAP                11        /* WAP                                */
#define NB_VAR_GENRE         12        /* Nombre maximum de genres.          */


/* Valeurs particulieres ....................................................*/

#define BYT_MIN      (CHAR)0x80        /* BYTE - Valeur minimale (- 128)     */
#define BYT_MAX      (CHAR)0x7F        /* BYTE - Valeur maximale (+ 127)     */
                                       /*                                    */
#define SHT_MIN  (SHORT) 0x8000        /* SHORT - Valeur minimale (- 32768)  */
#define SHT_MAX  (SHORT) 0x7FFF        /* SHORT - Valeur maximale (+ 32767)  */
#define CHR_MIN  (USHORT)     0        /* CHAR - Valeur minimale  (0)        */
#define CHR_MAX  (USHORT)0xFFFF        /* CHAR - Valeur maximale (+ 65 535)  */
                                       /*                                    */
#ifdef INT_MAX                         /*                                    */
#undef INT_MAX                         /*                                    */
#undef INT_MIN                         /*                                    */
#endif                                 /*                                    */
                                       /*                                    */
#define INT_MAX (INT)0x7FFFFFFF        /* INT - Valeur min. (+2147483647)    */
#define INT_MIN (INT)0x80000001        /* INT - Valeur max. (-2147483647)    */
#define LNG_MAX (INT)0x7FFFFFFF        /* INT - Valeur max. (+2147483647)    */
#define VIDEI   (INT)0x80000000        /* INT - Valeur vide (-2147483648)    */


/*
    2    Structures and typedefs
    ____________________________
*/


typedef struct                         /* Blocs pour les classes en memoire: */
  BEGIN                                /* ---------------------------------- */
    UINT            lg_buf           ; /* Taille de buffer de stockage/objet */
    USHORT          mod              ; /* Identifiant de module/package      */
    USHORT          super            ; /* Identifiant de la super-classe     */
    USHORT          nb_var           ; /* Nombre de variablepropres        */
    USHORT          nb_her           ; /* Nombre de variableheritees       */
    UCHAR           nb_buf           ; /* Nombre de buffers                  */
    UCHAR           nb_nom           ; /* Nombre de noms de buffers          */
    UCHAR           att              ; /* Attributs                          */
  END Hcla_ctx                       ; /* ---------------------------------- */

typedef struct                         /* Variable en memoire:               */
  BEGIN                                /* ---------------------------------- */
    UINT            offset           ; /* Offset valeur dans le buffer       */
    USHORT          lg               ; /* Longueur dans le buffer            */
    USHORT          att              ; /* Attributs de la variable           */
    UCHAR           type             ; /* Type de la variable                */
    UCHAR           par              ; /* Parametre de la variable           */
  END Hcla_var                       ; /* ---------------------------------- */

typedef struct                         /* Variable tableau:                  */
 BEGIN                                 /* ---------------------------------- */
  Hcla_var          var              ; /* Description minimum de la variable */
  USHORT            offset           ; /* Offset entre 2 elements            */
  USHORT            dim1             ; /* Nombre d'elements dimension 1      */
  USHORT            dim2             ; /* Nombre d'elements dimension 2      */
  UCHAR             dim              ; /* Nombre de dimensions (1/2)         */
 END Hcla_arr                        ; /* ---------------------------------- */

typedef struct                         /* Interface du debuffer:             */
  BEGIN                                /* ---------------------------------- */
    IWORD         (*get_var_name)()  ; /* Donne nom d'une val. a adr. connue.*/
  END Hint_cladbg                    ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Module class.c............................................................*/

  IWORD cla_pop_defaut      (void                                            );
  IWORD cla_push_defaut     (IWORD                                           );
  IWORD class_alloc_ctx     (Hcla_ctx**                                      );
  IWORD class_free_set      (IWORD                                           );
  IWORD is_class_module     (INT,INT                                         );
  IWORD find_class_set      (CHAR*,INT,IWORD,INT*                            );
  IWORD class_init          (void                                            );
  IWORD compare_object      (CHAR*,CHAR*,INT*                                );
  IWORD copy_object         (CHAR*,CHAR*                                     );
  IWORD del_object_oid2     (INT,INT                                         );
  IWORD set_gr_oid          (INT,INT                                         );
  IWORD get_gr_oid          (INT,INT*                                        );
  IWORD super_class         (INT,INT                                         );
  IWORD set_ivar_val        (INT,INT,INT,INT,INT                             );
  IWORD delete_class_module (IWORD                                           );
  IWORD empty_object        (CHAR*                                           );
  IWORD get_array_nblines   (CHAR[],INT*                                     );
  IWORD empty_zone          (IWORD,UCHAR*                                    );
  IWORD find_class_att      (CHAR*,INT*,INT*                                 );
  IWORD get_abs_elt         (Hcla_var*,UCHAR*,INT,UCHAR**,INT*               );
  IWORD get_class_info      (CHAR*,INT*,INT*,INT*,INT*                       );
  IWORD get_class_oid       (CHAR*,INT*,INT*,INT*                            );
  IWORD get_elt             (Hcla_var*,UCHAR*,INT,INT,UCHAR**,INT*           );
  IWORD get_var_type        (CHAR[],INT*,INT*,INT*                           );
  IWORD reset_array         (CHAR[]                                          );
  IWORD var_copy            (CHAR[],INT,INT,CHAR[],INT,INT,INT*              );
  IWORD getclavar           (IWORD,IWORD,Hcla_var**,UCHAR**                  );
  IWORD get_var_name        (IWORD,IWORD,CHAR*                               );
  IWORD getclavar2          (IWORD,IWORD,INT,Hcla_var**,UCHAR**              );
  IWORD init_cla_buf        (IWORD,IWORD                                     );
  IWORD reset_object        (CHAR*                                           );
  IWORD reset_zone          (IWORD,UCHAR*                                    );
  IWORD searchval           (CHAR[],INT*,INT*,Hcla_var**,UCHAR**             );
  IWORD searchvar           (INT,INT,INT*,Hcla_var**,UCHAR**                 );
  IWORD get_var_address     (CHAR[],Hcla_var**,UCHAR**,INT*,INT*,INT*        );
  IWORD set_array_nblines   (CHAR[],INT                                      );
  IWORD get_var_kind        (CHAR[],INT*                                     );
  IWORD set_class_oid2      (INT,INT,INT                                     );
  IWORD set_object_buffer   (CHAR*,INT                                       );
  IWORD get_object_buffer   (CHAR*,INT*                                      );
  IWORD set_class_oid       (CHAR*,INT,INT,INT*                              );

  
/* Module datatype.c ........................................................*/

  IWORD lgtype              (Hcla_var*,IWORD,IWORD*,IWORD*                   );
  IWORD integer_to_date     (IWORD,IWORD,IWORD,INT*                          );
  IWORD time_to_integer     (INT,IWORD*,IWORD*,IWORD*                        );
  IWORD lire_val            (UCHAR*,UCHAR*,INT                               );
  IWORD ecrire_val          (UCHAR*,UCHAR*,INT                               );
  IWORD conver              (Hcla_var*,UCHAR*,UCHAR*,IWORD,IWORD*,IWORD*     );


#endif                                 /* End of CLASSE_H.                   */
