/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/dac_stm32.h"           */
/*  Declaration of the procedures of the DACs'.                              */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/23/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DAC_STM32_H                    /* If this file is not already        */
#define DAC_STM32_H                    /* included, include it.              */

/*
    1    Defines
    ____________
*/



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parametres du device DAC           */
  BEGIN                                /* ---------------------------------- */
    UCHAR           mode             ; /* 0 -> mono, 1->stereo               */
    UCHAR           dac_num          ; /* Number of the DAC                  */
    UCHAR           tim_num          ; /* Timer number                       */
    UCHAR           dma_num          ; /* DMA reference number               */
//    UCHAR           dma_ch           ; /* DMA reference channel              */
//#ifdef VM_STM32M4
//    UCHAR           dma_st           ; /* DMA reference stream               */
//#endif
    UCHAR           sdev             ; /* DMA sub-device number              */
  END               Dac_stm32_def    ; /* ---------------------------------- */


typedef struct                         /* Low level Context                  */
  BEGIN                                /* ---------------------------------- */
    INT             dev              ; /* Device index for DAC driver        */
    UCHAR           dac_num          ; /* Number of the DAC (0 ou 1)         */
    UCHAR           tim_num          ; /* Number of the timer                */
    UCHAR           dma_num          ; /* DMA number                         */
    UCHAR           dma_ch           ; /* DMA channel number                 */
    UCHAR           dma_sdev         ; /* DMA channel/stream number          */
    UINT            width            ; /* Width                              */
    volatile Tim   *timaddr          ; /* Timer address                      */
  END               Dac_stm32        ; /* ---------------------------------- */



/*
    3    Public variables
    _____________________
*/

extern const Dac_desc dac_desc_stm32_stereo ; /* DAC  low level interface    */
extern const Dac_desc dac_desc_stm32_mono   ; /* DAC  low level interface    */

extern       Dac_stm32 dac_stm32[]   ; /* Context table of tim_stm32.c       */

/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD handlerDAC_stm32                 (IWORD,IWORD                          );
IWORD handlerdmaDAC_stm32              (IWORD,IWORD                          );


#endif                                 /* End of DAC_STM32_H                 */
