/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/dict.h"                */
/*  Constants and definitions for dictionaries.                              */
/*                                                                           */
/*  Author : FA                         Last modification : 22/10/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DICT_H                         /* If this file is not already        */
#define DICT_H                         /* included, include it.              */


/*
    1    Defines
    ____________
*/


#define DICT_ROM           0x01        /* Dictionnaire en ROM.               */
#define DICT_STATIQUE      0x02        /* Dictionnaire statique.             */
#define DICT_CLE           0x04        /* Dictionnaire avec cle.             */
#define DICT_MULTIPLE      0x08        /* Dictionnaire avec cle multiple.    */
#define DICT_DONNEE        0x10        /* Dictionnaire avec donnee etendue.  */
#define DICT_LG_NOM           7        /* Longueur du nom d'un dictionnaire. */


/*
    2    Structures and typedefs
    ____________________________
*/


/*  structure Dict, dictionnaire statique en ROM ----------------------------*/

typedef struct                         /* Dictionnaire en memoire.           */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[8]        ;   /* Nom du dictionnaire, 8 cara. max.  */
    USHORT          nbf            ;   /* Nombre de fichiers sources.        */
    USHORT          nbe            ;   /* Nombre total de chaines.           */
    USHORT          szt            ;   /* Nombre d'octets arbre des chaines. */
    USHORT          szd            ;   /* nombre d'octets donnees etendues.  */
    USHORT          typ            ;   /* type du dico:                      */
                                       /* 1 -> arbre compresse               */
                                       /* 2 -> liste                         */
                                       /* 0 -> liste sans clefs              */
    CHAR           *fname          ;   /* Adr. table des noms de fichiers.   */
    USHORT         *index          ;   /* Adr. table des numeros d'ordres.   */
    UCHAR          *tree           ;   /* Adr. arbre des chaines.            */
    USHORT         *offset         ;   /* Adr. table offset donnees etendues.*/
    UCHAR          *data           ;   /* Adr. donnees etendues.             */
  END               Dict           ;   /* ---------------------------------- */


/*  structure Dictstat, dictionnaire statique en ROM ------------------------*/

typedef struct                         /* Dictionnaire statique en memoire.  */
  BEGIN                                /* ---------------------------------- */
                                       /* Les structures utilisees pour la   */
                                       /* description d'un dictionnaire      */
                                       /* doivent toutes commencer par le nom*/
                                       /*  du dictionnaire.                  */
    CHAR        name[DICT_LG_NOM+1];   /* Nom du dictionnaire                */
                                       /* Le reste des champs est utilise    */
                                       /* specifiquement par la partie .     */
                                       /* dictionnaire statique ou           */
                                       /* dictionnaire dynamique, ils sont   */
                                       /* libres.                            */
    USHORT          nbf            ;   /* Nombre de fichiers sources.        */
    USHORT          nbe            ;   /* Nombre total de chaines.           */
    USHORT          szt            ;   /* Nombre d'octets arbre des chaines. */
    USHORT          szd            ;   /* nombre d'octets donnees etendues.  */
    UCHAR          *ushorts        ;   /* Liste de USHORT.                   */
    INT             index          ;   /* Off7 table des numeros d'ordres.   */
    INT             offset         ;   /* Off7 table offset donnees etendues.*/
    UCHAR          *uchars         ;   /* Liste d'octets.                    */
    INT             fname          ;   /* Off7 table des noms de fichiers.   */
    INT             tree           ;   /* Off7 arbre des chaines.            */
    INT             data           ;   /* Off7 donnees etendues.             */
  END               Dictstat       ;   /* ---------------------------------- */


/*  structure Dictdyn, dictionnaire dynamique -------------------------------*/

typedef struct                         /* Dictionnaire dynamique en memoire. */
  BEGIN                                /* ---------------------------------- */
                                       /* Les structures utilisees pour la   */
                                       /* description d'un dictionnaire      */
                                       /* doivent toutes commencer par le    */
                                       /* nom du dictionnaire.               */
    CHAR        name[DICT_LG_NOM+1];   /* Nom du dictionnaire                */
                                       /* Le reste des champs est utilise    */
                                       /* specifiquement par la partie       */
                                       /* dictionnaire statique ou           */
                                       /* dictionnaire dynamique, ils sont   */
                                       /* libres.                            */
    INT         hash_sze           ;   /* Taille table de hashcoding.        */
    USHORT      nbe                ;   /* Nombre total de chaines.           */
    UCHAR       off_data           ;   /* Offset 1er octet de donnee.        */
    CHAR        conv               ;   /* Conversion de la casse des cles.   */
    UCHAR       strid              ;   /* Identifiant d'utilisateur memstr.  */
  END           Dictdyn            ;   /* ---------------------------------- */


/*  Dictionnaire statique - Reconstruction de la chaine d'une cle -----------*/

struct epile                           /*                                    */
  BEGIN                                /*                                    */
    IWORD           p                ; /*                                    */
    IWORD           nb               ; /*                                    */
    UCHAR           st               ; /*                                    */
  END                                ; /*                                    */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Module dictintf.c ........................................................*/

  void  init_dict           (void                                            );
  IWORD create_dict         (INT,CHAR*,CHAR*,INT,INT*,INT*                   );
  IWORD delete_dict         (INT,INT*                                        );
  IWORD get_dict_data       (INT,INT,UCHAR**,INT*,INT*                       );
  IWORD get_dict_data2      (INT,INT,UCHAR**,INT*,INT*                       );
  IWORD get_dict_id         (CHAR*,INT*,INT*                                 );
  IWORD get_dict_name       (INT,CHAR*,INT*                                  );
  IWORD get_dict_list       (CHAR*                                           );
  IWORD get_dict_index      (INT,INT*,INT*                                   );
  IWORD get_dict_file_list  (INT,CHAR*,INT*                                  );
  IWORD get_dict_info       (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,INT*         );
  IWORD get_dict_string     (INT,CHAR*,CHAR*,INT*                            );
  IWORD search_dict_key     (INT,CHAR*,INT,INT*,INT*                         );
  IWORD search_dict_key2    (INT,CHAR*,INT,INT*,INT*                         );
  IWORD search_dict_num     (INT,INT,CHAR*,INT*                              );
  IWORD search_dict_num2    (INT,INT,CHAR**,INT*                             );
  IWORD add_dict_entry      (INT,CHAR*,INT,UCHAR*,INT,INT*,INT*              );
  IWORD add_dict_entry2     (INT,CHAR*,INT,UCHAR*,INT,INT*,INT*              );
  IWORD del_dict_entry      (INT,INT,INT*                                    );
  IWORD del_dict_entry2     (INT,INT,INT*                                    );


/* Module dictsta1.c ........................................................*/

  IWORD cre_dictstat        (INT,INT,CHAR*,INT                               );
  IWORD del_dictstat        (INT                                             );
  IWORD get_dictstat_data   (INT,INT,UCHAR**,INT*                            );
  IWORD sch_dictstat_num    (INT,INT,IWORD,CHAR**                            );
  IWORD sch_dictstat_key    (INT,CHAR*,INT,INT*,INT*                         );
  IWORD get_dictstat_index  (INT,INT*                                        );
  IWORD get_dictstat_file_list(INT,CHAR*                                     );
  IWORD get_dictstat_info   (INT,INT*,INT*,INT*,INT*,UCHAR**,USHORT**        );


/* Module dictsta2.c ........................................................*/

  IWORD search_dict_keys    (INT,CHAR*,INT,INT*,INT,CHAR*,INT,INT*,INT*      );
  IWORD filter_list         (CHAR*,CHAR*,INT,CHAR*,INT,INT*,INT*             );


/* Module dictdyn.c .........................................................*/

  IWORD cre_dictdyn         (INT,INT,CHAR*,INT                               );
  IWORD del_dictdyn         (INT                                             );
  IWORD get_dictdyn_data    (INT,INT,UCHAR**,INT*                            );
  IWORD sch_dictdyn_num     (INT,INT,IWORD,CHAR**                            );
  IWORD sch_dictdyn_key     (INT,CHAR*,INT,INT*,INT*                         );
  IWORD cre_dictdyn_data    (INT,CHAR*,INT,UCHAR*,INT,INT*                   );
  IWORD cpy_dictdyn_data    (INT,INT                                         );
  IWORD del_dictdyn_data    (INT,INT                                         );
  IWORD get_dictdyn_info    (INT,INT*,INT*,INT*,INT*                         );


#endif                                 /* End of DICT_H.                     */
