/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_i2c.h"             */
/*  Constants and definitions for I2C driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 28/02/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  I2C - Parametre de "i2c_io" - I2c_io
            2.2  I2C - Bas niveau I2C - I2c_desc/I2c_drvdef/I2c_softdef
            2.3  I2C - Contexte d'un bus I2C - I2c_ctx
            2.4  EEPROM - Contexte I2C d'une EEPROM - I2c_eepctx
            2.5  EEPROM - Bas niveau EEPROM - Eep_devdesc / Eep_devdef
            2.6  EEPROM - Contexte d'une EEPROM - Eep_ctx
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef DRV_I2C_H                      /* If this file is not already        */
#define DRV_I2C_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  I2C - Parametre de "i2c_io" - I2c_io
   -----------------------------------------
*/

typedef struct                         /* Element de liste de "i2c_io"       */
  BEGIN                                /* ---------------------------------- */
    UCHAR           io               ; /* 0:Ecriture 1:lecture 2:attente     */
    UCHAR           flags            ; /* Flags d'ecriture                   */
    UCHAR          *adr              ; /* Adresse d'emission ou reception    */
    USHORT          lg               ; /* Nb d'octets a lire ou ecrire       */
  END               I2c_io           ; /* ---------------------------------- */


/*
   2.2  I2C - Description d'un bas niveau I2C - I2c_desc/I2c_drvdef/I2c_softdef
   ----------------------------------------------------------------------------
*/

typedef struct                         /* Descr. driver bas niveau I2C       */
  BEGIN                                /* ---------------------------------- */
    void            (*reset)   ()    ; /* Initialision                       */
    void            (*put)     ()    ; /* Ecriture sur le bus                */
    void            (*get)     ()    ; /* Lecture  sur le bus                */
    CHAR           *name             ; /* Nom du device                      */
  END               I2c_desc         ; /* ---------------------------------- */

typedef struct                         /* Parametres du driver I2C           */
  BEGIN                                /* ---------------------------------- */
    INT             wait             ; /* Duree attente entre 2 requetes     */
  END               I2c_drvdef       ; /* ---------------------------------- */

typedef struct                         /* Parametres du driver I2C soft      */
  BEGIN                                /* ---------------------------------- */
    UCHAR           pio_port         ; /* Numero du port PIO des fils I2C    */
    UCHAR           sda_pio          ; /* Numero du PIO du fil SDA           */
    UCHAR           scl_pio          ; /* Numero du PIO du fil SCL           */
    USHORT          hold             ; /* Temps de maintien en us            */
    USHORT          setup            ; /* Temps de chargement en us          */
  END               I2c_softdef      ; /* ---------------------------------- */


/*
   2.3  I2C - Contexte d'un bus I2C - I2c_ctx
   ------------------------------------------
*/


typedef struct                         /* Structure de controle de bcle      */
  BEGIN                                /* ---------------------------------- */
    USHORT          cnt              ; /* Compteur de boucle                 */
    UCHAR           begloop          ; /* Indice I2c_io de debut de bcle     */
  END               I2c_loop         ; /* ---------------------------------- */

#define MLOOP      3

typedef struct                         /* Contexte I2C                       */
  BEGIN                                /* ---------------------------------- */
    void            (*reset)    ()   ; /* Reset du controleur                */
    void            (*put)      ()   ; /* Emettre 1 ou N octets              */
    void            (*get)      ()   ; /* Lire des octets sur le bus         */
                                       /*                                    */
    IWORD           sreset           ; /* Compteur etapes du RESET           */
    UINT            mode             ; /* mode, 0 si inactif                 */
    UINT            speed            ; /* Vitesse du bus I2C (en kHz)        */
    INT             wait             ; /* Duree d'attente entre 2 requetes   */
                                       /*                                    */
    SHORT           code             ; /* Code de l'evenement REQ_...        */
    UCHAR           ad               ; /* Adresse I2C ou -1                  */
    UCHAR           flags            ; /* Flags pour ecritures               */
    USHORT          subad            ; /* Sous-adresse selection de reg      */
    UCHAR           *trame           ; /* Adresse du buffer de lect/ecr      */
    INT             lg               ; /* Nombre d'octets a lire             */
    USHORT          maxtry           ; /* Nombre maximum de tentatives       */
    USHORT          rowsize          ; /* Nombre d'octets d'une rangee EEP   */
    SHORT           raut             ; /* Automate a qui repondre            */
    USHORT          rvl              ; /* Voie logique a qui repondre        */
                                       /*                                    */
    UCHAR           *tracur          ; /* Pointeur de lecture/ecriture.      */
    IWORD           try              ; /* Compteur de tentatives             */
    IWORD           tryadr           ; /* Compteur de tentatives adr I2C     */
    USHORT          block            ; /* Pour signaler fin de read/write    */
    I2c_loop        loop[MLOOP]      ; /* Pile de boucles                    */
    UCHAR           ploop            ; /* Pointeur de la pile "loop[]"       */
    UCHAR           ioidx            ; /* Index dans la liste de I2c_io      */
                                       /*                                    */
    UINT            idto             ; /* Identificateur TO_COM en cours     */
    USHORT          et               ; /* Etat automate "ti2c"               */
    USHORT          ev               ; /* Evenement automate "ti2c"          */
                                       /*                                    */
    UCHAR           msg[130]         ; /* Message selection de registre      */
  END               I2c_ctx          ; /* ---------------------------------- */


/*
   2.4  EEPROM - Contexte I2C d'une EEPROM - I2c_eepctx
   ----------------------------------------------------
*/


typedef struct                         /* Contexte I2c pour une EEPROM       */
  BEGIN                                /* ---------------------------------- */
    UCHAR          *cacheadr         ; /* Adresse du cache de l'EEPROM       */
    INT             rowsize          ; /* Taille en octets d'une rangee      */
  END               I2c_eepctx       ; /* ---------------------------------- */
 

/*
   2.5  EEPROM - Description d'un bas niveau EEPROM - Eep_devdesc / Eep_devdef
   ---------------------------------------------------------------------------
*/


typedef struct                         /* Descr. driver bas niveau EEPROM    */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init)    ()    ; /* Initialision                       */
    IWORD           (*send)    ()    ; /* Ecriture sur le bus                */
    IWORD           (*receive) ()    ; /* Lecture  sur le bus                */
    CHAR           *name             ; /* Nom du driver de bas niveau        */
  END               Eep_devdesc      ; /* ---------------------------------- */

typedef struct                            /* Parametres description EEPROM   */
  BEGIN                                   /* ------------------------------- */
    INT            bus                  ; /* Numero de bus de l'EEPROM       */
    INT            wadr                 ; /* Adresse I2C paire de l'EEPROM   */
    UCHAR          *cacheadr            ; /* Adresse RAM du cache EEPROM     */
    INT            cachesize            ; /* Taille en octets du cache RAM   */
    INT            rowsize              ; /* Taille en octets d'une rangee   */
    CHAR           *name                ; /* Nom du device EEPROM            */
  END              Eep_devdef           ; /* ------------------------------- */


/*
   2.6  EEPROM - Contexte d'une EEPROM - Eep_ctx
   ---------------------------------------------
*/

typedef struct                         /* Entree file d'attente EEP          */
  BEGIN                                /* ---------------------------------- */
    INT             code             ; /* Requete(REQ_WRITE, REQ_READ)       */
    INT             aut              ; /* Automate du demandeur.             */
    INT             vl               ; /* Voie logique du demandeur.         */
    USHORT          lg               ; /* Nombre d'octets a lire ou ecrire   */
    INT             eepadr           ; /* Adresse dans l'EEPROM              */
    INT             iod              ; /* Iod du device EEPROM concerne      */
  END               Eep_job          ; /* ---------------------------------- */

#define MAX_JOBE   10

typedef struct                         /* Contexte EEP de AUT_EEP            */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init)     ()   ; /* Reset du controlleur               */
    IWORD           (*send)     ()   ; /* Emettre 1 ou N octets              */
    IWORD           (*receive)  ()   ; /* Lire des octets sur le bus         */
                                                                             
    Eep_job         liste[MAX_JOBE]  ; /* file d'attente des travaux EEP     */
    UCHAR           lec              ; /* pointeur de lecture de la file     */
    UCHAR           ecr              ; /* pointeur d'ecriture de la file     */
    UCHAR           nbjob            ; /* Nombre d'elements de la file       */
                                                                             
    Eep_devdef     *eep              ; /* Ad structure de parametres         */
  END               Eep_ctx          ; /* ---------------------------------- */
                                                                             
typedef struct                         /* Contexte EEP procs interface       */
  BEGIN                                /* ---------------------------------- */
    UCHAR           wait_init        ; /* 1: on a recu au moins 1 reponse    */
    Eep_devdef     *eep              ; /* Ad structure de parametres         */
    Eep_devdesc    *eeplow           ; /* Ad structure driver bas niveau     */
    UCHAR           txt[STDLINE]     ; /* Buffer procedure d'ecriture        */
  END               Eepproc_ctx      ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD i2c_set             (INT,INT                                         );
  IWORD i2c_send            (INT,INT,UCHAR*,INT,INT,INT,INT                  );
  IWORD i2c_receive         (INT,INT,UCHAR*,INT,INT,INT,INT,INT              );
  void  i2c_write           (INT,INT,UCHAR*,INT                              );
  void  i2c_read            (INT,INT,UCHAR*,INT,INT                          );
  IWORD i2c_io              (INT,I2c_io*,INT,INT,INT                         );


#endif                                 /* End of DRV_I2C_H.                  */
