/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_itf.h"             */
/*  Constants and definitions for low level driver interface.                */
/*                                                                           */
/*  Author : JYA                        Last modification : 09/13/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef DRV_ITF_H                      /* If this file is not already        */
#define DRV_ITF_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


#define MAXMAJ    256                 /* Numero maximum d'un major. <= 256  */
#define COND     (1<<7)



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Interface context for one IOD/MODD */
  BEGIN                                /* ---------------------------------- */
    USHORT     node                  ; /* Remote node                        */
    USHORT     aut                   ; /* Remote automaton number            */
    USHORT     vl                    ; /* Remote logical way number          */
    USHORT     sub_vl_of7            ; /* Sub-channels logical Ways start    */
    USHORT     sub_vl_nb             ; /* Sub-channels logical Ways numbers  */
  END          Itf_ctx               ; /* ---------------------------------- */


#define ACTSZ (IND_DATA-RESP_INIT+1)

typedef struct                         /* Contexte d'un Driver/Proto/Service */
  BEGIN                                /* ---------------------------------- */
    Itf_ctx   *itfctx                ; /* Table des contextes Itf_ctx.       */
    USHORT    *sub_vl                ; /* Sub-channels logical ways table    */
    USHORT     nbiod                 ; /* Nombre d'IOD                       */
    USHORT     nbmodd                ; /* Nombre de MODD                     */
    UCHAR      sessid_cond_act[ACTSZ]; /* b7:condition, b2-b0: action        */
    USHORT     sub_vl_free           ; /* First free entry in "sub_vl[]"     */
  END          Drv_itf_ctx           ; /* ---------------------------------- */



/*
    3    Public variables
    _____________________
*/

extern Drv_itf_ctx drv_itf_ctx[]     ; /* Table des contextes internes.      */
extern const Sys_driver sysevt_driver; /* Forward System drivers events      */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD drv_dps_info        (INT,INT*,INT*,INT*,INT*,INT*, const CHAR**      );
  IWORD drv_dps_init        (INT,INT,INT,UINT*,INT,INT,INT,INT               );
  IWORD drv_dps_end         (INT,INT,INT                                     );
  IWORD drv_dps_alloc       (INT,INT,INT,UINT*,INT,INT                       );
  IWORD drv_dps_free        (INT,INT,INT                                     );
  IWORD drv_dps_open_device (INT,INT,UINT*,INT                               );
  IWORD drv_dps_close_device(INT,INT                                         );
  IWORD drv_dps_alloc_sub   (INT,INT,INT,UINT*,INT,INT                       );
  IWORD drv_dps_free_sub    (INT,INT,INT                                     );
  IWORD drv_dps_connect     (INT,INT,UINT*,INT                               );
  IWORD drv_dps_listen      (INT,INT,UINT*,INT                               );
  IWORD drv_dps_disconnect  (INT,INT,INT                                     );
  IWORD drv_dps_abort       (INT,INT                                         );
  IWORD drv_dps_write       (INT,UCHAR*,INT,INT,INT                          );
  IWORD drv_dps_read        (INT,INT,INT,INT,UCHAR*,INT                      );
  IWORD drv_dps_put         (INT,UCHAR*,UINT,UINT,INT,INT,INT                );
  IWORD drv_dps_get         (INT,UCHAR*,UINT,UINT,INT,INT,INT                );
  IWORD drv_dps_setval      (INT,INT,UINT*,INT,INT                           );
  IWORD drv_dps_getval      (INT,INT,UINT*,INT                               );
  IWORD drv_dps_rpc         (INT,INT,INT,INT,UINT*,INT,INT                   );
  IWORD drv_dps_evt         (INT,S_evt*                                      );
  IWORD drv_sys_evt         (INT,S_evt*                                      );

  IWORD drv_vol_init        (INT,INT*                                        );
  IWORD drv_vol_mount       (INT,INT,INT,INT*,INT*                           );
  IWORD drv_vol_umount      (INT,INT                                         );
  IWORD drv_vol_vinfo       (INT,INT,INT*,INT*                               );
  IWORD drv_vol_credir      (INT,INT,UCHAR*,INT                              );
  IWORD drv_vol_deldir      (INT,INT,UCHAR*,INT                              );
  IWORD drv_vol_crefile     (INT,INT,UCHAR*,UINT,INT                         );
  IWORD drv_vol_delfile     (INT,INT,UCHAR*,INT                              );
  IWORD drv_vol_renfile     (INT,INT,UCHAR*,UCHAR*,INT                       );
  IWORD drv_vol_flist       (INT,INT,UCHAR*,INT,UCHAR**,INT*                 );
  IWORD drv_vol_finfo       (INT,INT,UCHAR*,INT,UINT*                        );
  IWORD drv_vol_fopen       (INT,INT,UCHAR*,INT,INT,INT*,UINT*,INT*          );
  IWORD drv_vol_fclose      (INT,INT                                         );
  IWORD drv_vol_pseek       (INT,UINT,INT,INT*                               );
  IWORD drv_vol_fput        (INT,UCHAR*,INT,INT,INT,INT*                     );
  IWORD drv_vol_gseek       (INT,UINT,INT                                    );
  IWORD drv_vol_fget        (INT,INT,INT,UCHAR**,INT*,INT*                   );
  IWORD drv_vol_evt         (INT,S_evt*                                      );

  IWORD drv_req             (INT,INT,INT,UCHAR*,INT,UINT,UINT,INT            );
  IWORD vol_req             (INT,INT,INT,UCHAR*,INT,INT,UINT                 );
  void  get_itf_ctx         (UINT, Drv_itf_ctx**,Itf_ctx**,INT*              );
  void  drv_mod_sessid      (INT                                             );

  IWORD drv_initdrv         (INT,INT*                                        );
  IWORD chk_maj             (INT,IWORD*                                      );

#endif                                 /* End of DRV_ITF_H.                  */
