/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/engine.h"              */
/*  Constants and definitions for automaton engine.                          */
/*                                                                           */
/*  Author : FA/JYA                     Last modification : 07/31/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ENGINE_H                       /* If this file is not already        */
#define ENGINE_H                       /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Penser a tenir a jour vmk2.c .............................................*/

#define EVT_PASS1          (INT)0x00000000  /* Passe 1                       */
#define EVT_PASS2          (INT)0x00008000  /* Passe 2                       */
#define EVT_MASK_DRVKBD    (INT)0x00000001  /* Mask driver clavier           */
#define EVT_MASK_DRVMSE    (INT)0x00000002  /* Mask driver souris            */
#define EVT_MASK_DRVCHR    (INT)0x00000004  /* Mask driver caractere         */
#define EVT_MASK_TLCM      (INT)0x00000008  /* Mask telecom                  */
#define EVT_MASK_DRVBLK    (INT)0x00000010  /* Mask driver block             */
#define EVT_MASK_AUTKBD    (INT)0x00000020  /* Mask automate clavier         */
#define EVT_MASK_AUTMSE    (INT)0x00000040  /* Mask automate souris          */
#define EVT_MASK_WTA       (INT)0x00000080  /* Mask Warning Trace Alarm      */
#define EVT_MASK_EXPOSE    (INT)0x00000100  /* Mask expose win manager       */
#define EVT_MASK_MODEND    (INT)0x00000200  /* Mask fin de module            */
#define EVT_MASK_TLLOAD    (INT)0x00000400  /* Mask telechargement           */
#define EVT_MASK_AC        (INT)0x00000800  /* Mask controle d'acces         */
#define EVT_MASK_INTERN    (INT)0x00001000  /* Mask autre evts internes      */
#define EVT_MASK_OTHERS    (INT)0x00002000  /* Mask evenement > 256          */


/* Masques sur evts pour tests et transcodage par defaut ....................*/

#define EVT_EQUAL          (INT)0x00000001  /* Valeur exacte                 */
#define EVT_ANY            (INT)0x00000002  /* Tous les codes                */
#define EVT_LIMITS         (INT)0x00000004  /* Bornes                        */
#define EVT_LIST           (INT)0x00000008  /* Elements liste bornee         */
#define EVT_TYPE           (INT)0x00000010  /* Type d'evenements             */
#define RES_EQUAL          (INT)0x00000001  /* Valeur exacte                 */
#define RES_ANY            (INT)0x00000002  /* Tous les reserves             */
#define RES_LIMITS         (INT)0x00000004  /* Bornes                        */
#define RES_LIST           (INT)0x00000008  /* Elements liste bornee         */
#define RES_MASK           (INT)0x00000010  /* Application d'un masque       */
#define PRI_STD            (INT)         0  /* Event_filter: NFA_STD_P3      */
#define PRI_SRV            (INT)         1  /* Event_filter: NFA_STD_PS      */


/* Valeurs speciales du champ "longueur" ....................................*/

#define OLD_SESSID         (INT)        -2  /* Evenement obsolete            */


/*****************************************************************************
 * Codes d'evenements du VMOS : Il faut imperativement modifier le tableau   *
 * "nomcode[]" qui se trouvent dans "vmk/vmk2.c" des que l'on ajoute /       *
 * detruit / modifie un de ces define. L'ajout ne doit pas se faire          *
 * "a l'economie", c'est-a-dire en se contentant de rajouter le code a la    *
 * fin de la liste, en utilisant le premier numero libre. Meme si les numeros*
 * doivent etre re-affectes et que cela prend du temps, il faut le faire.    *
 * Attention aux codes d'evenements telecom, egalement presents dans les     *
 * handlers en assembleur.                                                   *
 *****************************************************************************/


/* Evenements issus de drivers ..............................................*/

#define DRV_KEY_PRESS      (INT)         0  /* CLAVIER Touche enfoncee.      */
#define DRV_KEY_RELEASE    (INT)         1  /* CLAVIER Touche relachee       */
#define DRV_MOUSE_PRESS    (INT)         2  /* SOURIS  Bouton enfoncee       */
#define DRV_MOUSE_RELEASE  (INT)         3  /* SOURIS  Bouton relache        */
#define DRV_MOUSE_MOVE     (INT)         4  /* SOURIS  Deplacement           */
#define EXPOSE_EVENT       (INT)         5  /* WINDOW  Retracer fenetre      */
#define MOVE_CURSOR_END    (INT)         6  /* MOVE_CURSOR fin d'execution   */
#define IND_POWER          (INT)         7  /* Changement d'etat alimentation*/
                                            /* 8 Inutilise                   */
#define SMARTCARD_EVT      (INT)         9  /* Presence/Abscence carte a puce*/
                                            /* Codage du champ longueur :    */
                                            /* 0 = absent                    */
                                            /* 1 = insertion correct         */
                                            /* 2 = insertion incorrect       */


/* Evenements issus de drivers EEPROM/FLASH .................................*/

#define PLR_DONE           (INT)        10  /* PLR ok.                       */
#define PLR_ERROR          (INT)        11  /* PLR impossible -> formatter.  */
#define ERASE_DONE         (INT)        12  /* Erase completion event        */
#define ERASE_ERROR        (INT)        13  /* Erase error event             */
#define PGM_DONE           (INT)        14  /* Program completion event      */
#define PGM_ERROR          (INT)        15  /* Program error event           */
#define BACKUP_WRITE       (INT)        16  /* EEPROM request event          */
#define BACKUP_DONE        (INT)        17  /* EEPROM completion  event      */
#define BACKUP_ERROR       (INT)        18  /* Error       event             */
#define MMAP_READY         (INT)        19  /* Mem mapping ready to be done  */
#define MMAP_DONE          (INT)        20  /* Mem mapping completion        */


/*****************************************************************************
 * L'ordre relatif de AUT_KEY_SHORT a AUT_KEY_RELEASE ne doit PAS etre       *
 * modifie, de meme que l'ordre relatif de AUT_MOUSE_CLICK a AUT_MOUSE_MOVE. *
 * Ces valeurs servent d'intervales dans la procedure  iepanel_preambule     *
 * (kpaneln.c).                                                              *
 * De plus, la procedure "swi_kbd_select" de "kpan_inst.c" necessite que les *
 * deux groupes d'evenements AUT_KEY_*, PANEL_KEY_* soient obligatoirement   *
 * des codes consecutifs (servent d'indices dans des tableaux) et classes    *
 * dans l'ordre suivant :                                                    *
 * AUT_KEY_SHORT   AUT_KEY_LONG   AUT_KEY_XLONG   AUT_KEY_VALID   AUT_KEY_REL*
 * PANEL_KEY_SHORT PANEL_KEY_LONG PANEL_KEY_XLONG PANEL_KEY_VALIF PANEL_KEY_R*
 * Verifier dans ioaut.c ce qui est fait avec haut_rte[256] ...              *
 * Les numeros d'automates y sont redefinis, des tas d'evts des offsets dans *
 * des structures C.                                                         *
 *****************************************************************************/


#define PANEL_INIT         (INT)        21  /* PANEL  Initialisation panel   */
#define PANEL_END          (INT)        22  /* PANEL  Termination panel      */
#define PANEL_REFRESH      (INT)        23  /* PANEL  Raffraichissement panel*/
#define PO_MOUSE_CLICK     (INT)        24  /* WIDGET Click souris           */
#define PO_MOUSE_DCLICK    (INT)        25  /* WIDGET Double-click souris    */
#define PO_MOUSE_CONTINUE  (INT)        26  /* WIDGET Click continue         */
#define PO_MOUSE_RELEASE   (INT)        27  /* WIDGET Relachement souris     */
#define PO_MOUSE_MOVE      (INT)        28  /* WIDGET Mouvement souris       */
#define PO_MOUSE_ENTER     (INT)        29  /* WIDGET Entree focus souris    */
#define PO_MOUSE_LEAVE     (INT)        30  /* WIDGET Sortie focus souris    */
#define PO_KEY_SHORT       (INT)        31  /* WIDGET Touche courte          */
#define PO_KEY_RELEASE     (INT)        32  /* WIDGET Touche relachee        */
#define PO_KEY_ENTER       (INT)        33  /* WIDGET Entree focus clavier   */
#define PO_KEY_LEAVE       (INT)        34  /* WIDGET Sortie focus souris    */
#define PO_DEF_ENTER       (INT)        35  /* WIDGET Entree focus defaut    */
#define PO_DEF_LEAVE       (INT)        36  /* WIDGET Sortie focus defaut    */
#define PO_GRP_ENTER       (INT)        37  /* WIDGET Selection groupe       */
#define PO_GRP_LEAVE       (INT)        38  /* WIDGET Deselection groupe     */
#define PO_REFRESH         (INT)        39  /* WIDGET Raffraichissement      */
#define PO_KEY_LONG        (INT)        40  /* WIDGET Touche longue          */
#define PO_KEY_XLONG       (INT)        41  /* WIDGET Touche tres longue     */
#define PO_KEY_VALID       (INT)        42  /* WIDGET Touche validee         */
                                            /* 43     Inutilise              */
#define AUT_KEY_SHORT      (INT)        44  /* KBD    Appui court            */
#define AUT_KEY_LONG       (INT)        45  /* KBD    Appui long             */
#define AUT_KEY_XLONG      (INT)        46  /* KBD    Appui tres long        */
#define AUT_KEY_VALID      (INT)        47  /* KBD    Appui valide           */
#define AUT_KEY_RELEASE    (INT)        48  /* KBD    Relachement            */
#define AUT_MOUSE_CLICK    (INT)        49  /* MOUSE  Simple click           */
#define AUT_MOUSE_DCLICK   (INT)        50  /* MOUSE  Double click           */
#define AUT_MOUSE_CONTINUE (INT)        51  /* MOUSE  Pression continue      */
#define AUT_MOUSE_RELEASE  (INT)        52  /* MOUSE  Relachement            */
#define AUT_MOUSE_MOVE     (INT)        53  /* MOUSE  Deplacement souris     */
                                            /* 54     Inutilise              */
                                            /* 55     Inutilise              */
                                            /* 56     Inutilise              */
#define PANEL_KEY_SHORT    (INT)        57  /* PANEL  Touche appui court     */
#define PANEL_KEY_LONG     (INT)        58  /* PANEL  Touche appui long      */
#define PANEL_KEY_XLONG    (INT)        59  /* PANEL  Touche appui tres long */
#define PANEL_KEY_VALID    (INT)        60  /* PANEL  Touche validee         */
#define PANEL_KEY_RELEASE  (INT)        61  /* PANEL  Touche relachee        */
#define PANEL_MOUSE_CLICK  (INT)        62  /* PANEL  Simple click           */
#define PANEL_MOUSE_DCLICK (INT)        63  /* PANEL  Double click           */
#define PANEL_MOUSE_CONT   (INT)        64  /* PANEL  Appui continu          */
#define PANEL_MOUSE_REL    (INT)        65  /* PANEL  Relachement souris     */
#define PANEL_MOUSE_MOVE   (INT)        66  /* PANEL  Deplacement souris     */
#define DEF_KEY            (INT)        67  /* PANDEF Clavier non utilise    */
#define DEF_MOUSE          (INT)        68  /* PANDEF Souris  non utilisee   */
                                            /* 69     Inutilise              */
#define PANEL_SHOW         (INT)        70  /* PANEL  va etre cache          */
#define PANEL_HID          (INT)        71  /* PANEL  va etre montre         */
#define PANEL_RAISE        (INT)        72  /* PANEL  va etre place au dessus*/
#define PANEL_LOWER        (INT)        73  /* PANEL  va etre place en dessou*/
                                            /* 74     Inutilise              */
                                            /* 75     Inutilise              */
#define SET_LANGUAGE_END   (INT)        76  /* SET_LG fin de MAJ language    */
#define PANEL_DRAW         (INT)        77  /* Evenement trace/refresh widget*/
#define PANEL_CLEAR        (INT)        78  /* Evenement trace/refresh widget*/
#define PANEL_KEY_ENTER    (INT)        79  /* Evenement trace/refresh widget*/
#define PANEL_KEY_LEAVE    (INT)        80  /* Evenement trace/refresh widget*/
#define PANEL_MOUSE_ENTER  (INT)        81  /* Evenement trace/refresh widget*/
#define PANEL_MOUSE_LEAVE  (INT)        82  /* Evenement trace/refresh widget*/
#define PO_VALK_SHORT      (INT)        83  /*                               */
#define PO_VALK_LONG       (INT)        84  /*                               */
#define PO_VALK_XLONG      (INT)        85  /*                               */
#define PO_VALK_RELEASE    (INT)        86  /*                               */
#define PO_VALK_ENTER      (INT)        87  /*                               */
#define PO_VALK_LEAVE      (INT)        88  /*                               */
                                            /* 89      : Inutilise           */
#define DRV_TOUCH_PRESS    (INT)        90  /* Evenement appui tactile       */
#define DRV_TOUCH_RELEASE  (INT)        91  /* Evenement relacher tactile    */
#define DRV_TOUCH_MOVE     (INT)        92  /* Evenement deplacement tactile */
                                            /* 93 a 99 : Inutilises          */

/* Evenements de l'interpreteur MHEG5 .......................................*/

#define M5_ASY_QUIT        (INT)       100  /* Fin IHM MHEG-5                */
#define M5_CI_DATA         (INT)       121  /* Donnees recues module CI/CI+  */


/* Evenements entre app residente et apps interactives ......................*/

#define ITV_INIT           (INT)       133  /* Evt d'initialisation          */
#define ITV_CMD            (INT)       134  /* Commande a une app interactive*/
#define ITV_RES            (INT)       135  /* Evt ress dispos/non dispos    */
#define ITV_SVC            (INT)       136  /* Transmission mess d'un service*/
#define ITV_END            (INT)       137  /* Evt de terminaison            */
#define ITV_INFO           (INT)       138  /* Informations app interactive  */
#define ITV_REQ            (INT)       139  /* Requete d'une app interactive */
#define ITV_RESTORE        (INT)       140  /* Retablir reglages multimedia  */
                                            /* 141 Inutilise                 */
#define ITV_ENO            (INT)       142  /* Evenement EnOcean PJB Demo    */
                                            /* 143 Inutilise                 */
                                            /* 144 Inutilise                 */
                                            /* 145 Inutilise                 */
                                            /* 146 Inutilise                 */
#define PANEL_REQ          (INT)       147  /* Requete pour les panels.      */
#define PANEL_REQ_DONE     (INT)       148  /* Requete terminee avec succes. */
#define PANEL_REQ_ERROR    (INT)       149  /* Requete terminee avec erreur. */
                                            /* 150 a 166 Inutilises          */
#define PO_VALK_VALID      (INT)       167  /* Validation touche validation. */
#define PANEL_RESIZE       (INT)       168  /* Fond du widget redimensionne. */
#define PANEL_SET_POS      (INT)       169  /* Fond du widget repositionne.  */
                                            /* 170 a 185 Inutilises          */
#define PANEL_REQ_DRAW     (INT)       186  /* PANEL_DRAW envoyes.           */
                                            /* 187 Inutilise                 */
                                            /* 188 Inutilise                 */
                                            /* 189 Inutilise                 */


/* Evenements de l'API fichier / drivers blocs ..............................*/
/* Attention l'ordre des codes ci-dessous est utilise par la table "volaut(]"*/
/* de dro_itf.c                                                              */

#define REQ_MOUNT          (INT)       240  /* FILE Demande montage partition*/
#define REQ_UMOUNT         (INT)       241  /* FILE Demande demontage part   */
#define REQ_VINFO          (INT)       242  /* FILE Demande info volume      */
#define REQ_CREDIR         (INT)       243  /* FILE Demande creation dir     */
#define REQ_DELDIR         (INT)       244  /* FILE Demande destruction dir  */
#define REQ_CREFILE        (INT)       245  /* FILE Demande creation fichier */
#define REQ_DELFILE        (INT)       246  /* FILE Demande destruction fic  */
#define REQ_RENFILE        (INT)       247  /* FILE Demande destruction fic  */
#define REQ_FLIST          (INT)       248  /* FILE Demande liste de fichiers*/
#define REQ_FINFO          (INT)       249  /* FILE Demande informations fic */
#define REQ_FOPEN          (INT)       250  /* FILE Demande ouverture fichier*/
#define REQ_FCLOSE         (INT)       251  /* FILE Demande fermeture fichier*/
#define REQ_PSEEK          (INT)       252  /* FILE Demande pos ptr ecriture */
#define REQ_FPUT           (INT)       253  /* FILE Demande ecriture bloc    */
#define REQ_GSEEK          (INT)       254  /* FILE Demande pos ptr lecture  */
#define REQ_FGET           (INT)       255  /* FILE Demande lecture bloc     */
#define REQ_FMAP           (INT)       256  /* TEMPORAIRE                    */
                                            /* 257 Inutilise                 */
                                            /* 258 Inutilise                 */
                                            /* 259 Inutilise                 */
#define RESP_MOUNT         (INT)       260  /* FILE Reponse montage partition*/
#define RESP_UMOUNT        (INT)       261  /* FILE Reponse demontage part   */
#define RESP_VINFO         (INT)       262  /* FILE Reponse info volume      */
#define RESP_CREDIR        (INT)       263  /* FILE Reponse creation dir     */
#define RESP_DELDIR        (INT)       264  /* FILE Reponse destruction dir  */
#define RESP_CREFILE       (INT)       265  /* FILE Reponse creation fichier */
#define RESP_DELFILE       (INT)       266  /* FILE Reponse destruction fic  */
#define RESP_RENFILE       (INT)       267  /* FILE Reponse destruction fic  */
#define RESP_FLIST         (INT)       268  /* FILE Reponse liste de fichiers*/
#define RESP_FINFO         (INT)       269  /* FILE Reponse informations fic */
#define RESP_FOPEN         (INT)       270  /* FILE Reponse ouverture fichier*/
#define RESP_FCLOSE        (INT)       271  /* FILE Reponse fermeture fichier*/
#define RESP_PSEEK         (INT)       272  /* FILE Reponse pos ptr ecriture */
#define RESP_FPUT          (INT)       273  /* FILE Reponse ecriture bloc    */
#define RESP_GSEEK         (INT)       274  /* FILE Reponse pos ptr lecture  */
#define RESP_FGET          (INT)       275  /* FILE Reponse lecture bloc     */
#define RESP_DF            (INT)       276  /* FILE Reponse API              */
#define RESP_FMAP          (INT)       277  /* TEMPORAIRE                    */
                                            /* 278 Inutilise                 */
                                            /* 279 Inutilise                 */


/* Evenements de l'afficheur des sous-titres DVB ............................*/

#define DVBSTL_STOPPED     (INT)       380  /* Decodeur afficheur arrete     */
#define DVBSTL_ERR         (INT)       381  /* Overflow donnees entrantes.   */
#define DVBSTL_DISPLAY     (INT)       382  /* Demande de flush applicatif.  */
#define DVBSTL_CLEAR       (INT)       383  /* Demande d'effacement          */
                                            /* 384 Inutilise                 */
                                            /* 385 Inutilise                 */
                                            /* 386 Inutilise                 */
                                            /* 387 Inutilise                 */
                                            /* 388 Inutilise                 */
                                            /* 389 Inutilise                 */


/* Evenements de l'afficheur teletexte ......................................*/

#define TTXT_DISPLAY       (INT)       390  /* Page    a afficher            */
#define TTXT_HEADER        (INT)       391  /* En-tete a afficher            */
                                            /* 392 Inutilise                 */
                                            /* 393 Inutilise                 */
                                            /* 394 Inutilise                 */
                                            /* 395 Inutilise                 */
                                            /* 396 Inutilise                 */
                                            /* 397 Inutilise                 */
                                            /* 398 Inutilise                 */
                                            /* 399 Inutilise                 */


/* Codes d'evenements du VMIO ...............................................*/

#define EVTS_I2C_END       (INT)       400  /* Fin E/S I2C                   */
#define EVTS_IO_END        (INT)       401  /* Fin E/S disque ou autre       */
#define IT_RX              (INT)       402  /* IT de reception               */
#define IT_TX              (INT)       403  /* IT de fin d'emission          */
#define IT_INS             (INT)       404  /* IT insertion de carte         */
#define IT_EXT             (INT)       405  /* IT extraction de carte        */
#define IT_SYN0            (INT)       406  /* IT de synchro HDCP            */
#define IT_SYN127          (INT)       407  /* IT de synchro HDCP            */
#define EVTS_ASY_END       (INT)       408  /* Fin E/S driver ASY            */
                                            /* 409 Inutilise                 */


/* Evenements de requetes ...................................................*/

#define REQ_CMD            (INT)       410  /* Demande de demarrage          */
#define REQ_RESET          (INT)       411  /* Demande reset de carte/device */
#define REQ_PTS            (INT)       412  /* Demande envoi de PTS          */
#define REQ_WRITES         (INT)       413  /* Demande ecriture I2C          */
#define REQ_SUBREAD        (INT)       414  /* Demande select puis lect I2C  */
#define REQ_READES         (INT)       415  /* Demande lecture I2C           */
#define REQ_WRITEEP        (INT)       416  /* Demande ecriture EEPROM I2C   */
#define REQ_READEEP        (INT)       417  /* Demande lecture  EEPROM I2C   */
#define REQ_ENDS           (INT)       418  /* Demande terminaison systeme   */
#define REQ_CHANGE         (INT)       419  /* Demande changement de RCU     */


/* Evenements de time-out ...................................................*/

#define TO_POW             (INT)       420  /* Time-out POWER                */
#define TO_LED             (INT)       421  /* Time-out LED                  */
#define TO_DIS             (INT)       422  /* Time-out DISPLAY              */
#define TO_FRE             (INT)       423  /* Time-out FRONT-END            */
#define TO_SWI             (INT)       424  /* Time-out SWITCH               */
#define TO_SMC             (INT)       425  /* Time-out SMC (carte a puce)   */
#define TO_COM             (INT)       426  /* Time-out I2C                  */
#define TO_STEP            (INT)       427  /* Time-out etape suivante I2C/  */
#define TO_RCU             (INT)       428  /* Time-out RCU non reception    */
#define TO_SKIP            (INT)       429  /* Time-out RCU fin de surdite   */
 

/* Evenements conditions ....................................................*/

#define C_WAIT             (INT)       430  /* Condition d'attente           */
#define C_ERROR            (INT)       431  /* Condition d'erreur            */
#define C_RUN              (INT)       432  /* Condition de lancement        */
#define C_HALT             (INT)       433  /* Condition d'arret             */
#define C_MORE             (INT)       434  /* Condition de continuation     */
#define C_OPOSITIF         (INT)       435  /* Condition RESP_OPEN positif   */
#define C_ONEGATIF         (INT)       436  /* Condition RESP_OPEN negatif   */
#define C_OPENED           (INT)       437  /* Ouverture reussie             */
#define C_OPEN             (INT)       437  /* Ouverture reussie sans err    */
#define C_NOTOPENED        (INT)       438  /* Ouverture echouee             */
#define C_NOTOPEN          (INT)       438  /* Ouverture echouee sans err    */
#define C_ERRIO            (INT)       439  /* Erreur d'entree/sortie        */
#define C_CLOSED           (INT)       440  /* Driver ferme                  */
#define C_NOMORE           (INT)       441  /* Negation de C_MORE            */
#define C_INACTIVE         (INT)       442  /* Desactive                     */
#define C_IDLE             (INT)       443  /* Au repos                      */
#define C_END              (INT)       444  /* Fin de sequence               */
#define C_READ             (INT)       445  /* Lecture a faire               */
#define C_WRITE            (INT)       446  /* Ecriture a faire              */
#define C_SYN              (INT)       447  /* IT_SYN en attente             */
                                            /* 448 Inutilise                 */
                                            /* 449 Inutilise                 */


/* Evenements de l'automate PVR .............................................*/

#define REL_SHIFT          (INT)       450  /* Nouvelles donnees timeshift   */
#define DMX_PVR_STA        (INT)       451  /* Notif chgt etat de dmx_pvr    */
                                            /* 452 Inutilise                 */
                                            /* 453 Inutilise                 */
                                            /* 454 Inutilise                 */
                                            /* 455 Inutilise                 */
                                            /* 456 Inutilise                 */
                                            /* 457 Inutilise                 */
                                            /* 458 Inutilise                 */
                                            /* 459 Inutilise                 */


/* Evenements de l'automate DSK .............................................*/

#define TO_DSK1            (INT)       460  /* Time-out I/O disque           */
#define TO_DSK2            (INT)       461  /* Time-out standby disque       */
#define BUS_GOT            (INT)       462  /* Recuperation bus apres attente*/
                                            /* 463 Inutilise                 */
                                            /* 464 Inutilise                 */
                                            /* 465 Inutilise                 */
                                            /* 466 Inutilise                 */
                                            /* 467 Inutilise                 */
                                            /* 468 Inutilise                 */
                                            /* 469 Inutilise                 */


/* Evenements de l'automate FAN .............................................*/

#define TO_FAN             (INT)       470  /* Time-out FAN                  */
#define C_SATUR            (INT)       471  /* Condition PWM sature          */
                                            /* 472 Inutilise                 */
                                            /* 473 Inutilise                 */
                                            /* 474 Inutilise                 */
                                            /* 475 Inutilise                 */
                                            /* 476 Inutilise                 */
                                            /* 477 Inutilise                 */
                                            /* 478 Inutilise                 */
                                            /* 479 Inutilise                 */


/* Evenements de l'automate USB .............................................*/

#define USB_CSC            (INT)       480  /*                               */
#define USB_TORESET        (INT)       481  /*                               */
#define USB_IOEND          (INT)       482  /*                               */


/* Evenements de l'automate SCSI ............................................*/

#define TO_SCSI            (INT)       483  /*                               */
                                            /* 484 Inutilise                 */
                                            /* 485 Inutilise                 */
                                            /* 486 Inutilise                 */
                                            /* 487 Inutilise                 */
                                            /* 488 Inutilise                 */
                                            /* 489 Inutilise                 */

#define PANEL_ID           (INT)         0  /* Id ressource panel            */
#define DICT_ID            (INT)         1  /* Id ressource dictionnaire     */
#define CHANNEL_ID         (INT)         2  /* Id ressource telecom          */
#define EXTERN_ID          (INT)         3  /* Id ressource applicative      */


/* Evenements de l'automate UMS .............................................*/

#define UMS_INQUIRY        (INT)       490  /*                               */
#define UMS_RQSENSE        (INT)       491  /*                               */
#define UMS_TUREADY        (INT)       492  /*                               */
#define UMS_C_NODATA       (INT)       493  /*                               */
#define UMS_READ10         (INT)       494  /*                               */
#define UMS_RCAPACITY      (INT)       495  /*                               */
#define UMS_WRITE10        (INT)       496  /*                               */
                                            /* 497 Inutilise                 */
                                            /* 498 Inutilise                 */
                                            /* 499 Inutilise                 */


/* Fin de la tranche reservee 20 A 189 ......................................*/

/* Evenements de l'API Telecom / drivers caracteres .........................*/
/* Attention l'ordre des codes ci-dessous est utilise par la table "dpsaut(]"*/
/* de drv_itf.c                                                              */

#define REQ_INIT           (INT)       190  /* TLCM Demande initialisation   */
#define REQ_END            (INT)       191  /* TLCM Demande terminaison      */
#define REQ_ALLOC          (INT)       192  /* TLCM Demande allocation       */
#define REQ_FREE           (INT)       193  /* TLCM Demande liberation       */
#define REQ_OPEN           (INT)       194  /* TLCM Demande ouverture        */
#define REQ_CLOSE          (INT)       195  /* TLCM Demande fermeture        */
#define REQ_ALLOC_SUB      (INT)       196  /* TLCM Demande alloc sous-cnl   */
#define REQ_FREE_SUB       (INT)       197  /* TLCM Demande lib. sous-cnl    */
#define REQ_CONNECT        (INT)       198  /* TLCM Demande connexion        */
#define REQ_LISTEN         (INT)       199  /* TLCM Attente connexion        */
#define REQ_DISCONNECT     (INT)       200  /* TLCM Demande deconnexion      */
#define REQ_ABORT          (INT)       201  /* TLCM Abandon (de)connexion    */
#define REQ_WRITE          (INT)       202  /* TLCM Demande emission         */
#define REQ_READ           (INT)       203  /* TLCM Demande lecture          */
#define REQ_PUT            (INT)       204  /* TLCM Demande ecriture bloc    */
#define REQ_GET            (INT)       205  /* TLCM Demande lecture bloc     */
#define REQ_SETVAL         (INT)       206  /* TLCM Demande parametrage      */
#define REQ_GETVAL         (INT)       207  /* TLCM Demande lecture param    */
#define REQ_RPC            (INT)       208  /* TLCM Demande lecture param    */
                                            /* 209 Inutilise                 */
#define RESP_INIT          (INT)       210  /* TLCM Reponse a REQ_INIT       */
#define RESP_END           (INT)       211  /* TLCM Reponse a REQ_END        */
#define RESP_ALLOC         (INT)       212  /* TLCM Reponse a REQ_ALLOC      */
#define RESP_FREE          (INT)       213  /* TLCM Reponse a REQ_FREE       */
#define RESP_OPEN          (INT)       214  /* TLCM Reponse a REQ_OPEN       */
#define RESP_CLOSE         (INT)       215  /* TLCM Reponse a REQ_CLOSE      */
#define RESP_ALLOC_SUB     (INT)       216  /* TLCM Reponse a REQ_ALLOC_SUB  */
#define RESP_FREE_SUB      (INT)       217  /* TLCM Reponse a REQ_FREE_SUB   */
#define RESP_CONNECT       (INT)       218  /* TLCM Reponse a REQ_CONNECT    */
#define RESP_LISTEN        (INT)       219  /* TLCM Reponse a REQ_LISTEN     */
#define RESP_DISCONNECT    (INT)       220  /* TLCM Reponse a REQ_DISCONNECT */
#define RESP_ABORT         (INT)       221  /* TLCM Reponse a REQ_ABORT      */
#define RESP_WRITE         (INT)       222  /* TLCM Reponse a REQ_WRITE      */
#define RESP_READ          (INT)       223  /* TLCM Reponse a REQ_READ       */
#define RESP_PUT           (INT)       224  /* TLCM Reponse a REQ_PUT        */
#define RESP_GET           (INT)       225  /* TLCM Reponse a REQ_GET        */
#define RESP_SETVAL        (INT)       226  /* TLCM Reponse a REQ_SETVAL     */
#define RESP_GETVAL        (INT)       227  /* TLCM Reponse a REQ_GETVAL     */
#define RESP_RPC           (INT)       228  /* TLCM Reponse a REQ_RPC        */
#define IND_DISCONNECT     (INT)       229  /* TLCM Indication de deconnexion*/
#define IND_REPORT         (INT)       230  /* TLCM Indication modif param   */
#define IND_PNP            (INT)       231  /* TLCM Indication branchement   */
#define IND_DATA           (INT)       232  /* TLCM Indication de donnees    */
                                            /* 233 Inutilise                 */
                                            /* 234 Inutilise                 */
                                            /* 235 Inutilise                 */
                                            /* 236 Inutilise                 */
                                            /* 237 Inutilise                 */
                                            /* 238 Inutilise                 */
                                            /* 239 Inutilise                 */
#define REQ2RESP (RESP_INIT-REQ_INIT)


#define REQ_ALLOC_BUF      (INT)       280  /* MEMBUF Demande allocation     */
#define REQ_FREE_BUF       (INT)       281  /* MEMBUF Demande liberation     */
#define REQ_ALLOC_LINK     (INT)       282  /* MEMBUF Demande allocation     */
#define REQ_FREE_LINK      (INT)       283  /* MEMBUF Demande liberation     */
#define REQ_ALLOC_ALI      (INT)       284  /* MEMBUF Demande allocation     */
#define REQ_REALLOC        (INT)       285  /* MEMBUF Demande re-allocation  */
#define REQ_POST_BUF       (INT)       286  /* MEMBUF Demande post_buf       */
#define REQ_TAKE_MUT       (INT)       287  /* MUTEX Operation request       */
#define REQ_RELEASE_MUT    (INT)       288  /* MUTEX Operation request       */

#define RESP_ALLOC_BUF     (INT)       300  /* MEMBUF Reponse a REQ_ALLOC    */
#define RESP_FREE_BUF      (INT)       301  /* MEMBUF Reponse a REQ_FREE     */
#define RESP_ALLOC_LINK    (INT)       302  /* MEMBUF Reponse a REQ_ALLOC    */
#define RESP_FREE_LINK     (INT)       303  /* MEMBUF Reponse a REQ_FREE     */
#define RESP_ALLOC_ALI     (INT)       304  /* MEMBUF Reponse a REQ_ALLOC_ALI*/
#define RESP_REALLOC       (INT)       305  /* MEMBUF Reponse a REQ_REALLOC  */
#define RESP_POST_BUF      (INT)       306  /* MEMBUF Reponse a REQ_POST_BUF */
#define RESP_TAKE_MUT      (INT)       307  /* MUTEX take response           */
#define RESP_RELEASE_MUT   (INT)       308  /* MUTEX Operation response      */


/* Evenements issus des thread / scripts / telechargements ..................*/

#define THREAD_START       (INT)       320  /* Debut de thread.              */
#define THREAD_INTR        (INT)       321  /* Interruption de thread.       */
#define THREAD_END         (INT)       322  /* Fin de thread.                */
#define THREAD_ERROR       (INT)       323  /* Erreur dans un thread.        */
#define REQ_SCRIPT         (INT)       324  /* SCRIPT  : demande d'execution */
#define RESP_SCRIPT        (INT)       325  /* SCRIPT  : fin d'execution     */
#define SET_DOWNLOADED     (INT)       326  /* Fichier ressources telecharge */
#define SET_REF_SOLVED     (INT)       327  /* References resolues           */
#define SET_INITIALIZED    (INT)       328  /* Ressources initialisees       */
#define SET_LOADED         (INT)       329  /* Ressources chargee/utilisable */
#define SET_LOAD_ABORTED   (INT)       330  /* Chargement ressources annule  */
#define SET_UNLOADED       (INT)       331  /* Ressources dechargees         */
#define DOWNLOAD_CONF      (INT)       332  /* DNLOAD Telechargement termine */
#define MODULE_END         (INT)       333  /* MODULE Indication fin module  */
#define MODULE_UNLOAD_END  (INT)       334  /* MODULE fin de dechargement    */
                                            /* 335 Inutilise                 */


/* Evenements DSMCC .........................................................*/

#define DSMCC_SRG_DONE     (INT)       336  /* Fin telechargement du SRG     */
#define DSMCC_OBJ_DONE     (INT)       337  /* Fin telechargement d'un objet */


/* Evenements d'emission de messages d'alarme ...............................*/

#define WRITE_ALARM        (INT)       338  /* Emission d'une alarme.        */
                                            /* 339 Inutilise                 */

/* Evenements des Controles d'acces .........................................*/

#define CA_RESP_INIT       (INT)       340  /* Demarrage controle d'acces.   */
#define CA_RESP_END        (INT)       341  /* Arret controle d'acces.       */
#define CA_TABLE_SURVEY    (INT)       342  /* Demande surveillance table DVB*/
#define CA_ACTIVE_EVT      (INT)       343  /* Etat fonctionnement CA modifie*/
#define CA_CARD_EVT        (INT)       344  /* Modif du statut CA d'une carte*/
#define CA_RESP_SETVAL     (INT)       345  /* Reponse modif de parametres.  */
#define CA_RESP_GETVAL     (INT)       346  /* Reponse de lect de parametres.*/
#define CA_STREAM_STATUS   (INT)       347  /* Accesssibilite stream elem.   */
#define CA_REQ_PIN         (INT)       348  /* Requete de code PIN par le CA */
#define CA_RESP_PIN        (INT)       349  /* Saisie code PIN utilisateur   */
#define CA_REQ_USER        (INT)       350  /* Requete CA vers utilisateur   */
#define CA_RESP_USER       (INT)       351  /* Reponse utilisateur.          */
#define CA_IND_REPORT      (INT)       352  /* Indication CA.                */
#define CA_POPUP           (INT)       353  /* Gestion de Pop-Up CA.         */
#define CA_SYSTEM_ERR      (INT)       354  /* Erreur systeme du CA.         */
#define CA_CHANNEL_LOCK    (INT)       355  /* Verrouillage CA d'un programme*/
                                            /* 356 Inutilise                 */
                                            /* 357 Inutilise                 */
                                            /* 358 Inutilise                 */
                                            /* 359 Inutilise                 */


/* Evenements en provenance des service .....................................*/

#define REQ_CONF           (INT)       360  /* Confirmation de reglage       */
#define REQ_SEND           (INT)       361  /* Information                   */
                                            /* 362 Inutilise                 */
                                            /* 363 Inutilise                 */
                                            /* 364 Inutilise                 */
                                            /* 365 Inutilise                 */
                                            /* 366 Inutilise                 */
                                            /* 367 Inutilise                 */
                                            /* 368 Inutilise                 */
                                            /* 369 Inutilise                 */


/* Evenements du chargeur de librairie dynamique ............................*/

#define DL_OPEN_DONE       (INT)       370  /* Fin de chargement.            */
#define DL_CLOSE_DONE      (INT)       371  /* Fin de dechargement           */
                                            /* 352 a 359 Inutilises          */

/*
    2    Structures and typedefs
    ____________________________
*/


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of ENGINE_H.                   */
