/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/executif.h"            */
/*  Constants and definitions for interpretors.                              */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 05/09/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  Structures de descriptions des scripts charges en RAM
            2.2  Structures de descriptions des threads en execution
            2.3  Structures de descriptions des proceduresde librairies
         3  Public variable
            3.1  Defined in "hypspace.h"
            3.2  Defined in "executif.h"
         4  Prototypes of public procedures


*/


#ifndef EXECUTIF_H                     /* If this file is not already        */
#define EXECUTIF_H                     /* included, include it.              */


/*
    1    Defines
    ____________
*/

/* Valeurs possibles de STOP_CALC ...........................................*/

#define CPI_STOP_END          1        /* Fin du script                      */
#define CPI_STOP_HLT          2        /* Arret sur instruction HALT         */
#define CPI_STOP_INST         3        /* Arret sur erreur instruction       */
#define CPI_STOP_PROC         4        /* Arret sur erreur procedure         */
#define CPI_STOP_LOAD         5        /* Arret sur attente chargement       */
#define CPI_STOP_CONT         6        /* Arret mais demande relance         */
#define CPI_STOP_WAIT         7        /* Arret sur interruption             */
#define CPI_STOP_NOP          8        /* Arret sur pt d'arret pas a pas     */
#define CPI_STOP_BRK          9        /* Arret sur pt d'arret               */
#define CPI_STOP_BRV         10        /* Arret sur modif variable           */
#define CPI_STOP_SCHED       11        /* Arret sur scheduler                */
#define CPI_STOP_VM          12        /* Arret sur stop_vm()                */


/* Etats possibles d'un contexte d'execution ................................*/

#define SCR_STATE_EMPTY       0        /* Contexte libre                     */
#define SCR_STATE_ALLOC       1        /* Contexte alloue                    */
#define SCR_STATE_LOAD        2        /* Contexte en attente chargement     */
#define SCR_STATE_RUN         4        /* Contexte en cours d'execution      */
#define SCR_STATE_WAIT        8        /* Contexte interrompu                */
#define SCR_STATE_DBG        16        /* Contexte avec debugger branche     */
#define SCR_STATE_WAITDBG    32        /* Contexte en attente debugger       */
#define SCR_STATE_MASK     0xF0        /* Masque pour ALLOC/LOAD/RUN/WAIT    */
 

/*  Types des methodes interfacees ..........................................*/

#define ITF_EMPTY             0        /* Interface le vide                  */
#define ITF_METHOD            1        /* Interface une methode d'un script  */
#define ITF_PROC              2        /* Interface une procedure            */
#define ITF_ERROR             3        /* Interface methode non implementee  */


/* Etats possibles d'un contexte memoire ....................................*/

#define CLA_STATE_NOINIT      0        /* Contexte libre                     */
#define CLA_STATE_GOINIT      1        /* Contexte alloue                    */
#define CLA_STATE_INITOK      2        /* Contexte en attente chargement     */
#define CLA_STATE_ERROR       4        /* Contexte en cours d'execution      */
 

/* Types des valeurs utilisables par la CPI .................................*/


/*****************************************************************************
 * L'ordre facilite la distinction entre les valeurs numeriques et les       * 
 * valeurs non numeriques et les operations de conversion par promotion      * 
 * de type.                                                                  * 
 *****************************************************************************/


#define CPI_INT                0       /* Valeur entiere signee (4 octets)   */
#define CPI_LNG                1       /* Valeur entiere signee (8 octets)   */
#define CPI_FLT                2       /* Valeur reelle         (4 octets)   */
#define CPI_DBL                3       /* Valeur reelle         (8 octets)   */
#define CPI_OBJ                4       /* Reference a un objet  (4 octets)   */
#define CPI_STR                5       /* Chaine de caracteres               */
#define CPI_ADR                6       /* Adresse buffer telecom             */
#define CPI_MEM                7       /* Zone memoire                       */
#define CPI_WAP                8       /* Valeur WAP                         */
#define CPI_INV                9       /* Valeur invalide                    */
#define CPI_PC                10       /* Program counter                    */
#define HND_BYT               11       /* Handle variable genre G_BYT        */
#define HND_SHT               12       /* Handle variable genre G_SHT        */
#define HND_INT               13       /* Handle variable genre G_INT        */
#define HND_LNG               14       /* Handle variable genre G_LNG        */
#define HND_FLT               15       /* Handle variable genre G_FLT        */
#define HND_DBL               16       /* Handle variable genre G_DBL        */
#define HND_OBJ               17       /* Handle variable genre G_OBJ        */
#define HND_CHR               18       /* Handle variable genre G_CHR        */
#define HND_STR               19       /* Handle variable genre G_STR        */
#define HND_ADR               20       /* Handle variable genre G_ADR        */
#define HND_MEM               21       /* Handle variable genre G_MEM        */
#define HND_WAP               22       /* Handle variable genre G_WAP        */
#define NB_CPI_TYPE           23       /* Nombre de types CPI                */


/* Manipulation des constantes ..............................................*/

#define GET_CTE_TYP(s)        (((s) AND (USHORT)0xF000)>>12) 
#define GET_CTE_OFF(s)        ( (s) AND (USHORT)0x0FFF)  
#define SET_CTE_TYP(s,type)   (s) = ((s) AND 0x0FFF) OR ((USHORT)(type)<<12);
#define SET_CTE_OFF(s,off7)   (s) = ((s) AND 0xF000) OR (USHORT)(off7)      ;

#define CTE_BYTE      (USHORT)0x00     /* Constante de type CPI_BYT          */
#define CTE_SHORT     (USHORT)0x01     /* Constante de type CPI_SHT          */
#define CTE_INT       (USHORT)0x02     /* Constante de type CPI_INT          */
#define CTE_FLOAT     (USHORT)0x03     /* Constante de type CPI_FLT          */
#define CTE_STR       (USHORT)0x04     /* Constante de type CPI_STR          */
#define CTE_CLASS     (USHORT)0x05     /* Constante de type CLASS_REF        */
#define CTE_ARRAY     (USHORT)0x06     /* Constante de type ARRAY            */
#define CTE_METHOD    (USHORT)0x07     /* Constante de type METHODE_REF      */
#define CTE_ITF       (USHORT)0x08     /* Constante de type METHOD_ITF_REF   */
#define CTE_FIELD     (USHORT)0x09     /* Constante de type FIELD_REF        */
#define CTE_FIELD_INS (USHORT)0x0A     /* Constante de type FIELD_INS_REF    */
#define CTE_ERR       (USHORT)0x0B     /* Constante de type ERREUR.          */
#define CTE_LONG      (USHORT)0x0C     /* Constante de type CPI_LNG          */
#define CTE_DOUBLE    (USHORT)0x0D     /* Constante de type CPI_DBL          */
#define CTE_SPE1      (USHORT)0x0E     /* Constante de type specifique       */
#define CTE_SPE2      (USHORT)0x0F     /* Constante de type specifique       */


/* Valeurs des constantes de type CTE_ERR (erreurs non critiques capturees   */
/* par le chargeur/verifieur.                                                */
 
#define C_CLASS_NOT_FOUND    -1        /* NoClassDefFoundError.              */
#define C_CLASS_ILL_ACC      -2        /* IllegalAccessError.                */
#define C_FIELD_NOT_FOUND    -3        /* NoSuchFieldError.                  */
#define C_FIELD_ILL_ACC      -4        /* IllegalAccessError.                */
#define C_ITF_NOT_FOUND      -5        /* NoSuchMethodError.                 */
#define C_ITF_ILL_ACC        -6        /* IllegalAccessError.                */
#define C_ITF_INCOMP         -7        /* IncompatibleClassChangeError.      */
#define C_MET_NOT_FOUND      -8        /* NoSuchFieldError.                  */
#define C_MET_ILL_ACC        -9        /* IllegalAccessError.                */
#define C_MET_INCOMP        -10        /* IncompatibleClassChangeError.      */
#define C_MET_ABSTRACT      -11        /* AbstractMethodError.               */


/* Manipulation des handles de variable .....................................*/

#define HND_TYP_GLB  (USHORT)1         /* Marque un handle variable globale  */
#define HND_TYP_LOC  (USHORT)0         /* Marque un handle variable locale   */

#define HND_GET_TYP(handle)      (((handle)->var AND (USHORT)0x8000) >> 15)  
#define HND_SET_TYP(handle,i)    ((handle)->var = (handle->var) OR ((i) << 15) 
#define NV_SET_TYP(nv,i)         ((nv) OR ((i) << 15))
#define NV_GET_TYP(nv)           (  (USHORT)((nv) AND 0x8000) >> 15)  
#define NV_CLR_TYP(nv)           ((nv) AND (USHORT)0x7FFF)  


/* Types des arguments des fonctions de librairie ...........................*/

                                       /* Type CPI             Type C        */
                                       /* ---------------------------------- */
#define ARGVAL_BYT            1        /* INT,LNG              CHAR          */
#define ARGVAL_SHT            2        /* INT,LNG              SHORT         */
#define ARGVAL_INT            3        /* INT,LNG              INT           */
#define ARGVAL_LNG            4        /* INT,LNG              LONG          */
#define ARGVAL_FLT            5        /* FLT,DBL              FLOAT         */
#define ARGVAL_DBL            6        /* FLT,DBL              DOUBLE        */
#define ARGVAL_OBJ            7        /* OBJ                  INT           */
#define ARGVAL_STR            8        /* STR,HND_STR          CHAR    []    */
#define ARGVAL_ADR            9        /* ADR                  UCHAR   *     */
#define ARGVAL_MEM           10        /* MEM,ADR,HND_MEM      UCHAR   []    */
#define ARGVAL_CHR           11        /* INT,LNG              USHORT        */
#define ARGVAL_BLN           12        /* WAP                  CHAR          */

                                       /* Type CPI (HND_)      Type C        */
                                       /* ---------------------------------- */
#define ARGADR_BYT           13        /* BYT,SHT,INT,LNG      CHAR    *     */
#define ARGADR_SHT           14        /* SHT,INT,LNG          SHORT   *     */
#define ARGADR_INT           15        /* INT,LNG              INT     *     */
#define ARGADR_LNG           16        /* LNG                  LONG    *     */
#define ARGADR_FLT           17        /* FLT,DBL              FLOAT   *     */
#define ARGADR_DBL           18        /* DBL                  DOUBLE  *     */
#define ARGADR_OBJ           19        /* OBJ                  INT     *     */
#define ARGADR_STR           20        /* STR                  CHAR    []    */
#define ARGADR_ADR           21        /* ADR                  UCHAR   **    */
#define ARGADR_MEM           22        /* MEM,ADR              UCHAR   []    */
#define ARGADR_CHR           23        /* CHR                  SHORT   *     */
#define ARGADR_BLN           24        /* BLN                  CHAR    *     */

                                       /* Type CPI (HND_)      Type C        */
                                       /* ---------------------------------- */
#define ARGTAB_BYT           25        /* BYT                  CHAR    []    */
#define ARGTAB_SHT           26        /* SHT                  SHORT   []    */
#define ARGTAB_INT           27        /* INT                  INT     []    */
#define ARGTAB_LNG           28        /* LNG                  LONG    []    */
#define ARGTAB_FLT           29        /* FLT                  FLOAT   []    */
#define ARGTAB_DBL           30        /* DBL                  DOUBLE  []    */
#define ARGTAB_OBJ           31        /* OBJ                  INT     []    */
#define ARGTAB_STR           32        /* STR                  CHAR   *[]    */
#define ARGTAB_ADR           33        /* ADR                  UCHAR  *[]    */
#define ARGTAB_MEM           34        /* ADR,MEM              UCHAR  *[]    */
#define ARGTAB_CHR           35        /* CHR                  SHORT   []    */
#define ARGTAB_BLN           36        /* BLN                  CHAR    []    */


/* Types des valeurs de retour des proceduresde librairie ..................*/

#define LIB_RET_NO            1        /* Description sans valeur de retour  */
#define LIB_RET_YES           2        /* Description avec valeur de retour  */

                                       /* Type CPI             Type C        */
                                       /* ---------------------------------- */
#define RETVAL_VOID           0        /* -                    void          */
#define RETVAL_BYT            1        /* BYT                  CHAR          */
#define RETVAL_SHT            2        /* SHT                  SHORT         */
#define RETVAL_INT            3        /* INT                  INT           */
#define RETVAL_LNG            4        /* LNG                  LONG          */
#define RETVAL_FLT            5        /* FLT                  FLOAT         */
#define RETVAL_DBL            6        /* DBL                  DOUBLE        */
#define RETVAL_OBJ            7        /* OBJ                  INT           */
#define RETVAL_CHR            8        /* CHR                  SHORT         */
#define RETVAL_STR            9        /* STR                  INT           */
#define RETVAL_BLN           10        /* BLN                  CHAR          */


/* Types possibles d'erreur (champ ret_typ de Scr_cpi) ......................*/

#define CPI_ERR_CODE          1        /* Code d'erreur simple               */
#define CPI_ERR_CID           2        /* Id classe d'erreur fourni          */
#define CPI_ERR_OBJ           3        /* Id objet description erreur fourni */


/* Macros et definitions pour le micro_interpreteur .........................*/

#define IE(par0,par1,par2,opcode)   ((par0 << 20) | (par1 << 16) |    \
                                     (par2 << 12) | (opcode    ) ) 

#define IE_GET_PARAM0(inst)   (((inst) >> 20) AND 0x00F)
#define IE_GET_PARAM1(inst)   (((inst) >> 16) AND 0x00F)
#define IE_GET_PARAM2(inst)   (((inst) >> 12) AND 0x00F)
#define IE_GET_OPCODE(inst)   ( (inst)        AND 0xFFF)

#define IE_SYNC            0x01        /* Methode courante synchronisee      */
#define IE_EXCP            0x02        /* Exception generee                  */
#define IE_WEVT            0x04        /* Attend un evenement                */
#define IE_INTR            0x08        /* Demande interruption               */


/* Constantes pour le debugger ..............................................*/

#define METH_LGNAME          20        /* Lg d'un nom de methode             */
#define DBS_LGNAM            32        /* Lg d'un nom script.methode         */
#define DBS_LGMOD            16        /* Lg d'un nom de module              */
#define DBS_LGINT            16        /* Lg d'un nom d'interpreteur         */
#define DBS_LGCTX             4        /* Lg d'un numero de contexte         */
#define DBS_LGLIN             5        /* Lg d'un numero de ligne            */
#define DBS_IT                0        /* Script interrompu par un pt d'arret*/
#define DBS_FREE              1        /* Script efface avant sa fin normale */
#define DBS_RUN               0        /* Mode normal                        */
#define DBS_BRK               1        /* Arret sur les points d'arret       */
#define DBS_NOP               2        /* Arret a chaque instruction NOP     */
#define DBS_JSC               3        /* Arret apres le script              */
#define DBS_ASC               4        /* Arret apres sortie du script       */
#define DBS_STP               5        /* Arret a chaque instruction         */


/* Macros et structures utiles ..............................................*/

/* Ces macros alignent une longueur sur 2, 4 ou 8 octets :                   */

#define PAD2(lg)   (((UINT)lg % ZS) ? ((((UINT)lg + ZS)>>1)<<1) : (UINT)lg)
#define PAD4(lg)   (((UINT)lg % ZI) ? ((((UINT)lg + ZI)>>2)<<2) : (UINT)lg)
#define PAD8(lg)   (((UINT)lg % ZD) ? ((((UINT)lg + ZD)>>3)<<3) : (UINT)lg)


/* Ces macros evitent un appel a memcpy pour recopier un SHORT, un INT,      */
/* un FLOAT, un DOUBLE ou une adresse.                                       */

#define NGET_ZS(src,dst) {CHAR *p=(CHAR*)&dst;                      \
                         *(p++) = *(src++); *(p++) = *(src++);      }

#define NGET_ZI(src,dst) {CHAR *p=(CHAR*)&dst;                      \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p)   = *(src++);      }
 
#define NGET_ZF(src,dst) {CHAR *p=(CHAR*)&dst;                      \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p)   = *(src++);      }
 
#define NGET_ZD(src,dst) {CHAR *p=(CHAR*)&dst;                      \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p)   = *(src++);      }
 
#define NGET_LGAD(src,dst)  {CHAR *p=(CHAR*)&dst;                   \
                         *(p++) = *(src++); *(p++) = *(src++);      \
                         *(p++) = *(src++); *(p)   = *(src++);      }


/* Cas d'erreurs ............................................................*/
 
#define Cpierr1(t)   {  stop_calc = t ; goto error ;                }
#define Cpierr2(t)   {  stop_calc = t ;                             }
#define Cpierr3(t)   {  retour = t ; goto error ;                   }
#define Cpierr4(t,s) {  cpi_stack(s) ; stop_calc = t ; goto error ; }
#define Cpierr5(s)   {  cpi_stack(s) ; goto error ;                 }
 

/* Alignement ...............................................................*/

#define ALIGN_2(P)   (P)+=(unsigned long)(P) AND 1L;
#define ALIGN_4(P)   if((unsigned long)(P) AND 3L)                    \
                         (P)+=4-((unsigned long)(P) AND 3L);
#define ALIGN_8(P)   if((unsigned long)(P) AND 7L)                    \
                         (P)+=8-((unsigned long)(P) AND 7L);
#define ALIGN_N(P,N) {if((unsigned long)(P) AND (unsigned long)(N)-1) \
                       (P)+=(N)-(unsigned long)(P) AND (unsigned long)(N)-1;}

#define IALIGN_2(P)   (INT)((P)+((unsigned long)(P) AND 1L)) 

#define IALIGN_4(P)   (INT)(((unsigned long)(P) AND 3L)                      ?\
                         (unsigned long)(P)+4L-((unsigned long)(P) AND 3L)   :\
                         (P)                                                 )
#define IALIGN_8(P)   (INT)(((unsigned long)(P) AND 7L)                      ?\
                         (unsigned long)(P)+8L-((unsigned long)(P) AND 7L)   :\
                         (P)                                                 )
#define IALIGN_N(P,N) (INT)(((unsigned long)(P) AND (unsigned long)(N)-1L)   ?\
                             (unsigned long)(P)  +  (unsigned long)(N) -      \
                            ((unsigned long)(P) AND (unsigned long)(N)-1L)   :\
                         (P)                                                 )
                          

/* Codes d'erreurs globales des micro-instructions ..........................*/


/* Erreurs generees explicitement, sous forme specifique --------------------*/

#define ERR_EXPL_0               0     /* Inutilise.                         */
#define ERR_EXPL_1               1     /* Inutilise.                         */
#define ERR_EXPL_2               2     /* Inutilise.                         */
#define ERR_EXPL_3               3     /* Inutilise.                         */


/* Erreurs de nature generale ou inconnues ----------------------------------*/

#define ERR_ERROR                4     /* Error.                             */
#define ERR_EXC                  5     /* Exception.                         */
#define ERR_ERR_2                6     /* Inutilise.                         */
#define ERR_ERR_3                7     /* Inutilise.                         */


/* Erreurs concernant une instruction opcode --------------------------------*/

#define ERR_INST_UNKNOWN         8     /* Instruction inconnue.              */
#define ERR_INST_PARA            9     /* Probleme avec le para de l'inst.   */
#define ERR_INST_FORMAT         10     /* Probleme avec le format du para.   */
#define ERR_INST_3              11     /* Inutilise.                         */


/* Erreurs concernant la pile des methodes ----------------------------------*/

#define ERR_METHSTACK_OVERFLOW  12     /* Plus de place dans la pile.        */
#define ERR_METHSTACK_UNDERFLOW 13     /* Pas de methode a depiler.          */
#define ERR_METHSTACK_2         14     /* Inutilise.                         */
#define ERR_METHSTACK_3         15     /* Inutilise.                         */


/* Erreurs lors de l'appel d'une methode ------------------------------------*/

#define ERR_METH_UNKNOWN        16     /* Methode introuvable ou inconnue.   */
#define ERR_METH_ARGUMENTS      17     /* Arguments invalides.               */
#define ERR_METH_ABSTRACT       18     /* Methode est abstraite.             */
#define ERR_METH_3              19     /* Inutilise.                         */


/* Erreurs concernant un tableau --------------------------------------------*/

#define ERR_ARR_IDX             20     /* Index dans tableau invalide.       */
#define ERR_ARR_NEGATIV_SIZE    21     /* Taille tableau negative.           */
#define ERR_ARR_TYP             22     /* Probleme avec le type du tableau.  */
#define ERR_ARR_3               23     /* Inutilise.                         */


/* Erreurs concernant une chaine --------------------------------------------*/

#define ERR_STR_IDX             24     /* Index dans chaine invalide.        */
#define ERR_STR_NEGATIV_SIZE    25     /* Taille de chaine negative.         */
#define ERR_STR_FORMAT          26     /* Format de la chaine invalide.      */
#define ERR_STR_ENCODING        27     /* Codage inconnu.                    */


/* Erreurs concernant la pile des operandes ---------------------------------*/

#define ERR_OPSTACK_VAL         28     /* Valeur sur la pile invalide.       */
#define ERR_OPSTACK_TYP         29     /* Type sur la pile invalide.         */
#define ERR_OPSTACK_OVERFLOW    30     /* Pile pleine.                       */
#define ERR_OPSTACK_UNDERFLOW   31     /* Pile vide.                         */


/* Erreurs concernant l'acces aux constantes d'un script --------------------*/

#define ERR_CONSTANT_IDX        32     /* Index dans le cp invalide.         */
#define ERR_CONSTANT_TYP        33     /* Type constante invalide.           */
#define ERR_CONSTANT_VAL        34     /* Inutilise.                         */
#define ERR_CONSTANT_3          35     /* Inutilise.                         */


/* Erreurs concernant la creation d'un objet --------------------------------*/

#define ERR_OCREAT_CLASS_INV    36     /* Class inconnue.                    */
#define ERR_OCREAT_CLASS_ACC    37     /* Classe pas instantiable.           */
#define ERR_OCREAT_2            38     /* Inutilise.                         */
#define ERR_OCREAT_3            39     /* Inutilise.                         */


/* Erreurs concernant l'acces a un objet ------------------------------------*/

#define ERR_OBJ_INVALID         40     /* Identifiant objet nul ou invalide. */
#define ERR_OBJ_FIELD_UNKNOWN   41     /* Champ n'existe pas.                */
#define ERR_OBJ_CLASS_CAST      42     /* Affectation objet impossible.      */
#define ERR_OBJ_3               43     /* Inutilise.                         */


/* Erreurs concernant des operations arithmetiques --------------------------*/

#define ERR_DIVIDE_BY_ZERO      44     /* Division par zero.                 */
#define ERR_LOGARITHM           45     /* Valeur negative ou nulle pour log. */
#define ERR_ARITHM_2            46     /* Inutilise.                         */
#define ERR_ARITHM_3            47     /* Inutilise.                         */


/* Erreurs concernant les informations sur un ensemble ----------------------*/

#define ERR_SET_INVALID         48     /* Ensemble invalide.                 */
#define ERR_SET_UNKNOWN         49     /* Ensemble inconnu.                  */
#define ERR_SET_INIT            50     /* Erreur lors de l'init de l'ensemle */
#define ERR_SET_INCOMPAT        51     /* Inutilise.                         */


/* Erreurs concernant l'acces a des donnees ---------------------------------*/

#define ERR_ACC_ILLEGAL         52     /* Acces a la donnee impossible.      */
#define ERR_ACC_SECURITY        53     /* Acces a la donnee interdit.        */
#define ERR_ACC_2               54     /* Inutilise.                         */
#define ERR_ACC_3               55     /* Inutilise.                         */


/* Erreurs de memoire -------------------------------------------------------*/

#define ERR_OUTOF_MIDMEM        56     /* Memoire MID_MEM plein.             */
#define ERR_OUTOF_LOWMEM        57     /* Memoire LOW_MEM plein.             */
#define ERR_OUTOF_HIGHMEM       58     /* Memoire HIGH_MEM plein.            */
#define ERR_OUTOF_MEM           59     /* Plus de memoire.                   */


/* Erreurs concernant la synchronisation ------------------------------------*/

#define ERR_SYNCHRO_LOCK        60     /* Impossible de synchroniser.        */
#define ERR_SYNCHRO_UNLOCK      61     /* Impossible de desynchroniser       */
#define ERR_SYNCHRO_2           62     /* Inutilise.                         */
#define ERR_SYNCHRO_3           63     /* Inutilise.                         */


/* Erreurs concernant les variablelocales ---------------------------------*/

#define ERR_LOCVAR_IDX          64     /* No de l'a var locale invalide.     */
#define ERR_LOCVAR_TYP          65     /* type de la var locale invalide.    */
#define ERR_LOCVAR_CLA          66     /* Classe locale inconnue.            */
#define ERR_LOCVAR_3            67     /* Inutilise.                         */


/* Operandes, variable procedures-----------------------------------------*/

#define VAL_STR                  1     /*                                    */
#define VAL_ES                   2     /*                                    */
#define LIB_ARG_MAX             12     /* Nombre maximum d'arguments         */
 

/*
    2    Structures and typedefs
    ____________________________
*/


/*
    2.1  Structures de descriptions des scripts charges en RAM
    ----------------------------------------------------------
*/


typedef struct                         /* Structure du bloc CTX_MEMOIRE      */
  BEGIN                                /* ---------------------------------- */
    USHORT          mod              ; /* Identifiant reference/module       */
    USHORT          bits             ; /* Proprietes du script               */
    USHORT          class            ; /* Numero de la classe attachee       */
    USHORT          nb_met           ; /* Nbre de methodes du script         */
    USHORT          nb_cte           ; /* Nbre de constantes                 */
    USHORT          age              ; /* Age derniere utilisation du script */
    UCHAR           language         ; /* Language des methodes du script    */
    UCHAR           index            ; /* Type transcodage evenements        */
  END               Hscr_ctx         ; /* ---------------------------------- */

typedef struct                         /* Parametre de retour d'une methode- */
  BEGIN                                /* ---------------------------------- */
    USHORT          type             ; /* Numero classe instanciee           */
    UCHAR           type_obj         ; /* Reference/Primitif                 */
    UCHAR           dim              ; /* Nombre de dimensions               */
    UCHAR           type_arg         ; /* Type de parametre (ARGxxx_xxx)     */
  END               Hscr_arg         ; /* ---------------------------------- */

typedef struct                         /* Description minimum d'une methode  */
  BEGIN                                /* ---------------------------------- */
    Hscr_arg        ret              ; /* Description du parametre de retour */
    INT             lg_loc           ; /* Taille de la zone variable locale. */
    UINT            entree           ; /* Point d'entree dans le bloc CODE   */
    USHORT          bits             ; /* Champ de bits, decrit la methode   */
    USHORT          lg               ; /* Longueur du byte-code              */
    USHORT          lg_spe           ; /* Taille zone specifique frame       */
    USHORT          nb_ope           ; /* Nombre d'operandes                 */
    USHORT          nb_loc           ; /* Nombre de variablelocales        */
    USHORT          nb_exc           ; /* Nombre d'exceptions gerees         */
    UCHAR           nb_arg           ; /* Nombre d'arguments                 */
  END               Hscr_method      ; /* ---------------------------------- */

typedef struct                         /* Reference a un element d'ensemble  */
  BEGIN                                /* ---------------------------------- */
    USHORT          set              ; /* Numero ensemble                    */
    USHORT          elt              ; /* Numero element   / Code d'erreur   */
  END               Hscr_ref         ; /* ---------------------------------- */

typedef struct                         /* Description minimum d'une methode  */
  BEGIN                                /* ---------------------------------- */
    CHAR            type             ; /* Type de l'interface                */
    CHAR            loader           ; /* Chargeur du script                 */
    USHORT          entree           ; /* Point d'entree du code             */
    INT             script           ; /* Nom du script/de la librairie      */
    INT             method           ; /* Nom de methode/de procedure        */
  END               Hscr_lib         ; /* ---------------------------------- */


/*
    2.2  Structures de descriptions des threads en execution
    --------------------------------------------------------
*/

typedef struct                         /* Handle variable locale/globale     */
  BEGIN                                /* ---------------------------------- */
    INT             idx              ; /* Index dans le tableau              */
    USHORT          cla              ; /* Identifiant classe/frame           */
    USHORT          var              ; /* Identifiant variable               */
  END               Hexe_handle      ; /* ---------------------------------- */

typedef union                          /* Valeurs d'un operande              */
  BEGIN                                /* ---------------------------------- */
    CHAR            c                ; /* Valeur donnee CHAR                 */
    UCHAR           uc               ; /* Valeur donnee UCHAR                */
    SHORT           s                ; /* Valeur donnee SHORT                */
    USHORT          us               ; /* Valeur donnee USHORT               */
    INT             i                ; /* Valeur donnee INT                  */
    UINT            ui               ; /* Valeur donnee UINT                 */
    LONG            l                ; /* Valeur donnee LONG                 */
    UCHAR           *ad              ; /* Valeur donnee ADRESS               */
    Hexe_handle     hnd              ; /* Handle d'une variable loc/glb      */
    CHAR            tc[8]            ; /* Lec/Ecr     1 octet  par 1 octet   */
    INT             ti[2]            ; /* Lec/Ecr     4 octets par 4 octets  */
    SHORT           ts[4]            ; /* Lec/Ecr     2 octets par 2 octets  */
    UCHAR           tuc[8]           ; /* Lec/Ecr     1 octet  par 1 octet   */
    UINT            tui[2]           ; /* Lec/Ecr     4 octets par 4 octets  */
    USHORT          tus[4]           ; /* Lec/Ecr     2 octets par 2 octets  */
  END               Hexe_valeur      ; /* ---------------------------------- */

typedef struct                         /* Parametres de fonctionnement CPI   */
  BEGIN                                /* ---------------------------------- */
    CHAR            byte_order       ; /* MACHINE, MOTOROLA, INTEL           */
    CHAR            strings          ; /* RAW, L1, L2, C, MPEG               */
    CHAR            trigo            ; /* Radians, degres ou grades          */
  END               Hexe_param       ; /*                                    */

typedef struct                         /* Donnees courantes de la CPI.       */
  BEGIN                                /* ---------------------------------- */
    USHORT          pc               ; /* Program counter                    */
    USHORT          line             ; /* Numero ligne de script             */
    UCHAR           ret_arg          ; /* Nombre d'argumetns methode lib.    */
    CHAR            pop              ; /* Pointeur pile des operandes        */
    UCHAR           fzero            ; /* Flag de dernier resultat nul       */
    UCHAR           fnega            ; /* Flag de dernier resultat negatif   */
    UCHAR           ie_flag          ; /* Flag     micro-instruction         */
    const INT      *ie_pc            ; /* Pointeur micro-instruction         */
  END               Hexe_curcpi      ; /*                                    */

typedef struct                         /* Entete du bloc FRAME               */
  BEGIN                                /* ---------------------------------- */
    Hexe_curcpi     curcpi           ; /* Ensemble des donnees courantes.    */
    USHORT          off_val_ope      ; /* Offset valeurs OPERANDES.          */
    USHORT          off_val_loc      ; /* Offset valeurs        VL.          */
    USHORT          off_typ_ope      ; /* Offset type    OPERANDES.          */
    USHORT          off_typ_loc      ; /* Offset type           VL.          */
    USHORT          off_spe          ; /* Offset partie specifique.          */
  END               Hexe_frame       ; /* ---------------------------------- */

typedef struct                         /* Frame de la methode 0.             */
  BEGIN                                /* ---------------------------------- */
    Hexe_frame      fra              ; /* La frame.                          */
    Hexe_valeur     val[5]           ; /* Pile des operandes : valeurs.      */
    UCHAR           typ[5]           ; /* Pile des operandes : types.        */
  END               Hexe_fra0        ; /* ---------------------------------- */

typedef struct                         /* Description d'une methode a exec.  */
  BEGIN                                /* ---------------------------------- */
    Hexe_frame     *adr_frame        ; /* Adresse de la frame de la methode. */
    USHORT          set_global       ; /* Le set global definissant la met.  */
    USHORT          set_script       ; /* Le set script definissant la met.  */
    USHORT          method           ; /* Le numero de methode dans le script*/
    UCHAR           language         ; /* Identifiant du langage/ chargeur.  */
  END               Hexe_call        ; /* ---------------------------------- */
   
typedef struct                         /* Bloc CPI                           */
  BEGIN                                /* -----------------------------------*/

    /*************************************************************************
     * Definition de la 1ere methode a empiler dans ce thread et de ses      *
     * arguments.                                                            *
     *************************************************************************/

    INT             method           ; /* Nom de la premiere methode         */
    INT             para             ; /* Parametres 1ere methode            */

    /*************************************************************************
     * Parametres de fonctionnement des proceduresde librairie et des        *
     * instructions qui manipulent des angles, des zones memoire et des      *
     * chaine.                                                               *
     *************************************************************************/

    Hexe_param      param            ; /* Parametre de fonctionnement        */

    /*************************************************************************
     * Ces champs sont utilises pour conserver la definition d'une methode   *
     * appelee apres qu'elle ait ete depilee                                 *
     *************************************************************************/

    Hexe_valeur     ret_val          ; /* Valeur de retour                   */
    UCHAR           ret_typ          ; /* Type de la valeur de retour        */
    UCHAR           ret_arg          ; /* Nombre d'arguments utilises        */

    /*************************************************************************
     * Ces champs sont utilises pour conserver la definition d'une erreur    *
     * rencontree par une micro-instruction.                                 *
     *************************************************************************/

    INT             err_cod          ; /* Code de l'erreur                   */
    INT             err_msg          ; /* Id du message d'erreur.            */
    INT             err_stk          ; /* Id valeur chaine description pile  */

    /*************************************************************************
     * Les champs suivants contiennent des informations globales             *
     * independantes de la methode empilee                                   *
     *************************************************************************/

    INT             timer            ; /* Id du timer d'attente d'evenements */
    UCHAR           events           ; /* Nbre de routes vers ce contexte.   */
    UCHAR           dbg              ; /* Indicateur mode pas a pas          */
    CHAR            call_auto        ; /* Identifiant  de l'automate appel.  */
    UCHAR           call_vl          ; /* Voie logique de l'automate appel.  */

    /* Champs pour le retour d'appel d'une procedure de librairie            */

    UCHAR           fct_nb_arg       ; /* Nb d'argument fonction librairie   */
    USHORT          fct_id           ; /* Id fonction librairie.             */
    IWORD           (*callback)()    ; /* Procedure MAJ variablede retour    */
    INT             par              ; /* Parametre procedure de call-back   */
    INT             did              ; /* Id dictionnaire des procedures     */
    IWORD           (*pvr)()         ; /* Procedure valeur variable          */
    UCHAR           *ttr             ; /* Table de transcodage des types     */
    INT             proc_ret         ; /* Valeur de retour ou pas pour proc. */
  END               Hexe_cpi         ; /* ---------------------------------- */

typedef struct                         /* Structure du bloc CTX_EXECUTION    */
  BEGIN                                /* ---------------------------------- */
    Hexe_call      *calls            ; /* Pile des methodes appellees.       */
    Hexe_cpi       *cpi              ; /* Adresse du bloc CPI                */
    UCHAR           etat             ; /* Etat de ce contexte                */
    CHAR            nb_met           ; /* Nombres des methodes empilees.     */
    INT             rte_load         ; /* Routes creees pour chargement      */
    UCHAR          *pantalk          ; /* Addresse de la table allouee pour  */
                                       /* stocker les associations           */
                                       /* objet<->buffer du thread.          */
  END               Hexe_ctx         ; /* ---------------------------------- */


/*
    2.3  Structures de descriptions des proceduresde librairies
    ------------------------------------------------------------
*/


typedef union                          /* Type generique pour les passages   */
  BEGIN                                /* de parametres au procedures       */
                                       /* ---------------------------------- */
    CHAR            c                ; /* Argument = Val d'entier  s 1 o.    */
    SHORT           s                ; /* Argument = Val d'entier  s 2 o.    */
    INT             i                ; /* Argument = Val d'entier  s 4 o.    */
    LONG            l                ; /* Argument = Val d'entier  s 8 o.    */
    UCHAR           uc               ; /* Argument = Val d'entier ns 1 o.    */
    USHORT          us               ; /* Argument = Val d'entier ns 2 o.    */
    UINT            ui               ; /* Argument = Val d'entier ns 4 o.    */
                                       /*                                    */
    CHAR           *pc               ; /* Argument = Ptr d'entier  s 1 o.    */
    SHORT          *ps               ; /* Argument = Ptr d'entier  s 2 o.    */
    INT            *pi               ; /* Argument = Ptr d'entier  s 4 o.    */
    LONG           *pl               ; /* Argument = Ptr d'entier  s 8 o.    */
    UCHAR          *puc              ; /* Argument = Ptr d'entier ns 1 o.    */
    USHORT         *pus              ; /* Argument = Ptr d'entier ns 2 o.    */
    UINT           *pui              ; /* Argument = Ptr d'entier ns 4 o.    */
                                       /*                                    */
    void           *ad               ; /* Adresse generique                  */
                                       /*                                    */
  END               Lib_arg          ; /* ---------------------------------- */

typedef union                          /* Valeur associee a un argument de   */
  BEGIN                                /* procedure de librairie.            */
    CHAR            c                ; /* Valeur donnee CHAR                 */
    SHORT           s                ; /* Valeur donnee SHORT                */
    INT             i                ; /* Valeur donnee INT                  */
    LONG            l                ; /* Valeur donnee LONG                 */
    UCHAR           uc               ; /* Valeur donnee UCHAR                */
    USHORT          us               ; /* Valeur donnee USHORT               */
    UINT            ui               ; /* Valeur donnee UINT                 */
    UCHAR          *ad               ; /* Valeur donnee UCHAR*               */
                                       /*                                    */
    struct                             /* Structure utilisee pour les        */
      BEGIN                            /* variabletableau et de lg variable*/
        INT         nelt             ; /* Nb d'elements du tableau           */
        INT         size             ; /* Taille maximum de la variable      */
     END            dvar             ; /*                                    */
                                       /*                                    */
  END               Lib_val          ; /* ---------------------------------- */


/* Structures pour le debugger ..............................................*/


/*****************************************************************************
 * Sert a memoriser les evenements deroutees lorsque le debugger a           *
 * interrompu un Thread.                                                     *
 *****************************************************************************/


typedef struct                         /* Contexte du debugger.              */
  BEGIN                                /* ---------------------------------- */
    SHORT           numauto          ; /* Automate interrompu                */
    USHORT          voielog          ; /* Voie logique interrompue           */
    SHORT           firstevt         ; /* Indice 1er evenement dans la pile  */
    USHORT          rid              ; /* Numero de l'interpreteur           */
    UCHAR           nbevt            ; /* Nb d'evenements empiles            */
  END               Dbs_ctx          ; /* ---------------------------------- */

typedef struct                         /* Point d'arret.                     */
  BEGIN                                /* ---------------------------------- */
    INT             mod              ; /* Identifiant du nom du module       */
    INT             scr              ; /* Identifiant du nom du script       */
    INT             met              ; /* Identifiant du nom de la methode   */
    SHORT           li               ; /* Numero de ligne dans le script     */
    USHORT          rid              ; /* Numero de l'interpreteur           */

    /*************************************************************************
     * Ces champs sont en dernier car ils sont les seuls differences entre un*
     * point d'arret deja declare et un point d'arret qui va etre declare,   *
     * cela permet de faire une comparaison memoire.                         *
     *************************************************************************/

    SHORT           offset           ; /* Offset dans le code                */
    USHORT          nctx             ; /* Numero ctx memoire                 */
  END               Dbs_bri          ; /* ---------------------------------- */

typedef struct                         /* Interface de l'interpreteur.       */
  BEGIN                                /* ---------------------------------- */
    IWORD         (*add_bri)    ()   ; /* Ajouter un point d'arret script    */
    IWORD         (*clr_bri)    ()   ; /* Effacer un point d'arret script    */
    IWORD         (*go)         ()   ; /* Relancer l'interpretation          */
    IWORD         (*add_brv)    ()   ; /* Ajouter un point d'arret variable  */
    IWORD         (*clr_brv)    ()   ; /* Effacer un point d'arret variable  */
    IWORD         (*lst_brv)    ()   ; /* Lister les points d'arret variable */
    IWORD         (*get_val)    ()   ; /* Lire la valeur d'une variable      */
    IWORD         (*set_val)    ()   ; /* Modifier la valeur d'une variable  */
    IWORD         (*get_var)    ()   ; /* Formatter la valeur d'une variable */
    IWORD         (*get_arr)    ()   ; /* Formatter la valeur d'un tableau   */
    IWORD         (*get_thr)    ()   ; /* Rendre les parametres d'un thread  */
  END               Dbs_int          ; /* ---------------------------------- */


/*
    3    Public variable
    ______________________
*/


/*
    3.1  Defined in "hypspace.h"
    ----------------------------
*/


extern const USHORT     hscr_max     ; /* Maximum d'ensembles scripts        */
extern const USHORT     hexe_max     ; /* Maximum de threads                 */
extern const UCHAR      hexe_max_met ; /* Nb de methodes empilees/thread     */
extern const USHORT     hexe_sta_nb  ; /* Nombre de ctx CPI statique         */
extern const USHORT     hscr_sta_nb  ; /* Nombre contextes statiques         */
extern       USHORT     hexe_etat[]  ; /* Table variabled'etat SCR_EXE     */
extern       UCHAR      hexe_sp  []  ; /* Stack pointers       SCR_EXE     */
extern       Hexe_fra0  hexe_fra0[]  ; /* Tables frames methode 0.           */
extern       Hexe_ctx   hexe_ctx[]   ; /* Ctx statique des threads           */
extern       Hexe_call  hexe_calls[] ; /* Tables description methodes.       */
extern       Hexe_cpi   hexe_cpi[]   ; /* Ctx statique CPI                   */
extern       Hscr_ctx   hscr_sta[]   ; /* Table  contextes statiques         */


/*
    3.2  Defined in "executif.h"
    ----------------------------
*/

extern INT              heap_int     ; /* Identifiant heap interpreteur.     */
extern INT              hscr_nbexe   ; /* Nombre de ctx d'execution alloues. */
extern INT              hscr_nbdae   ; /* Nombre de ctx d'execution demons.  */
extern INT              stop_calc    ; /* Flag d'arret de la CPI             */
extern INT              hexe_nexe    ; /* Numero du ctx d'execution courant  */
extern Hexe_ctx        *hexe_pexe    ; /* Adr du ctx d'execution courant     */
extern INT              hexe_nscr    ; /* Numero du ctx memoire courant      */
extern Hscr_ctx        *hexe_pscr    ; /* Adr du ctx memoire courant         */
extern INT              hexe_nmet    ; /* Numero de la methode courante      */
extern Hscr_method     *hexe_pmet    ; /* Adr de la methode courante         */
extern Hexe_cpi        *hexe_pcpi    ; /* Adr bloc CPI courant               */
extern Hexe_frame      *hexe_pfra    ; /* Adr bloc FRAME courant             */
extern Hexe_call       *hexe_pcall   ; /* Adr bloc Desc execution courante   */
extern Hexe_valeur     *hexe_popv    ; /* Sommet pile valeurs des operandes  */
extern Hexe_valeur     *hexe_popm    ; /* Base   pile valeurs des operandes  */
extern Hexe_valeur     *hexe_popM    ; /* Limite pile valeurs des operandes  */
extern UCHAR           *hexe_popt    ; /* Sommet pile types   des operandes  */
extern UCHAR           *hexe_locv    ; /* Liste valeur des variablelocales */
extern UCHAR           *hexe_loct    ; /* Liste type   des variablelocales */
extern UCHAR           *hexe_pcod    ; /* Adr premier octet apres parametres */
extern UCHAR           *hexe_pcom    ; /* Base   du code de la methode       */
extern UCHAR           *hexe_pcoM    ; /* Limite du code de la methode       */
extern UCHAR           *hexe_pspe    ; /* Adr zone specifique dans la frame  */
extern INT              hexe_inst    ; /* Code de l'instruction              */
extern Hexe_valeur      hexe_parn[4] ; /* Parametres de l'instruction        */
extern void       (*const *ie_tab)() ; /* Proc traitement micro-instructions */
extern const INT       *ie_pc        ; /* Ad micro-instruction courante      */
extern UCHAR            ie_flag      ; /* Etat du micro-interpreteur         */
extern INT              ie_inst      ; /* Micro-instruction courante         */
extern Hexe_param       hexe_global_param ; /* Parametres globaux de la CPI  */


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD scr_init            (void                                            );
  void  scr_fin             (void                                            );
  IWORD scr_alloc_memctx    (Hscr_ctx**                                      );
  IWORD scr_free_memctx     (INT                                             );
  IWORD scr_load            (CHAR*,CHAR*,UCHAR*,INT*,INT*                    );
  IWORD scr_unload          (INT,INT                                         );
  IWORD scr_swapall         (void                                            );
  IWORD scr_alloc_exectx    (INT,INT,CHAR*,CHAR*,INT,INT,INT*                );
  IWORD scr_free_exectx     (INT                                             );
  IWORD scr_run             (INT                                             );
  void  scr_break           (void                                            );
  void  scr_abort           (void                                            );
  void  putevt_reqscript    (INT                                             );
  IWORD get_thread_id       (INT*                                            );
  IWORD wait_event          (INT,INT,INT*,INT*,UCHAR**,INT*,INT*,INT*,INT*   );
  IWORD module_end          (void                                            );
  IWORD run_script          (CHAR*,INT,INT*                                  );
  IWORD background_script   (CHAR*,INT,INT*                                  );
  IWORD run_script_full     (CHAR*,CHAR*,CHAR*,CHAR*,INT*                    );
  IWORD background_script_full(CHAR*,CHAR*,CHAR*,CHAR*,INT*                  );
  IWORD set_vm_type         (CHAR*,INT*                                      );
  IWORD get_interpreters    (void                                            );
  IWORD get_current_module  (INT,INT*                                        );
  IWORD break_script        (UINT,IWORD(*)(),INT                             );
  IWORD check_script_module (INT                                             );
  void  delete_script_module(INT                                             );
  void  scr_init_mem        (void                                            );
  void  scr_end_mem         (void                                            );
  void  move_frame          (UCHAR*,INT,INT                                  );
  void  move_scrmem         (UCHAR*,INT,INT                                  );
  void  move_cpi            (UCHAR*,INT,INT                                  );
  IWORD push_method         (Hexe_ctx*,INT,INT                               );
  IWORD run_script_id       (INT,INT,CHAR*,INT*                              );
  IWORD run_oid_met         (INT,INT,CHAR*,CHAR*,INT*                        );
  IWORD get_arg_type        (INT*                                            );
  IWORD cte_get             (INT,INT,INT,UCHAR**                             );
  IWORD set_iret_val        (INT                                             );
  IWORD set_lret_val        (LONG                                            );
  IWORD set_oret_val        (INT                                             );
  IWORD show_script         (INT                                             );
  IWORD show_method         (INT,INT                                         );
  IWORD scr_select_exectx   (INT                                             );
  IWORD scr_unselect_exectx (INT                                             );
  IWORD scr_run             (INT                                             );


#endif                                 /* End of EXECUTIF_H.                 */
