/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/gra.h"                 */
/*  Constants and definitions for the graphic interface.                     */
/*                                                                           */
/*  Author : JYA                        Last modification : 01/07/2010       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
            2.1  Structures couleur et rectangle
            2.2  Structures pour "drawable"
            2.3  Structures ROM de description du driver systeme GRA
            2.4  Structures de description d'un ecran et d'un display
            2.5  Structures pour les fenetres
            2.6  Structures pour les fontes et polices
            2.7  Structures pour les images
         3  Public variable
         4  Prototypes of public procedures
            4.1  Prototypes de "gra_scr.c"
            4.2  Prototypes de "gra_win.c"
            4.3  Prototypes de "gra_clip.c"
            4.4  Prototypes de "gra_draw.c"
            4.5  Prototypes de "gra_str.c"
            4.6  Prototypes de "gra_img.c"
            4.7  Prototypes de "img_jpg.c"
            4.8  Prototypes de "img_png.c"

*/


#ifndef GRA_H                          /* If this file is not already        */
#define GRA_H                          /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Valeurs maximales imposees ...............................................*/

#define MAXSCR                6        /* Nombre max d'ecrans                */
#define MAXLAY                4        /* Nombre max de layers par ecran     */
#define MAXFRB                8        /* Nombre max de buffers par layer    */
#define MAXWIN              100        /* Nombre max de fenetres par display */
#define MAXPPT              100        /* Nombre max de sommets polyligne    */
#define MAXTPF               10        /* Nombre max de police               */
#define MAXFNT               32        /* Nombre max de font par police      */


/* Constantes de definition des types d'identifiants ........................*/

#define IDT_SCR               0        /* Type d'identifiant d'ecran         */
#define IDT_LAY               1        /* Type d'identifiant de layer        */
#define IDT_DSP               2        /* Type d'identifiant de display      */
#define IDT_BUF               3        /* Type d'identifiant de buffer       */
#define IDT_WIN               4        /* Type d'identifiant de fenetre      */
#define IDT_OLN               5        /* Type d'identifiant de contour      */
#define IDT_ARE               6        /* Type d'identifiant de surface      */
#define IDT_PXM               7        /* Type d'identifiant de pixmap       */
#define IDT_TPF               8        /* Type d'identifiant de police       */
#define IDT_FNT               9        /* Type d'identifiant de fonte        */
#define IDT_LUT              12        /* Type d'identifiant de lut          */
#define IDT_DRB              16        /* Identifiant de buffer ou de pixmap */


/* Constantes espace de couleurs ............................................*/

#define ELUT                  0        /* Espace de couleur de type LUT      */
#define ERGB                  1        /* Espace de couleur de type RGB      */
#define EYCBCR                2        /* Espace de couleur de type YCBCR    */


/* Constantes de dimensionnement ............................................*/

#define NTPIX                42        /* Nombre de types de pixmap hw       */
#define NDRATT              101        /* Nombre d'attributs de trace        */


/* Constantes pour le  generateur de code QR ................................*/

#define QR_VERSION_AUTO       0        /* Version automatique                */
#define QR_VERSION_1          1        /* Version 1                          */
#define QR_VERSION_2          2        /* Version 2                          */
#define QR_VERSION_3          3        /* Version 3                          */
#define QR_VERSION_4          4        /* Version 4                          */
#define QR_VERSION_5          5        /* Version 5                          */
#define QR_VERSION_6          6        /* Version 6                          */
#define QR_VERSION_7          7        /* Version 7                          */
#define QR_VERSION_8          8        /* Version 8                          */
#define QR_VERSION_9          9        /* Version 9                          */
#define QR_VERSION_10        10        /* Version 10                         */
#define QR_VERSION_11        11        /* Version 11                         */
#define QR_VERSION_12        12        /* Version 12                         */
#define QR_VERSION_13        13        /* Version 13                         */
#define QR_VERSION_14        14        /* Version 14                         */
#define QR_VERSION_15        15        /* Version 15                         */
#define QR_VERSION_16        16        /* Version 16                         */
#define QR_VERSION_17        17        /* Version 17                         */
#define QR_VERSION_18        18        /* Version 18                         */
#define QR_VERSION_19        19        /* Version 19                         */
#define QR_VERSION_20        20        /* Version 20                         */
#define QR_VERSION_21        21        /* Version 21                         */
#define QR_VERSION_22        22        /* Version 22                         */
#define QR_VERSION_23        23        /* Version 23                         */
#define QR_VERSION_24        24        /* Version 24                         */
#define QR_VERSION_25        25        /* Version 25                         */
#define QR_VERSION_26        26        /* Version 26                         */
#define QR_VERSION_27        27        /* Version 27                         */
#define QR_VERSION_28        28        /* Version 28                         */
#define QR_VERSION_29        29        /* Version 29                         */
#define QR_VERSION_30        30        /* Version 30                         */
#define QR_VERSION_31        31        /* Version 31                         */
#define QR_VERSION_32        32        /* Version 32                         */
#define QR_VERSION_33        33        /* Version 33                         */
#define QR_VERSION_34        34        /* Version 34                         */
#define QR_VERSION_35        35        /* Version 35                         */
#define QR_VERSION_36        36        /* Version 36                         */
#define QR_VERSION_37        37        /* Version 37                         */
#define QR_VERSION_38        38        /* Version 38                         */
#define QR_VERSION_39        39        /* Version 39                         */
#define QR_VERSION_40        40        /* Version 40                         */


/*****************************************************************************
 * Taille du pixmap par defaut utilise pour les traces de texte              *
 * Cette taille est etudiee pour avoir un bon compromis entre la memoire     *
 * occupee et la proportion de textes qui n'auront pas besoin d'allouer un   *
 * nouveau pixmap plus grand.                                                *
 *                                                                           *
 * Pour la hauteur, on a besoin de la hauteur maximale + padding de la plus  *
 * grosse fonte qu'on utilise, qui est le Impact 72 actuellement. Sa hauteur *
 * maximale est de 86 pixels, a quoi on ajoute 2 pixels de padding, soit 88. *
 * Pour la largeur, une etude (tres) rapide montre que moins de 5% des       *
 * traces necessitent plus de 400 pixels de large, et que moins de 1%        *
 * necessitent plus de 700 pixels de large.                                  *
 *                                                                           *
 * Il faut egalement savoir que lors de l'allocation d'un pixmap, les        *
 * dimensions sont arrondies au multiple de 16 superieur.                    *
 *                                                                           *
 * Enfin, derniere consideration, un buffer telecom est alloue pour contenir *
 * le(s) bitmaps(s) d'un pixmap. Les tailles de buffer disponibles qui sont  *
 * susceptibles de nous interesser sont les suivantes : 147424 octets,       *
 * 294880 octets et 589792 octets. Vu que le buffer sera reserve en entier,  *
 * autant maximiser son utilisation si possible.                             *
 *                                                                           *
 * On retient 88 pixels de hauteur maximale, arrondis au multiple de 16      *
 * superieur soit 96 pixels pour la hauteur. La largeur maximale de pixmap   *
 * associee (en supposant 4 octets par pixel) est pour chaque taille de buf: *
 *   .  73696 octets ->  176 pixels de large ( 65% des traces)               *
 *   . 147424 octets ->  368 pixels de large ( 95% des traces)               *
 *   . 294880 octets ->  752 pixels de large ( 99% des traces)               *
 *   . 589792 octets -> 1520 pixels de large (100% des traces)               *
 *****************************************************************************/


#define STRPXM_W            752        /* Largeur et hauteur du pixmap de    */
#define STRPXM_H             96        /* trace des chaines de caracteres    */



/*
    2    Structures and typedefs
    ____________________________
*/


/*
    2.1  Structures couleur et rectangle
    ____________________________________
*/


typedef union                          /* Valeur "hardware" d'une couleur    */
  BEGIN                                /* ---------------------------------- */
    UINT            val              ; /* Valeur 32 bits de la couleur       */
    UCHAR           oct[4]           ; /* Valeur des 4 octets separement     */
  END               Tpix_col         ; /* ---------------------------------- */

typedef struct sgra_rec                /* Description d'un rectange          */
  BEGIN                                /* ---------------------------------- */
    INT             x ,y             ; /* Coordonnees du CSG                 */
    INT             w ,h             ; /* Dimensions du rectangle            */
    struct sgra_rec *prev            ; /* Precedent dans la chaine           */
    struct sgra_rec *next            ; /* Suivant   dans la chaine           */
  END               Gra_rec          ; /* ---------------------------------- */


/*
    2.2  Structures pour "drawable"
    _______________________________
*/


typedef struct                         /* Description d'une composante       */
  BEGIN                                /* ---------------------------------- */
    INT             n                ; /* Numero du bitmap de la composante  */
    INT             wc               ; /* Nb de pixels d'une ligne           */
    INT             hc               ; /* Nb de lignes de la composante      */
    INT             dpp              ; /* Distance en octets entre 2 pixels  */
    INT             dll              ; /* Distance en octets entre 2 lignes  */
    INT             o                ; /* Petit offset sur le 1er pixel      */
    UCHAR         **lig              ; /* Ad. table adresses des lignes      */
  END               Gra_drbcmp       ; /* ---------------------------------- */

typedef struct                         /* Description d'une zone " drawable" */
  BEGIN                                /* ---------------------------------- */
    UCHAR           tpix             ; /* Organisation des pixels            */
    UCHAR           cmpmask          ; /* Autorise trace des composantes     */
    UCHAR           maxt             ; /* Valeur maximale de transparence    */
    UCHAR           premult          ; /* Flag: transparence pre-multipliee  */
    USHORT          wb,hb            ; /* Dimensions des bitmaps             */
    Gra_drbcmp      drbcmp[4]        ; /* Description des 4 composantes      */
    Tpix_col        hwc              ; /* Couleur "hardware" de trace        */
    IWORD           alpha            ; /* Facteur de melange ou -1 si WRITE  */
    Gra_rec         dirty            ; /* Zone "sale" de la zone tracable    */
    USHORT          wsz[4]           ; /* Nb. d'octets entre 2 lignes bitmap */
    UCHAR           *bmp[4]          ; /* Ad. des 1 a 4 bitmaps de la tranche*/
  END               Gra_drb          ; /* ---------------------------------- */
 
typedef struct                         /* Description d'un pixmap RAM        */
  BEGIN                                /* ---------------------------------- */
    UCHAR           free             ; /* Flag de structure utilisee/libre   */
    Gra_drb         pxm              ; /* Zone "tracable"                    */
  END               Gra_pxm          ; /* ---------------------------------- */


/*
    2.3  Structures ROM de description du driver systeme GRA
    ________________________________________________________
*/

/*
  AL 10/04/15 : Les procedures"paintrect", "copyarea" et "drawpixmap"
  sont appelees depuis du code C++ pour le wrapper de webkit. En C++,
  l'absence de parametres dans la declaration d'une procedure implique
  que cette procedure ne prend aucun parametre. C'est un equivalent du
  parametre de type "void" en C. On est donc oblige ici d'opter pour la
  declaration explicite des parametres pour ces trois procedures
*/

typedef const struct                   /* Description d'un layer             */
  BEGIN                                /* ---------------------------------- */
    IWORD         (*init         )() ; /* Initialisation du layer            */
    IWORD         (*end          )() ; /* Termination    du layer            */
    IWORD         (*movedisplay  )() ; /* Modification d'un display          */
    IWORD         (*setdispsource)() ; /* Selection rectangle source affiche */
    IWORD         (*setdisplut   )() ; /* Specification table couleurs       */
    IWORD         (*setdisptransp)() ; /* Reglage niveau global de transpar  */
    IWORD         (*setbuffer    )() ; /* Selection buffer affiche           */
    IWORD         (*getbufdrb    )() ; /* Retourne struct. drawable hardware */
                                       /* Trace rectangle plein              */
    IWORD         (*paintrect    )(Gra_drb*,UCHAR*[4],IWORD,IWORD,IWORD,IWORD);
                                       /* Copie de pixels                    */
    IWORD         (*copyarea     )(Gra_drb*,IWORD,IWORD,IWORD,IWORD,Gra_drb*,
                                   IWORD,IWORD                            );
                                       /* Trace d'un pixmap                  */
    IWORD         (*drawpixmap   )(Gra_drb*,UCHAR*[4],IWORD,IWORD,IWORD,IWORD,
                                   Gra_drb*,UCHAR*[4],IWORD,IWORD,IWORD,IWORD,
                                   Tpix_col,IWORD,IWORD,IWORD*            );
    IWORD         (*flush        )() ; /* Executer les operations differees  */
    IWORD         (*sync         )() ; /* Attente fin trace hardware         */
    INT             dsp_nb           ; /* Nombre de displays pour ce plan    */
    INT             buf_nb           ; /* Nombre de buffers pour ce plan     */
    INT             dsp_pos          ; /* Positionnement des displays        */
    INT             dsp_alix         ; /* Align horizontal CSG display       */
    INT             dsp_aliy         ; /* Align vertical   CSG display       */
    INT             dsp_stpw         ; /* Pas de largeur  d'un display       */
    INT             dsp_stph         ; /* Pas de hauteur  d'un display       */
    INT             dsp_info         ; /* Infos generales displays           */
    UINT            pxinfo[2]        ; /* Masque des organisations bitmaps   */
    INT             pxdef            ; /* Pix hard par default layer         */
  END               Gra_desc_lay     ; /* ---------------------------------- */

typedef struct                         /* Description de l'interface etendue */
  BEGIN                                /* ---------------------------------- */
    IWORD         (*createwindow  )(); /* Creation             d'une fenetre */
    IWORD         (*deletewindow  )(); /* Destruction          d'une fenetre */
    IWORD         (*lowerwindow   )(); /* Mise en arriere plan d'une fenetre */
    IWORD         (*upperwindow   )(); /* Mise en avant   plan d'une fenetre */
    IWORD         (*showwindow    )(); /* Rendre visible         une fenetre */
    IWORD         (*hidewindow    )(); /* Rendre invisible       une fenetre */
    IWORD         (*changewindow  )(); /* Modification         d'une fenetre */
                                       /*                                    */
    IWORD         (*drawline      )(); /* Tracer un segment de droite        */
    IWORD         (*drawarc       )(); /* Tracer un arc de cercle            */
    IWORD         (*drawrectangle )(); /* Tracer un rectangle                */
    IWORD         (*drawroundrect )(); /* Tracer un rect aux coins arrondis  */
    IWORD         (*drawpoint     )(); /* Tracer un point sur l'ecran        */
    IWORD         (*drawstring    )(); /* Tracer une chaine de caracteres    */
    IWORD         (*getstringsize )(); /* Donne largeur/hauteur d'une chaine */
    IWORD         (*getfontsize   )(); /* Donne la hauteur d'une police      */
    IWORD         (*drawpolyline  )(); /* Tracer une ligne polygonale        */
    IWORD         (*paintarc      )(); /* Peindre un arc de cercle           */
    IWORD         (*paintrectangle)(); /* Peindre un rectangle               */
    IWORD         (*paintroundrect)(); /* Peindre un rect. aux coins arrondis*/
    IWORD         (*paintpolyline )(); /* Peindre une zone polygonale        */
  END               Gra_desc_ext     ; /* ---------------------------------- */

typedef struct                         /* Description du driver GRA          */
  BEGIN                                /* ---------------------------------- */
    IWORD         (*init       )()   ; /* Procedure d'initialisation         */
    void          (*end        )()   ; /* Procedure de terminaison           */
    IWORD         (*setdsporder)()   ; /* Ordre d'affichage des displays     */
    Gra_desc_lay   *desc_lay[MAXLAY] ; /* Interface des layers               */
    Gra_desc_ext   *desc_ext         ; /* Interface etendue stations         */
  END               Gra_desc         ; /* ---------------------------------- */
 
typedef struct                         /* Parametres du driver systeme GRA   */
  BEGIN                                /* ---------------------------------- */
    INT             width            ; /* Largeur de l'ecran en pixels       */
    INT             height           ; /* Hauteur de l'ecran en pixels       */
    INT             type             ; /* 0:Ecran  1:Traceur  2:IMP 3:Autre  */
  END               Gra_def          ; /* ---------------------------------- */


/*
    2.4  Structures de description d'un ecran et d'un display
    _________________________________________________________
*/

typedef struct                         /* Description d'un display           */
  BEGIN                                /* ---------------------------------- */
    INT             xd,yd            ; /* Coordonnees du CSG du display      */
    INT             wd,hd            ; /* Dimensions du display              */
    INT             xs,ys            ; /* Coordonnees du CSG du rect. source */
    INT             ws,hs            ; /* Dimensions du rect. source         */
    INT             lut[1+256]       ; /* La LUT du display                  */
    UCHAR           root             ; /* Numero de la fenetre racine        */
    UCHAR           transp           ; /* Valeur de transparence globale     */
    UCHAR           disbuf           ; /* Numero du buffer qui est affiche   */
    UCHAR           drawbuf          ; /* Numero du buffer de trace courant  */
  END               Gra_dsp          ; /* ---------------------------------- */

typedef struct                         /* Description d'un layer             */
  BEGIN                                /* ---------------------------------- */
    UINT            allbuf           ; /* Bitmap des buffers alloues         */
    UCHAR           nbbuf            ; /* Nombre total de buffers            */
    Gra_drb         buf  [MAXFRB]    ; /* Ad. des pixmaps, frame-buffers     */
  END               Gra_lay          ; /* ---------------------------------- */

typedef struct                         /* Description d'un ecran             */
  BEGIN                                /* ---------------------------------- */
    const CHAR     *scrname          ; /* Nom de l'ecran                     */
    IWORD         (*setdsporder)()   ; /* Ordre d'affichage des displays     */
    Gra_desc_lay   *desc_lay[MAXLAY] ; /* Interface des layers               */
    Gra_desc_ext   *desc_ext         ; /* Interface etendus des stations     */
    INT             width            ; /* Largeur de l'ecran en pixels       */
    INT             height           ; /* Hauteur de l'ecran en pixels       */
    INT             type             ; /* 0:Ecran 1:Traceur 2:IMP ...        */
    UCHAR           dsp_nb           ; /* Nb. total de diplays de l'ecran    */
    UCHAR           dsp_ord[MAXLAY*16];/* Ordre d'affichage layers/displays  */
    Gra_lay         gra_lay[MAXLAY]  ; /* Description des layers             */
  END               Gra_scr          ; /* ---------------------------------- */


/*
    2.5  Structures pour les fenetres
    _________________________________
*/

typedef struct sgra_win                /* Description d'une fenetre          */
  BEGIN                                /* ---------------------------------- */
    INT             id               ; /* Identifiant de la fenetre          */
    UCHAR           dirty            ; /* Flag grille clipping a refaire     */
    UCHAR           ns,nl,nd         ; /* Numero d'ecran/layer/display       */
    UCHAR           np               ; /* Numero de pixmap ou FFh            */
    UCHAR           nu               ; /* Numero de LUT ou FFh               */
    USHORT          x,y              ; /* Coordonnees du CSG dans le display */
    USHORT          w,h              ; /* Dimensions de la fenetre           */
    struct sgra_win *mother          ; /* Ad. de la mere                     */
    struct sgra_win *sister          ; /* Ad. de la soeur au dessus          */
    struct sgra_win *daught          ; /* Ad. de la fille au dessous         */
    Gra_rec         *recvis          ; /* Ad. chaine des rect visibles       */
    Gra_rec         *recapp          ; /* Ad. chaine des rect applicatifs    */
    Gra_rec         *recdraw         ; /* Ad. chaine des rect a retracer     */
    UCHAR           *grid            ; /* Ad. buffer de la grille            */
    INT             att              ; /* Attribut de la fenetre             */
    INT             drawatt[NDRATT]  ; /* Ensemble d'attributs de trace      */
    Tpix_col        hwfcol           ; /* Couleur "harware" de trace         */
    Tpix_col        hwbcol           ; /* Couleur "harware" de fond          */
    Tpix_col        hwncol           ; /* Couleur "harware" a ne pas tracer  */
  END               Gra_win          ; /* ---------------------------------- */

typedef struct                         /* Description d'une pile de fenetres */
  BEGIN                                /* ---------------------------------- */
    Gra_win        *win              ; /* Ad. de la fenetre empilee          */
    IWORD           pos              ; /* Position d'empilement              */
  END               Gra_winstk       ; /* ---------------------------------- */


/*
    2.6  Structures pour les fontes et polices
    __________________________________________
*/


typedef struct                         /* Description d'une fonte            */
  BEGIN                                /* ---------------------------------- */
    UINT            cset[2]          ; /* Masque de 64 bits des codesets     */
    UCHAR           size             ; /* Taille de la fonte                 */
    UCHAR           style            ; /* Style, Gras/Souligne/Italique      */
    USHORT          scale            ; /* Definition de l'unite "Font Unit"  */
    USHORT          ascent           ; /* Hauteur    max au-dessus  baseline */
    USHORT          descent          ; /* Profondeur max en-dessous baseline */
    USHORT          widthmax         ; /* Largeur du plus large des glyphs   */
    USHORT          nglyphs          ; /* Nombre de glyphs definis           */
    USHORT          ncodes           ; /* Nombre de codes de glyphs definis  */
    UINT            indexsz          ; /* Taille de l'index en octets        */
    UINT            gdatasz          ; /* Taille des donnees glyphs, en octs */
    UCHAR           data[4]          ; /* Ad. des donnees index, puis glyphs */
  END               Gra_fnt          ; /* ---------------------------------- */

typedef struct                         /* Description d'une police           */
  BEGIN                                /* ---------------------------------- */
    CHAR            name[21]         ; /* Nom de la police                   */
    UCHAR           nfnt             ; /* Nombre de fontes dans cette police */
    Gra_fnt        *fnt[MAXFNT]      ; /* Adresse des structures "fonte"     */
  END               Gra_tpf          ; /* ---------------------------------- */


/*
    2.7  Structures pour les images
    _______________________________
*/

typedef struct                         /* Description d'une composante image */
  BEGIN                                /* ---------------------------------- */

/* Donnees utiles au clipping ...............................................*/

    INT             xmdp , ymdp      ; /* Centre du rectangle destination    */
    INT             x1   , y1        ; /* CSG rectangle de clipping          */
    INT             x2   , y2        ; /* CID rectangle de clipping          */
    Gra_rec        *clip             ; /* Liste chainee de rectangles de clip*/

/* Donnnes utiles aux calculs d'adresses dans le "drawable" .................*/
 
    INT             bwi              ; /* Largeur du bloc dans espace image  */
    INT             bhi              ; /* Hauteur du bloc dans espace image  */
    INT             bwp              ; /* Largeur en pix du bloc ecrit (<=64)*/
    INT             bhp              ; /* Hauteur en pix du bloc ecrit (<=64)*/
    INT             ximul            ; /* Multiplicateur de "xi" pour calcul */
    INT             yimul            ; /* Multiplicateur de "yi" pour calcul */
    INT             xfmul            ; /* Multiplicateur de "xf" pour calcul */
    INT             yfmul            ; /* Multiplicateur de "yf" pour calcul */

/* Donnnes de "guidage" de l'ecriture d'un bloc 8x8 .........................*/

    UCHAR          *itab             ; /* Indices dans le bloc 8x8           */
    INT            *otab             ; /* Table d'offsets par rapport a base */
    IWORD           maxtab           ; /* Nombre d'elements de itab/otab     */
 
/* Adresse de la grille de clipping de pixels ...............................*/

    UCHAR         **ind              ; /* Ad du buffer telecom "grille"      */
  END               Gra_imgcmp       ; /* ---------------------------------- */

typedef struct                         /* Contexte d'une image               */
  BEGIN                                /* ---------------------------------- */
    
/* Valeurs constantes recopies pour optimisation ............................*/
 
    INT             ns               ; /* Numero de l'ecran                  */
    INT             nl               ; /* Numero de la couche/plan graphique */
    INT             nd               ; /* Numero du display                  */
    INT             nw               ; /* Numero de la fenetre               */
    INT             np               ; /* Numero du pixmap ou 0xFF           */
    UCHAR           tpixout          ; /* Code du type de pixmap hardware    */
 
/* Parametres fournis par XSet_Image_Size ...................................*/
 
    INT             iwp              ; /* Largeur en pixels de l'image       */
    INT             ihp              ; /* Hauteur en pixels de l'image       */
    UCHAR           tpixin           ; /* Code du type de pixmap de l'image  */
    INT             iwb              ; /* Largeur en blocs de l'image        */
    INT             ihb              ; /* Hauteur en blocs de l'image        */
 
/* Parametres fournis par XSet_Image_source .................................*/
 
    INT             xsp              ; /* X CSG zone a tracer                */
    INT             ysp              ; /* Y CSG zone a tracer                */
    INT             wsp              ; /* Largeur en pixels zone a tracer    */
    INT             hsp              ; /* Hauteur en pixels zone a tracer    */
 
/* Parametres fournis par XSet_Image_dest ...................................*/
 
    INT             xdp              ; /* X destination                      */
    INT             ydp              ; /* Y destination                      */
    INT             wdp              ; /* Largeur en pixels                  */
    INT             hdp              ; /* hauteur en pixels                  */
    UINT            flags            ; /* Parametres du decodage             */

/* 4 contextes de composantes image .........................................*/
 
    Gra_imgcmp      imgcmp[4]        ; /* 4 contextes de composantes         */

/* Rectangle source "limite" par les dimensions de l'image ..................*/

    INT             xspc , yspc      ; /* Valeurs "limitees" de xsp et ysp   */
    INT             wspc , hspc      ; /* Valeurs "limitees" de wsp et hsp   */
    INT             xmsi , ymsi      ; /* Centre du rectangle precedent      */
  END               Gra_img          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________


    4.1  Prototypes de "gra_scr.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD xget_screen_count   (INT* ,INT* ,INT* ,INT* ,INT*                    );
  IWORD xget_screen_info    (INT  ,CHAR*,INT* ,INT*                          );
  IWORD xget_layer_info     (INT  ,INT* ,INT*                                );
  IWORD xget_display_info   (INT  ,INT* ,INT*                                );
  IWORD xset_display_area   (INT  ,INT  ,INT  ,INT  ,INT  ,INT*              );
  IWORD xget_display_area   (INT  ,INT* ,INT* ,INT* ,INT* ,INT*              );
  IWORD xset_display_source (INT  ,INT  ,INT  ,INT  ,INT  ,INT*              );
  IWORD xget_display_source (INT  ,INT* ,INT* ,INT* ,INT* ,INT*              );
  IWORD xset_display_transparency (INT  ,INT  ,INT*                          );
  IWORD xget_display_transparency (INT  ,INT* ,INT*                          );
  IWORD xset_display_lut    (INT  ,INT  ,INT*                                );
  IWORD xget_display_lut    (INT  ,INT  ,INT*                                );
  IWORD xalloc_buffer       (INT  ,INT* ,INT*                                );
  IWORD xfree_buffer        (INT  ,INT*                                      );
  IWORD xset_drawing_buffer (INT  ,INT  ,INT* ,INT*                          );
  IWORD xget_drawing_buffer (INT  ,INT* ,INT*                                );
  IWORD xget_dirty_rectangle(INT  ,INT* ,INT* ,INT* ,INT* ,INT*              );
  IWORD xreset_dirty_rectangle(INT  ,INT*                                    );
  IWORD xset_current_buffer (INT  ,INT  ,INT* ,INT*                          );
  IWORD xget_current_buffer (INT  ,INT* ,INT*                                );
  IWORD xflush              (INT  ,INT* ,INT*                                );
  IWORD xcopy_area          (INT,INT,INT,INT,INT,INT,INT,INT,INT*            );
  IWORD xalloc_pixmap       (INT  ,INT  ,INT  ,INT  ,INT* ,INT*              );
  IWORD xfree_pixmap        (INT  ,INT*                                      );
  IWORD xconvert_color      (INT,INT,INT,INT,INT,INT*,INT*,INT*,INT*         );
  IWORD xcode_color         (INT,INT,INT,INT,INT,INT,INT*,INT*               );
  IWORD xdecode_color       (INT  ,INT  ,INT* ,INT* ,INT* ,INT* ,INT*        );
  IWORD xfind_color         (INT,INT,INT,INT,INT,INT,INT*,INT*               );
  IWORD xcast_color         (INT  ,INT  ,INT  ,UCHAR[]    ,INT*              );
  IWORD xalloc_lut          (INT  ,INT* ,INT*                                );
  IWORD xfree_lut           (INT  ,INT*                                      );
  IWORD xcopy_lut           (INT  ,INT  ,INT  ,INT  ,INT  ,INT*              );
  IWORD xset_lut_color      (INT  ,INT  ,INT  ,INT*                          );
  IWORD xget_lut_color      (INT  ,INT  ,INT* ,INT*                          );
  IWORD xdither_lut_color   (INT  ,INT  ,INT  ,INT  ,INT  ,INT*              );


/* Procedures publiques C uniquement ....................................... */

  IWORD chk_graid           (INT   ,IWORD ,IWORD*,IWORD*,IWORD*,IWORD*       );
  IWORD chk_color           (INT   ,IWORD ,IWORD*,IWORD*,IWORD*,IWORD*,IWORD*);
  IWORD cast_color          (Gra_drb*,IWORD,INT  ,Tpix_col*                  );
  IWORD search_color        (IWORD,IWORD,IWORD,IWORD,INT const*              );
  void  cast_lut            (const INT*   ,Tpix_col*                         );
  void  get_drawbuf         (IWORD ,IWORD ,IWORD ,Gra_drb**                  );
  void  add_dirtyrec        (Gra_drb* ,IWORD ,IWORD ,IWORD ,IWORD            );
  void  add_dirtyarea       (Gra_drb* ,UCHAR*                                );


/*
    4.2  Prototypes de "gra_win.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD xcreate_window      (INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT,INT*,INT*);
  IWORD xdelete_window      (INT ,INT*                                       );
  IWORD xget_root_window    (INT ,INT*,INT*                                  );
  IWORD xget_window_area    (INT ,INT*,INT*,INT*,INT*,INT*                   );
  IWORD xget_window_info    (INT ,INT*,INT*,INT*,INT*,INT*                   );
  IWORD xlower_window       (INT ,INT*                                       );
  IWORD xraise_window       (INT ,INT*                                       );
  IWORD xshow_window        (INT ,INT*                                       );
  IWORD xhide_window        (INT ,INT*                                       );
  IWORD xset_window_lut     (INT ,INT ,INT*                                  );
  IWORD xget_window_lut     (INT ,INT*,INT*                                  );


/* Procedures publiques C uniquement ....................................... */

  IWORD create_win          (IWORD ,IWORD ,IWORD ,IWORD ,IWORD ,IWORD ,IWORD  ,
                             IWORD ,IWORD ,INT   ,INT   ,IWORD               );
  void  delete_win          (IWORD                                           );


/*
    4.3  Prototypes de "gra_clip.c"
    -------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD xset_cliplist       (INT ,INT[][4] ,INT ,INT*                        );
  IWORD xunset_cliplist     (INT ,INT*                                       );
  IWORD xget_cliplist       (INT ,INT[][4] ,INT ,INT*,INT*                   );
  IWORD xset_cliprect       (INT ,INT ,INT ,INT ,INT ,INT*                   );
  IWORD xget_cliprect       (INT ,INT*,INT*,INT*,INT*,INT*                   );


/* Procedures publiques C uniquement ....................................... */
 
  IWORD set_recvis          (Gra_win*                                        );
  void  free_recvis         (Gra_win*                                        );
  IWORD set_recvis_tree     (Gra_win*                                        );
  void  free_recvis_tree    (Gra_win*                                        );
  IWORD get_recdraw         (IWORD,INT,INT,INT*,INT*,INT*,INT*               );
  IWORD int_recrec          (Gra_rec* ,Gra_rec* ,INT*,INT*,INT*,INT*         );
  void  union_recrec        (Gra_rec* ,Gra_rec* ,INT*,INT*,INT*,INT*         );
  IWORD div_recbyrec        (Gra_rec* ,Gra_rec* ,Gra_rec**,Gra_rec**,Gra_rec**,
                             IWORD*   ,IWORD*                                );
  IWORD int_listlist        (Gra_rec* ,Gra_rec* ,Gra_rec**,Gra_rec**,IWORD*  );
  IWORD div_recbylist       (Gra_rec* ,Gra_rec* ,Gra_rec**,Gra_rec**,IWORD*  );
  IWORD div_listbyrec       (Gra_rec**,Gra_rec*                              );
  IWORD nodup_list          (Gra_rec**                                       );
  IWORD group_list          (Gra_rec**                                       );
  IWORD canon_list          (Gra_rec**                                       );
  IWORD scale_list          (Gra_rec*,IWORD,IWORD,IWORD,IWORD,IWORD,IWORD     ,
                             Gra_rec**                                       );
  Gra_rec* alloc_rec        (IWORD,IWORD,IWORD,IWORD,Gra_rec**,Gra_rec**     );
  void     free_rec         (Gra_rec**,Gra_rec**,Gra_rec**                   );
  void     free_all_rec     (Gra_rec**                                       );
  IWORD    calc_grid        (Gra_win*                                        );


/*
    4.4  Prototypes de "gra_draw.c"
    -------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD xget_drawing_att    (INT ,INT ,INT*,INT*                             );
  IWORD xset_drawing_att    (INT ,INT ,INT ,INT*                             );
  IWORD xdraw_point         (INT ,INT ,INT ,INT*                             );
  IWORD xdraw_line          (INT ,INT ,INT ,INT ,INT ,INT*                   );
  IWORD xdraw_polyline      (INT ,INT[][2] ,INT ,INT*                        );
  IWORD xdraw_rectangle     (INT ,INT ,INT ,INT ,INT ,INT*                   );
  IWORD xdraw_arc           (INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT*    );
  IWORD xdraw_roundrect     (INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT*         );
  IWORD xpaint_polyline     (INT ,INT[][2] ,INT ,INT*                        );
  IWORD xpaint_rectangle    (INT ,INT ,INT ,INT ,INT ,INT*                   );
  IWORD xpaint_arc          (INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT*    );
  IWORD xpaint_roundrect    (INT ,INT ,INT ,INT ,INT ,INT ,INT ,INT*         );
  IWORD xpolygon_lpnt       (INT[][2]      ,INT ,INT*,INT*                   );
  IWORD xarc_lpnt           (INT ,INT ,INT ,INT ,INT ,INT ,INT, INT,INT*,INT*);
  IWORD xroundrect_lpnt     (INT ,INT ,INT ,INT ,INT ,INT ,INT*,INT*         );
  IWORD xdraw_lpnt          (INT ,INT ,INT*                                  );
  IWORD xfree_lpnt          (INT ,INT*                                       );
  IWORD xshape_area         (INT ,INT ,INT*,INT*,INT*                        );
  IWORD xlpnt_area          (INT ,INT ,INT*,INT*                             );
  IWORD xopbin_area         (INT ,INT ,INT ,INT*,INT*                        );
  IWORD xdraw_area          (INT ,INT ,INT*                                  );
  IWORD xfree_area          (INT ,INT*                                       );
  IWORD xdraw_pixmap        (INT,INT,INT,INT,INT,INT,INT,INT,INT,INT,INT*    );


/* Procedures publiques C uniquement ....................................... */

  void  get_drawdrb         (Gra_win * ,Gra_drb**                            );
  IWORD paint_rect          (Gra_win * ,Tpix_col  ,IWORD ,IWORD ,IWORD ,IWORD);


/*
    4.5  Prototypes de "gra_str.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD ximport_length      (CHAR*,INT,INT,INT,INT*,INT*,INT*                );
  IWORD ximport_string      (CHAR*,INT,INT,INT,CHAR*,INT*,INT*,INT*          );
  IWORD ximport_bstring     (CHAR*,INT,INT,INT,CHAR*,INT ,INT*,INT*,INT*     );
  IWORD xget_string_length  (CHAR*,INT*                                      );
  IWORD xget_typeface_list  (CHAR*,INT*,INT*                                 );
  IWORD xget_font_list      (INT  ,CHAR*,INT* ,INT*                          );
  IWORD xget_typeface_id    (CHAR*,INT* ,INT* ,INT*                          );
  IWORD xget_font_id        (CHAR*,INT* ,INT* ,INT*                          );
  IWORD xget_typeface_info  (INT,CHAR*  ,INT[],INT*                          );
  IWORD xget_font_info      (INT,CHAR*  ,INT[],INT*                          );
  IWORD xget_glyph_info     (INT,INT    ,INT[],INT*                          );
  IWORD xget_kern_info      (INT,INT,INT,INT[],INT*                          );
  IWORD xselect_font        (INT,INT[],INT,INT,INT*,INT*,INT*                );
  IWORD xdraw_string        (INT,INT,INT,CHAR*,INT*                          );
  IWORD xget_string_size    (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,INT*         );


/* Procedures publiques C uniquement ....................................... */

/*
    4.6  Prototypes de "gra_img.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD xget_bitmap_info    (INT,INT,INT*,INT*,INT*,INT*,INT*                );
  IWORD xget_bitmap_rect    (INT,INT,INT*,INT*,INT*,INT*,INT*                );
  IWORD xget_bitmap_slice   (INT,INT,INT,UCHAR**,INT*,INT*,INT*              );
  IWORD xget_bitmap_line    (INT,INT,INT,UCHAR**,INT*,INT*                   );
  IWORD xset_image_size     (INT,INT,INT,INT,INT*                            );
  IWORD xget_image_size     (INT,INT*,INT*,INT*,INT*                         );
  IWORD xset_image_source   (INT,INT,INT,INT,INT,INT*                        );
  IWORD xget_image_source   (INT,INT*,INT*,INT*,INT*,INT*                    );
  IWORD xset_image_dest     (INT,INT,INT,INT,INT,INT,INT*                    );
  IWORD xget_image_dest     (INT,INT*,INT*,INT*,INT*,INT*,INT*               );
  IWORD xdraw_image_block   (INT,INT,INT,INT,UCHAR*,UCHAR*,INT*              );


/* Procedures publiques C uniquement ....................................... */

  Gra_drb* get_win_drb      (INT                                             );
  void  drb_init_cmp        (Gra_drb*                                        );
  void  drb_free_cmp        (Gra_drb*                                        );
  void  img_init_ctx        (INT                                             );
  void  img_updt_ctx        (INT,INT                                         );
  void  img_free_ctx        (INT                                             );
  void  img_draw_block      (Gra_img*,INT,INT,INT,UCHAR*,UCHAR*              );
  void  img_draw_line       (Gra_img*,INT,INT,INT,INT,UCHAR*,UCHAR*          );
  IWORD img_alloc_itabotab  (Gra_img*                                        );

/*
    4.7  Prototypes de "img_jpg.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

  IWORD jpg_init            (INT*,INT*                                       );
  IWORD jpg_decode          (INT,UCHAR*,INT,INT,INT*,INT*                    );
  IWORD jpg_info            (INT,CHAR*,INT*,INT*,INT*,INT*,INT*,CHAR*,CHAR*   ,
                             CHAR*,INT*,INT*                                 );
  IWORD jpg_end             (INT,INT*                                        );
  IWORD jpg_setwin          (INT,INT,INT*                                    );


/*
    4.8  Prototypes de "img_png.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

/* Procedures publiques C uniquement ....................................... */

  IWORD png_init            (INT,INT*,INT*                                   );
  IWORD png_info            (INT,INT*,INT*,INT*,INT*                         );
  IWORD png_decode          (INT,UCHAR*,INT,INT,INT*,INT*                    );
  IWORD png_end             (INT,INT*                                        );
  IWORD png_setwin          (INT,INT,INT*                                    );


/*
    4.9  Prototypes de "img_cod.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

/* Procedures publiques C uniquement ....................................... */

  IWORD cod_init            (INT,INT*,INT*                                   );
  IWORD cod_info            (INT,INT*,INT*,INT*,INT*                         );
  IWORD cod_decode          (INT,UCHAR*,INT,INT,INT*,INT*                    );
  IWORD cod_end             (INT,INT*                                        );


/*
    4.10 Prototypes de "img_gif.c"
    ------------------------------
*/

/* Procedures interfacees en PanTalk ....................................... */

/* Procedures publiques C uniquement ....................................... */

  IWORD gif_init            (INT,INT*,INT*                                   );
  IWORD gif_info            (INT,INT*,INT*,INT*,INT*                         );
  IWORD gif_decode          (INT,UCHAR*,INT,INT,INT*,INT*,INT*               );
  IWORD gif_end             (INT,INT*                                        );
  IWORD gif_setwin          (INT,INT,INT*                                    );


#endif                                 /* Fin du ifdef GRA_H                 */
