/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/hhpf.h"                */
/*  Constants and definitions for Hyperpanel streams.                        */
/*                                                                           */
/*  Author : JYA                        Last modification : 15/03/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef HHPF_H                         /* If this file is not already        */
#define HHPF_H                         /* included, include it.              */


/*
    1    Defines
    ____________


    Codage des informations dans la structure Objet :
 
         0     5      12         23      31             47           63
         | (1) | (2)  |   (3)    | (4)   | (5) |   (6)  |    (7)     |
         +-----+------+----------+-------+---- +--------+------------+
         | verrou et compteur de |dimen- |description   |description |
         | referencement.        |sion   |generale      |specifique  |
         +-----------------------+-------+--------------+------------+
         |          UINT prop            |USHORT clatyp |USHORT spe  |
         +-------------------------------+--------------+------------+ 
 
    (1)  Nombre de verrous sur l'objet            sur  6 bits (0...63  )
    (2)  Numero du thread proprietaire du verrou  sur  7 bits (0...127 )
    (3)  Compteur de referencement                sur 11 bits (0...2047)
    (4)  Dimension de l'objet                     sur  8 bits (0...255 )
    (5)  Description generale   : type de l'objet sur  4 bits (0...15  )
         bit de finalisation                      sur  1 bit  
    (6)  Description generale   : classe          sur 11 bits (0...2047)
    (7)  Description necessaire specific (unused) sur 16 bits (0.......)

    (1)  prop              0xFC000000
    (2)  prop              0x03F80000
    (3)  prop              0x0007FF00
    (4)  prop              0x000000FF
    (5)  cla_typ           0x00007800
    ( )  cla_typ           0x00008000
    (6)  cla_typ           0x000007FF
    (7)  spe               0x0000FFFF

    Codage et decodage des informations dans la structure Object :

    HOBJ_GET_/SET_TYP      Type d'objet : OBJ_CLASS, OBJ_ARRAY_CLASS, ...
    HOBJ_GET_/SET_LOCK_NB  Le nombre de verrous sur l'objet
    HOBJ_GET_/SET_LOCK_NX  Le thread proprietaire du clef du verrou
    HOBJ_GET_/SET_NB_REF   Le nombre de references vers l'objet
    HOBJ_GET_/SET_DIM      La dimension
    HOBJ_GET_/SET_CLA      Classe instanciee par l'objet ou le type primitive
                           composant d'un tableau OBJ_ARRAY_PRIM
*/


/*****************************************************************************
 * Macros decodage des informations dans la structure Object                 *
 * pobj est le pointeur sur l'entete de l'objet structure Hhpf_obj           *
 *****************************************************************************/


#define HOBJ_GET_FIN(pobj)     (((pobj)->cla_typ >> 15) AND (USHORT)0x0001  )
#define HOBJ_GET_TYP(pobj)     (((pobj)->cla_typ >> 11) AND (USHORT)0x000F  )
#define HOBJ_GET_CLA(pobj)     (((pobj)->cla_typ      ) AND (USHORT)0x07FF  )
#define HOBJ_GET_LOCK_NB(pobj) (((pobj)->prop    >> 26) AND (UINT)0x0000003F)
#define HOBJ_GET_LOCK_NX(pobj) (((pobj)->prop    >> 19) AND (UINT)0x0000007F)
#define HOBJ_GET_NB_REF(pobj)  (((pobj)->prop    >>  8) AND (UINT)0x000007FF)
#define HOBJ_GET_DIM(pobj)     (((pobj)->prop         ) AND (UINT)0x000000FF)


/*****************************************************************************
 *        Macros codage des informations dans la structure l'objet           *
 *                  n, t ou c sont les valeurs a ecrire.                     *
 *     pobj est le pointeur sur l'entete de l'objet, structure Hhpf_obj      *
 *             Attention, on n'assure pas la taille des valeurs.             *
 *            (de toute facon, s'ils depassent, on a une erreur)             *
 *****************************************************************************/


#define HOBJ_SET_FIN(pobj)       (pobj)->cla_typ=(pobj)->cla_typ | 0x8000 ;
#define HOBJ_SET_TYP(t,pobj)     (pobj)->cla_typ=((pobj)->cla_typ  &   \
                                                  0x87FF         ) |   \
                                                 ((USHORT)(t)<<11)     ;

#define HOBJ_SET_CLA(c,pobj)     (pobj)->cla_typ=((pobj)->cla_typ  &   \
                                                  0xF800         ) |   \
                                                  (USHORT)(c)          ;

#define HOBJ_SET_LOCK_NB(n,pobj) (pobj)->prop   =((pobj)->prop     &   \
                                                  0x03FFFFFF     ) |   \
                                                 ((UINT)(n) << 26)     ;

#define HOBJ_SET_LOCK_NX(n,pobj) (pobj)->prop   =((pobj)->prop     &   \
                                                  0xFC07FFFF     ) |   \
                                                 ((UINT)(n) << 19)     ;

#define HOBJ_SET_NB_REF(n,pobj)  (pobj)->prop   =((pobj)->prop     &   \
                                                  0xFFF800FF     ) |   \
                                                 ((UINT)(n) <<  8)     ;

#define HOBJ_SET_DIM(n,pobj)     (pobj)->prop   =((pobj)->prop     &   \
                                                  0xFFFFFF00     ) |   \
                                                  (UINT)(n)            ; 

/*
    2    Structures and typedefs
    ____________________________
*/


/* Structure de description d'un heap .......................................*/
                         
typedef struct                         /* HEAP                               */
  BEGIN                                /* ---------------------------------- */
    INT             nom              ; /* Identifiant nom du heap.           */
    INT             nbo              ; /* Compteur nombre d'objet.           */
    INT             typ              ; /* 0/1 : HEAP_DIRECT/HEAP_DATABASE    */
  END               Hhpf_ctx         ; /* ---------------------------------- */

typedef struct                         /* CONTEXTE DE STREAM                 */
  BEGIN                                /* ---------------------------------- */
    INT             hpf              ; /* Identifiant du heap de la stream   */
    INT             oidc             ; /* Dernier  OID   renvoye             */
    INT             oidn             ; /* Prochain OID a renvoyer            */
    INT             clic             ; /* Idx classe stock last OID renvoye  */
    INT             clin             ; /* Idx classe stock OID a renvoyer    */
    INT             nb               ; /* Nombre d'OIDs renvoyes             */
    UCHAR           type             ; /* STREAM_CLASS/STREAM_FILTER/...     */
    UCHAR           sens             ; /* Sens deplacement FORWARD/BACKWARD  */
    UCHAR           des              ; /* Identifiant du contexte prive      */
    IWORD           lg               ; /* Longueur donnees privees de descr  */
    UCHAR          *data             ; /* Donnees privees de description     */
  END               Hstr_ctx         ; /* ---------------------------------- */

typedef struct                         /* DESCRIPTION DE STREAM              */
  BEGIN                                /* ---------------------------------- */
    USHORT         *clid             ; /* classes + donnees privees          */
    INT             nbcl             ; /* Nombre de classes concernees       */
    UCHAR           type             ; /* STREAM_FILTER/STREAM_CLASS/...     */
  END               Hstr_des         ; /* ---------------------------------- */

typedef struct                         /* FILTRE SUR UNE CLASSE              */
  BEGIN                                /* ---------------------------------- */
    INT             offt             ; /* Index du premier filtre            */
    INT             nbfl             ; /* Nombre de filtres                  */
  END               Hflt_obj         ; /* ---------------------------------- */

typedef struct                         /* FILTRE SUR UNE VARIABLE            */
  BEGIN                                /* ---------------------------------- */
    INT             offt             ; /* Offset    valeur dans le buffer    */
    USHORT          lg               ; /* Longueur  valeur dans le buffer    */
    USHORT          masc             ; /* Identifiant classe de var de mask  */
    USHORT          masv             ; /* Identifiant           var de mask  */
    UCHAR           cond             ; /* Condition sur valeur dans le buffer*/
  END               Hflt_cnd         ; /* ---------------------------------- */

typedef struct                         /* OIDS MASK/REFERENCE POUR UN FILTRE */
  BEGIN                                /* ---------------------------------- */
    INT             oidv             ; /* OID objet stockage des valeurs     */
    INT             oidm             ; /* OID objet stockage des masks       */
    UCHAR          *adrv             ; /* Adresse des donnees de l'objet oidv*/
    UCHAR          *adrm             ; /* Adresse des donnees de l'objet oidm*/
  END               Hflt_oid         ; /* ---------------------------------- */

/* Description d'un objet, entete de zone de stockage de l'objet ----------- */

typedef struct                         /* Objet.                             */
  BEGIN                                /* ---------------------------------- */
    UINT            prop             ; /* Les proprietes de gestion/ la dim. */
    USHORT          cla_typ          ; /* Le type et la classe.              */
    USHORT          spe              ; /* Champ specifique.                  */
  END               Hhpf_obj         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                /* HHPF_H                             */
