/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/i2c_stm32.h"           */
/*  Declaration of the procedures of the I2Cs'.                              */
/*                                                                           */
/*  Author : ADE                        Last modification : 10/01/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef I2C_STM32_H                    /* If this file is not already        */
#define I2C_STM32_H                    /* included, include it.              */




/*
    1    Defines
    ____________
*/



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parameters of the driver I2C       */
  BEGIN                                /* ---------------------------------- */
    UCHAR          i2c_num           ; /* Number of the controller I2C       */
    UINT           fbus              ; /* SCL frequency                      */
  END              I2c_stm32_def     ; /* ---------------------------------- */

typedef struct                         /* Low-level context                  */
  BEGIN                                /* Constant values ------------------ */
    IWORD          lg                ; /* Length of the tx/rx buffer         */
    IWORD          nb                ; /* Number of tx/rx bytes              */
    UCHAR          state             ; /* Automaton state                    */
    UCHAR          flags             ; /* Flags used for the current put/get */
    UCHAR          i2c_num           ; /* I2C number (1, 2 or 3)             */
    UCHAR          adr               ; /* Address I2C                        */
    UCHAR         *buf               ; /* Address of the tx/rx buffer        */
    volatile I2c  *reg               ; /* I2C registers base address         */
  END              I2c_stm32         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const I2c_desc i2c_desc_stm32 ; /* I2C low level interface            */


/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD handlerevI2C_stm32               (IWORD,IWORD                          );
IWORD handlererrI2C_stm32              (IWORD,IWORD                          );


#endif                                 /* End of I2C_STM32_H                 */
