/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/mem_seg.h"             */
/*  Constants and definitions for mem_seg manager.                           */
/*                                                                           */
/*  Author : OP                         Last modification : 10/04/2001       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef MEM_SEG_H                      /* If this file is not already        */
#define MEM_SEG_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Flags d'allocation .......................................................*/

                                       /* Ce flag est aussi utilise dans     */
                                       /* memsmall.c. A ne pas modifier sans */
                                       /* avoir verifie dans memsmall.c      */

#define MEM_ZERO             128       /* Initialisation de bloc a 0.        */


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    3    Public variables
    _____________________
*/


/* Structure de description d'un segment HIGH_SEG ****************************/

typedef struct struct_high_des         /* Description d'un segment HIGH_SEG. */
  BEGIN                                /*                                    */
   UCHAR                    *adr   ;   /* Adresse du segment.                */
   INT                       size  ;   /* Taille du segment.                 */
   struct struct_high_des   *next  ;   /* Suivant dans la liste chainee.     */
   struct struct_high_des   *prev  ;   /* Precedent dans la liste chainee.   */
  END High_des;                        /*                                    */

/* Structure de description d'un segment MID_SEG *****************************/
typedef struct                         /* Description d'un segment MID_SEG   */
  BEGIN                                /*                                    */
   UCHAR                    *next  ;   /* segment suivant dans la chaine     */
   UCHAR                    *prev  ;   /* segment precedent dans la chaine   */
   USHORT                    id    ;   /* identifiant du segment             */
   UCHAR                     user  ;   /* identifiant de l'utilisateur       */
   UCHAR                     alloue;   /* flag de segment alloue/libre       */
  END Desc_mid_seg;                    /*                                    */


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD init_memseg         (void                                            );
  IWORD alloc_low_seg       (UCHAR**,INT                                     );
  IWORD alloc_high_seg      (USHORT*,INT,IWORD                               );
  IWORD free_high_seg       (USHORT*                                         );
  IWORD alloc_mid_seg_user_id(IWORD,UCHAR*                                   );
  IWORD free_mid_seg_user_id(IWORD                                           );
  IWORD alloc_mid_seg       (IWORD,USHORT*,UCHAR**                           );
  IWORD free_mid_seg        (IWORD,IWORD                                     );
  void  print_seg           (void                                            );
  IWORD status_mem          (IWORD,CHAR*,INT,CHAR*,INT                       );
  IWORD alloc_sys_seg       (UCHAR**,IWORD,IWORD                             );
  IWORD realloc_sys_seg     (UCHAR**,IWORD                                   );
  IWORD free_sys_seg        (UCHAR**                                         );
  IWORD affich_bytes        (UCHAR* ,INT                                     );


#endif                                 /* End of MEM_SEG_H.                  */
