/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/memsmall.h"            */
/*  Constants and definitions for memsmall manager.                          */
/*                                                                           */
/*  Author : OP                         Last modification : 04/04/2001       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef MEMSMALL_H                     /* If this file is not already        */
#define MEMSMALL_H                     /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Definition des traitements de deplacement des zones MID_MEM ..............*/

#define    MEM_PROC_RIEN     1         /* Pas de traitement                  */
#define    MEM_PROC_DF       2         /* Traitement deplacement fichier     */
#define    MEM_PROC_FRAME    3         /* Traitement deplacement bloc FRAME  */
#define    MEM_PROC_SCRMEM   4         /* Traitement deplacement bloc CODE   */
#define    MEM_PROC_CPI      5         /* Traitement deplacement bloc CPI    */


/*
    2    Structures and typedefs
    ____________________________
*/


/* Structures ...............................................................*/

#define _DEBUG_MEMORY_                 /* Pour le moment.                    */

typedef struct                         /* Structure de description d'une     */
  BEGIN                                /* zone en memoire LOW_MEM.           */
   UCHAR           **ptr             ; /* Adr.   du pointeur d'allocation    */
   UINT              size            ; /* Taille de l'allocation             */
  END Low_zone                       ; /*                                    */
 
typedef struct low_free                /* Structure de description d'une     */
  BEGIN                                /* zone libre en memoire LOW_MEM.     */
   Low_zone          desc            ; /* Description de la zone             */
   struct low_free  *next            ; /* Zone libre suivante                */
   struct low_free  *prev            ; /* Zone libre precedente              */
  END Low_free                       ; /*                                    */

typedef struct high_free               /* Structure de description d'une     */
  BEGIN                                /* zone libre en memoire HIGH_MEM.    */
   struct high_free *next            ; /* Zone libre suivante                */
   struct high_free *prev            ; /* Zone libre precedente              */
  END High_free                      ; /*                                    */

/*****************************************************************************/
/* Macros de lecture/ecriture de nombres au format interne dans des zones    */
/* memoire au format MPEG (MOTOROLA).                                        */
/* Dans chaque macro, a represente l'adresse de la zone memoire au format    */
/* MPEG et v la valeur a lire ou ecrire.                                     */
/* les macros concernant les adresses fonctionnent uniquement avec des       */
/* adresses sur 4 octets.                                                    */

#define mgets(a,v) { UCHAR *_a=(UCHAR*)(a); v = (((USHORT)_a[0])<<8)+_a[1]; }
#define mgeti(a,v) { UCHAR *_a=(UCHAR*)(a); v = (((UINT)_a[0])<<24)+\
                      (((UINT)_a[1])<<16)+(((UINT)_a[2])<<8)+_a[3]; }
#define mgeta(a,v) { UCHAR *_a=(UCHAR*)(a); v = (UCHAR*) ((((UINT)_a[0])<<24)+\
                      (((UINT)_a[1])<<16)+(((UINT)_a[2])<<8)+_a[3]); }

#define mputs(a,v) { UCHAR *_a=(UCHAR*)(a); _a[0] = ((v)>>8)&0xff; \
                                            _a[1] = (v)&0xff; }
#define mputi(a,v) { UCHAR *_a=(UCHAR*)(a); _a[0] = ((v)>>24)&0xff; \
                                            _a[1] = ((v)>>16)&0xff; \
                                            _a[2] = ((v)>>8)&0xff; \
                                            _a[3] = (v)&0xff; }
#define mputa(a,v) { UCHAR *_a=(UCHAR*)(a); INT _v = (INT)(v); \
                                            _a[0] = ((_v)>>24)&0xff; \
                                            _a[1] = ((_v)>>16)&0xff; \
                                            _a[2] = ((_v)>>8)&0xff; \
                                            _a[3] = (_v)&0xff; }

/* Macros de lecture/ecriture de nombres au format interne dans des zones    */
/* memoire au format INTEL.                                                  */

#define igets(a,v) { UCHAR *_a=(UCHAR*)(a); v = (((USHORT)_a[1])<<8)+_a[0]; }
#define igeti(a,v) { UCHAR *_a=(UCHAR*)(a); v = (((UINT)_a[3])<<24)+\
                      (((UINT)_a[2])<<16)+(((UINT)_a[1])<<8)+_a[0]; }
#define igeta(a,v) { UCHAR *_a=(UCHAR*)(a); v = (UCHAR*) ((((UINT)_a[3])<<24)+\
                      (((UINT)_a[2])<<16)+(((UINT)_a[1])<<8)+_a[0]); }

#define iputs(a,v) { UCHAR *_a=(UCHAR*)(a); _a[1] = ((v)>>8)&0xff; \
                                            _a[0] = (v)&0xff; }
#define iputi(a,v) { UCHAR *_a=(UCHAR*)(a); _a[3] = ((v)>>24)&0xff; \
                                            _a[2] = ((v)>>16)&0xff; \
                                            _a[1] = ((v)>>8)&0xff; \
                                            _a[0] = (v)&0xff; }
#define iputa(a,v) { UCHAR *_a=(UCHAR*)(a); INT _v = (INT)(v); \
                                            _a[3] = ((_v)>>24)&0xff; \
                                            _a[2] = ((_v)>>16)&0xff; \
                                            _a[1] = ((_v)>>8)&0xff; \
                                            _a[0] = (_v)&0xff; }

/*
    3    Public variable
    ____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD init_memsmall       (void                                            );
  IWORD alloc_low_mem       (UCHAR**,INT,IWORD,CHAR*                         );
  IWORD realloc_low_mem     (UCHAR**,INT,CHAR*                               );
  IWORD free_low_mem        (UCHAR**                                         );
  IWORD set_low_mem         (UCHAR*,INT                                      );
  IWORD check_high_mem      (UCHAR**,INT                                     );
  IWORD alloc_high_mem      (UCHAR**,INT,IWORD                               );
  IWORD free_high_mem       (UCHAR *,INT,IWORD                               );
  IWORD status_memsmall     (CHAR*,INT                                       );
  IWORD check_mem           (CHAR*                                           );
  void  print_mem           (void                                            );


#endif                                 /* End of MEMSMALL_H                  */
