/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/pan.h"                 */
/*  Constants and definitions for panel manager.                             */
/*                                                                           */
/*  Author : SC/FA                      Last modification : 16/10/2009       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef PAN_H                          /* If this file is not already        */
#define PAN_H                          /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*****************************************************************************/
/* Classe PanTalk systemes:                                                  */
/*****************************************************************************/

#define PANEL_CLASS_FILENAME  "pancla:panel.cla"  /* Panel general attributes*/
#define WBACK_CLASS_FILENAME  "pancla:wback.cla"  /* Background panel widget */
#define BOX_CLASS_FILENAME    "pancla:box.cla"    /* Generic widget          */
#define CLIP_CLASS_FILENAME   "pancla:clip.cla"   /* Box clipping rectangle  */
#define DSCRL_CLASS_FILENAME  "pancla:box.cla"    /* Standard box.           */
#define CUI_CLASS_FILENAME    "pancla:widgtv.cla" /* Box with text parameters*/


/*****************************************************************************/
/* Codes d'erreur de l'emulateur 32 pour le gestionnaire de panel:           */
/*****************************************************************************/

#define E_BAD_REQ         -99          /*                                    */
#define E_LD_REG          -98          /*                                    */
#define E_NULL_OBJ        -97          /*                                    */


/*****************************************************************************/
/* Micro-instructions du language panel:                                     */
/*****************************************************************************/

#define IEPANEL_PREAMBULE   0          /* Prise ou reprise d'execution.      */
#define IEPANEL_PUSH_METHOD 1          /* Empilement d'une nouvelle methode. */
#define IEPANEL_POP_METHOD  2          /* Depilement vers une methode panel. */
#define IEPANEL_CONTINUE    3          /* Reprise interpretation EMU32       */
#define NB_REQ_MAX        256          /* Nombre d'entrees liste de requete  */


/*****************************************************************************/
/* Alias des noms de registres du gestionnaire de panel:                     */
/*****************************************************************************/

#define RTEMP1             R0          /*                                    */
#define RTEMP2             R1          /*                                    */
#define RTEMP3             R2          /*                                    */
#define RTEMP4             R3          /*                                    */
#define RTEMP5             R4          /*                                    */
#define WID_OID            R5          /*                                    */
#define PAN_ID             R6          /*                                    */
#define CLIP_OID           R7          /*                                    */
#define AD_WID             R8          /*                                    */
#define AD_PAN             R9          /*                                    */
#define AD_CLIP           R10          /*                                    */
#define EVT_CODE          R11          /*                                    */
#define EVT_RES           R12          /*                                    */
#define EVT_AD            R13          /*                                    */
#define EVT_LG            R14          /*                                    */
#define WMASK             R15          /*                                    */
#define PMASK             R16          /*                                    */
#define X                 R17          /*                                    */
#define Y                 R18          /*                                    */
#define W                 R19          /*                                    */
#define H                 R20          /*                                    */
#define RET               R21          /*                                    */
#define TRAIT             R22          /*                                    */
#define KFOC_PAN          R23          /*                                    */
#define MFOC_PAN          R24          /*                                    */
#define KFOC_SAISIE       R25          /*                                    */
#define KFOC_VALID        R26          /*                                    */
#define MFOC_WID          R27          /*                                    */
#define EVT_ECODE         R28          /*                                    */
#define IDX               R29          /*                                    */
#define ELT               R30          /*                                    */
#define ATT               R31          /*                                    */
#define VAL               R32          /*                                    */
#define SP_SAISIE         R33          /*                                    */
#define OIDS              R34          /*                                    */
#define SP_VALID          R35          /*                                    */
#define OIDV              R36          /*                                    */
#define AUT               R37          /*                                    */
#define LAF               R38          /*                                    */
#define WHAT              R39          /*                                    */
#define ROLD              R40          /*                                    */
#define REVT              R41          /*                                    */
#define RNEW              R42          /*                                    */
#define MET               R43          /*                                    */
#define ETAT              R44          /*                                    */
#define SP_MOUSE          R45          /*                                    */
                                       /*                                    */
#define STRICT         RTEMP5          /*                                    */
#define WID_NOM       WID_OID          /*                                    */
#define X1                  X          /*                                    */
#define X2                  W          /*                                    */
#define Y1                  Y          /*                                    */
#define Y2                  H          /*                                    */
#define TEXT           RTEMP5          /*                                    */
#define VAR            RTEMP5          /*                                    */
#define GROUP          RTEMP5          /*                                    */
#define CODE           RTEMP5          /*                                    */


/*****************************************************************************/
/* Noms de sous-programmes du programme interprete du gestionnaire:          */
/*                                                                           */
/* Attention, le gestionnaire de panels a besoin que les noms de sous        */
/* programme de WID_EVT2 a SET_WID_DYN2 soit dans le meme ordre que WID_EVT  */
/* a SET_WID_DYN. De plus cet ordre est utilise par le tableau liste_req qui */
/* sert a charger les registres de l'emulateur avec les contenu des requetes.*/
/* Tout changement dans ce fichier implique donc de changer "liste_req"      */
/* du module /vmos/panel/panel.c                                             */
/*****************************************************************************/

#define SPAN_EVT            0          /*                                    */
#define PAN_EVT             1          /*                                    */
#define APAN_EVT            2          /*                                    */
#define KBD_SELECT          3          /*                                    */
#define MSE_SELECT          4          /*                                    */
#define EXT_SELECT          5          /*                                    */
#define REFRESH_VAR         6          /*                                    */
#define GIVE_KBD_FOC        7          /*                                    */
#define WID_EVT             8          /*                                    */
                                       /*                                    */
#define INIT_WID            9          /*                                    */
#define END_WID            10          /*                                    */
#define REFRESH_WID        11          /*                                    */
#define SET_WID_ATT        12          /*                                    */
#define SET_WID_TEXT       13          /*                                    */
#define SET_WID_VAR        14          /*                                    */
#define SET_WID_STYLE      15          /*                                    */
#define SET_WID_SURFACE    16          /*                                    */
#define SET_WID_DIM        17          /*                                    */
#define SET_WID_PARAM      18          /*                                    */
#define SET_WID_GROUP      19          /*                                    */
#define SET_WID_DYN        20          /*                                    */
                                       /*                                    */
#define INIT_SPANEL        21          /*                                    */
#define END_SPANEL         22          /*                                    */
#define INIT_PANEL         23          /*                                    */
#define END_PANEL          24          /*                                    */
#define SET_PANEL_ATT      25          /*                                    */
#define SET_PANEL_PAGE     26          /*                                    */
#define HIDE_PANEL         27          /*                                    */
#define SHOW_PANEL         28          /*                                    */
#define LOWER_PANEL        29          /*                                    */
#define RAISE_PANEL        30          /*                                    */
#define SET_PANEL_SIZE     31          /*                                    */
#define SET_PANEL_POS      32          /*                                    */
#define SET_PANEL_ORIG     33          /*                                    */
#define REFRESH_PANEL      34          /*                                    */
#define SET_KFOCUS         35          /*                                    */
#define SET_MFOCUS         36          /*                                    */
                                       /*                                    */
#define SET_KFOCUS2        37          /*                                    */
#define SET_MFOCUS2        38          /*                                    */
#define WID_EVT2           39          /*                                    */
#define INIT_WID2          40          /*                                    */
#define END_WID2           41          /*                                    */
#define REFRESH_WID2       42          /*                                    */
#define SET_WID_ATT2       43          /*                                    */
#define SET_WID_TEXT2      44          /*                                    */
#define SET_WID_VAR2       45          /*                                    */
#define SET_WID_STYLE2     46          /*                                    */
#define SET_WID_SURFACE2   47          /*                                    */
#define SET_WID_DIM2       48          /*                                    */
#define SET_WID_PARAM2     49          /*                                    */
#define SET_WID_GROUP2     50          /*                                    */
#define SET_WID_DYN2       51          /*                                    */
                                       /*                                    */
#define GIVE_KBD_FOCUS     52          /*                                    */
#define GESTPAN            53          /*                                    */
#define CHK_FOCUS          54          /*                                    */
#define REFRESH_ALL        55          /*                                    */
#define VIDE               56          /*                                    */


/*****************************************************************************/
/* Mnemoniques des Clippings:                                                */
/*****************************************************************************/

#undef MIDDL_                          /*                                    */
                                       /*                                    */
#define LEFT           0x0100          /*                                    */
#define RIGHT          0x0200          /*                                    */
#define UP             0x0004          /*                                    */
#define DOWN           0x0008          /*                                    */
#define MIDDL_         0x0010          /*                                    */
#define MIDG           0x1000          /*                                    */
#define MM  (MIDG  OR MIDDL_)          /*                                    */
#define LU  (LEFT  OR UP    )          /*                                    */
#define RD  (RIGHT OR DOWN  )          /*                                    */


/*****************************************************************************/
/* Modes d'insertion des zones de rafraichissement dans la liste des zones:  */
/*****************************************************************************/

#define NO_INTERS           0          /*                                    */
#define IS_SMALLER          1          /*                                    */
#define IS_BIGGER           2          /*                                    */
#define INTERS              3          /*                                    */


/*****************************************************************************/
/* Definitionas des attributes de styles (MHEG5)                             */
/*****************************************************************************/

#define ATT_MASK_FONT_NUM          0x00000001
#define ATT_MASK_FONT_SIZE         0x00000002
#define ATT_MASK_CHAR_COLOR        0x00000004
#define ATT_MASK_BCHAR_COLOR       0x00000008
#define ATT_MASK_CHAR_STYLE        0x00000010
#define ATT_MASK_HIGH_INDENT       0x00000020
#define ATT_MASK_LOW_INDENT        0x00000040
#define ATT_MASK_TEXT_JUSTIF       0x00000080
#define ATT_MASK_BULLET_NUM        0x00000100
#define ATT_MASK_LEFT_INDENT       0x00000200
#define ATT_MASK_RIGHT_INDENT      0x00000400
#define ATT_MASK_TAB0_INDENT       0x00000800
#define ATT_MASK_TAB1_INDENT       0x00001000
#define ATT_MASK_TAB2_INDENT       0x00002000
#define ATT_MASK_TAB3_INDENT       0x00004000
#define ATT_MASK_TAB4_INDENT       0x00008000
#define ATT_MASK_TAB5_INDENT       0x00010000
#define ATT_MASK_TAB6_INDENT       0x00020000
#define ATT_MASK_TAB7_INDENT       0x00040000
#define ATT_MASK_TAB8_INDENT       0x00080000
#define ATT_MASK_TAB9_INDENT       0x00100000
#define ATT_MASK_BOX_COLOR         0x00200000
#define ATT_MASK_FRAME_STYLE       0x00400000
#define ATT_MASK_FRAME_COLOR1      0x00800000
#define ATT_MASK_FRAME_COLOR2      0x01000000
#define ATT_MASK_FRAME_THICK1      0x02000000
#define ATT_MASK_FRAME_THICK2      0x04000000
#define ATT_MASK_HIGHLIGHT         0x08000000
#define ATT_MASK_MOUSE_SHAPE       0x10000000
#define ATT_MASK_SHADOW_TYPE       0x08000000
#define ATT_MASK_SHADOW_COLOR      0x10000000
#define ATT_MASK_ALL               0x1FFFFFFF


/*
    2    Structures et typedefs
    ___________________________
*/

/*****************************************************************************/
/* Type utilise par le gestionnaire de panel.                                */
/*****************************************************************************/

typedef CHAR  Map_req[8]             ; /*                                    */

typedef struct            /* Structure de description des parametres des     */
 BEGIN                    /* traitements de transitions d'un widget          */
  INT       filter      ; /* Filtre sur le champ reserve de l'evenement      */
  SHORT     scrname     ; /* Index de la meth de script dans Panel.list_val  */
  CHAR      pad[2]      ; /* sizeof(Panel) % 4 vaut toujours 0.              */
 END NBoxpara           ; /*                                                 */

typedef struct            /* Structure de description des parametres des     */
 BEGIN                    /* traitements de transitions d'un widget          */
  INT       traitement  ; /* Adresse du code de la methode du gestionnaire.  */
  UINT      varg[8]     ; /* Arguments de la requete.                        */
  UINT      targ        ; /* Type de chacun des arguments.                   */
  INT       next        ; /* Requete suivante.                               */
  INT       prev        ; /* Requete precedente.                             */
  UCHAR     fifo        ; /* Numero de la FIFO de la requete.                */
  UCHAR     confirm     ; /* Flag de confirmation de requete demandee.       */
 END NPanreq            ; /*                                                 */

typedef struct            /* Structure de description d'un panel             */
 BEGIN                    /*                                                 */
  NBoxpara *list_lnk    ; /* Liste des parametres de traitements             */
  INT      *list_val    ; /* Identifiants de methode de script               */
  INT       panel_id    ; /* Identifiant du panel                            */
  INT       hypfile     ; /* Identifiant du fichier memoire du panel         */
  INT       panel       ; /* OID de l'objet panel                            */
  INT       wbkgr       ; /* OID du premier objet box du chainage            */
  INT       wlast       ; /* OID du dernier objet box du chainage            */
  INT       index       ; /* Index table de tests pour les evenements        */
  INT       clip        ; /* OID du premier rectangle de clipping.           */
  USHORT    modules[4]  ; /* Identifiants modules des panels et sous-panels  */
  SHORT     list_lnk_use; /* Nombre d'entrees utilisees de list_lnk          */
  SHORT     list_lnk_all; /* Nombre d'entrees allouees  de list_lnk          */
  SHORT     list_val_use; /* Nombre d'entrees utilisees de list_val          */
  SHORT     list_val_all; /* Nombre d'entrees allouees  de list_val          */
  CHAR      relaunch_auto;/* Automate     a relancer a la mort du panel      */
  CHAR      relaunch_vl ; /* Voie logique a relancer a la mort du panel      */
  CHAR      pad[2]      ; /* sizeof(Panel) % 4 vaut toujours 0.              */
 END NPanel             ; /*                                                 */

typedef struct            /* Variables globales du panel                     */
 BEGIN                    /*                                                 */
  INT       thread      ; /* Numero du thread panel                          */
  INT       script      ; /* Ensemble du script gestionnaire de panel        */
  INT       rteid       ; /* Identifiant de la route des evts externes       */
  INT       confirm     ; /* Flag de confirmation de requete a envoyer 0/1   */
  INT       requete     ; /* Numero de la requete a confirmer                */
  INT       error       ; /* Code d'erreur de la requete a confirmer         */
  INT       code        ; /* Code de l'evenement declencheur                 */
  NPanreq  *req         ; /* Adresse de la liste des requetes.               */
  INT       req_use     ; /* Nombre d'entrees utilisees de la liste          */
  INT       req_all     ; /* Nombre d'entrees allouees  de la liste          */
  INT       req_cur     ; /* Numero d'entree  de la requete en cours         */
  INT       fifo[3]     ; /* Pointeur des FIFOs.                             */
 END NPangl             ; /*                                                 */


/*****************************************************************************/
/* Structures C qui decrivent les classes utilisees par le gestionnaire de   */
/* panels. Les champs de ces structures sont paddes pour correspondrent a    */
/* l'alignement utilise par le module classe. Ces structures ont donc une    */
/* taille independante de la machine sur laquelle elles sont utilisees.      */
/* Attention, les champs marques d'un X sont des champs dont le type est     */
/* choisie pour que la classe puisse etre cree a l'aide de l'editeur de      */
/* du workshop qui ne permet pas de creer des variablede type CHAR/SHORT.  */
/*****************************************************************************/

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "panel".                                 */
  INT       panel_id     ;/* Identifiant du panel                            */
  INT       father       ;/* Identifiant du panel pere de ce panel           */
  INT       nom          ;/* Identifiant du nom du panel                     */
  INT       wbkgr        ;/* OID du premier objet box du chainage            */
  INT       wlast        ;/* OID du dernier objet box du chainage            */
  INT       wkeyfoc      ;/* OID de l'objet box qui a le focus clavier       */
  INT       wvalfoc      ;/* OID du l'objet box qui a le focus validation    */
  INT       wmsefoc      ;/* OID du l'objet box qui a le focus souris        */
  INT       evt_widget   ;/* OID de l'objet box du widget traite             */
  INT       evt_req      ;/* Numero de la requete qui envoie l'evenement     */
  INT       evt_code     ;/* Code recu par le gestionnaire de panels         */
  INT       evt_ecode    ;/* Code emis au widget traite                      */
  INT       evt_filter   ;/* Champ reserve de l'evenement                    */
  UCHAR    *evt_ad       ;/* Adresse du buffer associe                       */
  INT       evt_longueur ;/* Longueur du buffer associe                      */
  INT       evt_x        ;/* Abscisse decodee de la souris                   */
  INT       evt_y        ;/* Ordonnee decodee de la souris                   */
  INT       evt_w        ;/* Fenetre sous la souris                        X */
  INT       att          ;/* VISIBLE/ENABLE                                X */
  INT       priority     ;/* Niveau de priorite du panel                   X */
  INT       screen       ;/* Ecran de la fenetre du panel                  X */
  INT       window       ;/* Identifiant de la fenetre du panel            X */
  INT       wmin         ;/* Largeur minimum du panel                      X */
  INT       wmax         ;/* Largeur maximum du panel                      X */
  INT       hmin         ;/* Hauteur minimum du panel                      X */
  INT       hmax         ;/* Hauteur maximum du panel                      X */
 END NObj_panel          ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "clip".                                  */
  INT        x,y,w,h     ;/* description du rectangle de clipping.           */
  INT        next        ;/* Oid du rectangle suivant.                       */
 END NObj_clip           ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "box".                                   */
  INT       nom          ;/* Identifiant du nom du widget                    */
  INT       panel_id     ;/* Identifiant du sous-panel du widget             */
  INT       wprev        ;/* OID objet "box" du widget precedent             */
  INT       wnext        ;/* OID objet "box" du widget suivant               */
  INT       wvprev       ;/* OID objet "box" du widget visible precedent     */
  INT       wvnext       ;/* OID objet "box" du widget visible suivant       */
  INT       wasso        ;/* OID objet "box" du widget associe               */
  INT       wback        ;/* OID objet "box" du fond du widget               */
  INT       pages_mask1  ;/* Flag de presence dans chacune des pages   0- 31 */
  INT       pages_mask2  ;/* Flag de presence dans chacune des pages  32- 63 */
  INT       pages_mask3  ;/* Flag de presence dans chacune des pages  64- 95 */
  INT       group        ;/* Groupe du widget                              X */
  INT       priority     ;/* Priorite par rapport au panel                 X */
  INT       key_ctx      ;/* Numero du contexte clavier du widget          X */
  INT       text         ;/* Id de la valeur contenant le texte du widget    */
  INT       var_elt      ;/* Identifiant de la variable associee au widget   */
  INT       var_idx      ;/* Index dans la variable associee si tableau    X */
  INT       att          ;/* Champ de bits                                   */
  INT       stycurind    ;/* Identifiant du style courant du widget          */
  INT       stynxtind    ;/* Style suivant apres fin de traitement           */
  INT       tratabind    ;/* Identifiant de la table de transitions          */
  INT       stytabind    ;/* Identifiant de la table de transcodage          */
  INT       spec_para    ;/* Offset 1er param specifique des transitions   X */
  INT       etat         ;/* Etat du widget                                X */
  INT       next_etat    ;/* Etat suivant apres fin de traitement          X */
  INT       x1           ;/* Abscisse CSG                                  X */
  INT       y1           ;/* Ordonnee CSG                                  X */
  INT       x2           ;/* Abscisse CID                                  X */
  INT       y2           ;/* Ordonnee CID                                  X */
 END NObj_box            ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "fond"                                   */
  NObj_box   box         ;/* Super-classe = "box"                            */
  INT        class       ;/* Identifiant   de la classe de l'objet associe   */
  INT        oid         ;/* OID du buffer de stockage  de l'objet associe   */
  INT        dict_id     ;/* Identifiant du dictionnaire des textes          */
  INT        language    ;/* Langue chargee pour ce fond                   X */
  INT        page        ;/* Numero de la page courante de ce fond         X */
  INT        dx          ;/* Abscisse d'offset pour l'affichage            X */
  INT        dy          ;/* Ordonnee d'offset pour l'affichage            X */
  INT        style       ;/* Ensemble style associe a ce fond                */
  INT        laf         ;/* Ensemble LAF   associe a ce fond                */
  INT        spanel_id   ;/* Numero du sous-panel contenu                    */
  INT        icolib      ;/* Identifiant de la librairie d'icones            */
 END NObj_wback          ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "hlnk".                                  */
  NObj_box   box         ;/* Super-classe = "box"                            */
  INT        dxt,dyt     ;/* Offset point haut.                              */
  INT        dxb,dyb     ;/* Offset point bas .                              */
  INT        pad0        ;/* Nb d'elements dans dynamique.                   */
  INT        dynamic[5]  ;/* Valeurs du dynamique.                           */
 END NObj_hlnk           ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "himg".                                  */
  NObj_box   box         ;/* Super-classe = "box"                            */
  INT        w,h         ;/* Largeur et hauteur de l'image.                  */
  INT        xoff,yoff   ;/* Offset d'affichage.                             */
  INT        tiling      ;/* Option de replicage                             */
  INT        typ         ;/* Type de l'image  ge                             */
 END NObj_himg           ;/*                                                 */

typedef struct            /* Structure de description d'un objet de la       */
 BEGIN                    /* classe "html"                                   */
  NObj_box   box        ; /* Description generale du widget                  */
  INT        lcmds      ; /* Liste des commandes a executer par evenement    */
  INT        lgraphs    ; /* Transition graphique par evenement              */
 END NObj_widgtv        ; /*                                                 */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of PAN_H.                      */
