/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/rdrv.h"                */
/*  Registers and tags for drivers/protocols/services.                       */
/*                                                                           */
/*  Author : JYA                        Last modification : 31/01/2012       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variable
         4  Prototypes of public procedures


*/


#ifndef RDRV_H                         /* If this file is not already        */
#define RDRV_H                         /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvsys/drv_usb.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

/* Numero des registres .....................................................*/

#define NRUSBSTATE     0                              /* TAG_ENUM            */
#define NRUSBTYPE     (NRUSBSTATE   + 1)              /* TAG_ENUM            */
#define NBUSBREG      (NRUSBTYPE    + 1)              /* Nombre de registres */


/* Valeurs des registres TAG_ENUM ...........................................*/

#define USBSTATE_UNP       0                          /* TAG_ENUM "STATE"    */
#define USBSTATE_UMS      (USBSTATE_UNP +1)           /* TAG_ENUM "STATE"    */
#define USBSTATE_OTH      (USBSTATE_UMS +1)           /* TAG_ENUM "STATE"    */

#define T_INTERNAL         0                          /* TAG_ENUM "TYPE"     */
#define T_EXTERNAL         1                          /* TAG_ENUM "TYPE"     */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/drv_dmx.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres ...........................................*/

/* Numeros des registres globaux (indices dans dmx_ctx->reg[])...............*/

#define NRDMXTYPE       0              /* tag TYPE                           */
#define NRDMXNAME       1              /* tag NAME            ( 3 registres) */
#define NRNBPID         4              /* tag NBPID NBKEY                    */
#define NRFREE          5              /* tag FREE                           */
#define NRTRPPIDS       6              /* tag TRPPIDS         (17 registres) */
#define NRTRPUPIDS      23             /* tag TRPUPIDS        (17 registres) */
#define NRTRPSIDS       40             /* tag TRPSIDS         ( 3 registres) */
#define NRTRPTDVB       43             /* tag TRPONID TRPTSID                */
#define NRTRPNBPKT      44             /* tag TRPNBPKT                       */
#define NRNBSRC         45             /* tags FRE_NBSRC et MEM_NBSRC        */
#define NRSRC           46             /* tag SRC                            */
#define NRDMXSPEED      47             /* tag SPEED                          */
#define NRDMXBITRATE    48             /* tag BITRATE                        */
#define NRPATLITE       49             /* tag PATLITE                        */
#define NRSEGTO         50             /* tag SEG_TO                         */
#define NRSCRKEY        51             /* tag SCR_KEY         ( 4 registres) */
#define NRSCRIV         55             /* tag SCR_IV          ( 4 registres) */
#define NRSCRIVMODE     59             /* tags SCR_IVMODE et SCR_IVCHAIN.    */

#define MSK_IVMODE      0x00000003     /* Masque registre pour tag SCR_IVMODE*/
#define MSK_IVCHAIN     0x00000300     /* Masque registre pour tag SCR_IVCH. */
#define SFT_IVMODE      0              /* Shift registre pour tag SCR_IVMODE */
#define SFT_IVCHAIN     8              /* Shift registre pour tag SCR_IVCHAIN*/


/* Numeros des registres de filtres PID (indices dans flt_ctx->reg[])........*/

#define OF7_RFLT       (NRSCRIVMODE+1) /* Numero 1er registre filtre PID.    */

#define NRBUFSIZE       0              /* tag BUFSIZE                        */
#define NRMODF          1              /* tag MODE                           */
#define NRFSTAT         2              /* tag STATE                          */
#define NRPID           3              /* tag PID                            */
#define NRTO            4              /* tag TIMEOUT                        */
#define NRSCRAM         5              /* tag SCRAMBLED                      */
#define NRRSTK          6              /* tag RESETKEYS                      */
#define NRKEYO          7              /* tag KEY_ODD                        */
#define NRKEYE          15             /* tag KEY_EVEN                       */
#define NRKEYTYP        23             /* tag KEY_TYPE                       */
#define NROUTSTA        24             /* tag OUT_STATE                      */
#define NRVPS           25             /* tag VBI_VPS                        */
#define NRWSS           29             /* tag VBI_WSS                        */
#define NRNBFSEC        30             /* tag NBFSEC                         */
#define NRNUMFSEC       31             /* tag NUMFSEC                        */
#define NRSID           32             /* tag STREAMID                       */
#define NRMAXBUF        33             /* tags MAXBUF + NBBUF                */
#define NRSUBID         34             /* tag SUBID                          */
#define NREITPAR        35             /* tags PFACTUAL, PFOTHER, SCHEDULE.  */
#define NRRSTD          36             /* tag RESETDATA                      */
#define NRVIDFMT        37             /* tag VIDFORMAT                      */
#define NRBUFPART       38             /* tag BUFPART                        */
#define NRDST_IOD       39             /* tag DST_IOD                        */
#define NRTTL           40             /* tag TTL                            */
#define NRNBSSF         41             /* tag NBSF                           */
#define NRSSFVAL        42             /* tag SSFVAL                         */
#define NRSSFMSK        58             /* tag SSFMSK                         */
#define NRLOCK          74             /* tag LOCK                           */

#define NBRSSFVAL       14             /* Nombre de registres SSFVAL         */
#define NBRSSFMSK       NBRSSFVAL      /* Nombre de registres SSFMSK         */

#define MSK_NBPID       0x001F0000     /* Masque registre pour tag NBPID     */
#define MSK_NBKEY       0x00001F00     /* Masque registre pour tag NBKEY     */
#define MSK_FREE        0x000000FF     /* Masque registre pour tag FREE      */
#define MSK_SIZE        0x00000003     /* Masque registre pour tag BUFSIZE   */
#define MSK_MODE        0x000000FF     /* Masque registre pour tag MODE      */
#define MSK_STATE       0x00000003     /* Masque registre pour tag STATE     */
#define MSK_PID         0x00001FFF     /* Masque registre pour tag PID       */
#define MSK_SCRAM       0x00000001     /* Masque registre pour tag SCRAMBLED */
#define MSK_RSTK        0x00000001     /* Masque registre pour tag RESETKEYS */
#define MSK_TO          0xFFFFFFFF     /* Masque registre pour tag TO        */
#define MSK_OUTSTA      0x00000007     /* Masque registre pour tag OUT_STATE */
#define MSK_NBFSEC      0x000000FF     /* Masque registre pour tag NBFSEC    */
#define MSK_NUMFSEC     0x000000FF     /* Masque registre pour tag NUMFSEC   */
#define MSK_SID         0x000001FF     /* Masque registre pour tag STREAMID  */
#define MSK_STMTYP      0x00FF0000     /* Masque registre pour tag STM_TYPE  */
#define MSK_MAXBUF      0x000000FF     /* Masque registre pour tag MAXBUF    */
#define MSK_NBBUF       0x0000FF00     /* Masque registre pour tag NBBUF     */
#define MSK_STLPAGE     0x0000FFFF     /* Masque registre pour tag STLPAGE   */
#define MSK_STLANC      0x0000FFFF     /* Masque registre pour tag STLANC    */
#define MSK_BUFPART     0x00000003     /* Masque registre pour tag BUFPART   */
#define MSK_KEYTYP      0x00000003     /* Masque registre pour tag KEYTYPE   */
#define MSK_NBSSF       0x0000000F     /* Masque registre pour tag NBSSF     */

#define SFT_STMTYP      16             /* Shift registre pour tag STM_TYPE   */

/* Numeros des registres de filtres de section (indices dans fsec_ctx->reg[])*/
 
#define OF7_RSEC (OF7_RFLT+NRLOCK+1)   /* Numero du 1er reg de flt de section*/

#define NRFSEC          0              /* tag CRC USEFSEC                    */
#define NRTID           1              /* tag TID TIDEXT                     */
#define NRSN            2              /* tag SN                             */
#define NRVN            3              /* tag VN                             */
#define NRVALUE         4              /* tag VALUE                          */
#define NRMASK          6              /* tag MASK                           */
#define NRMASKN         8              /* tag MASKNEG                        */
#define NRTIDEXTS      10              /* tag TIDEXTS                        */
#define NRXIDVAL       11              /* tag XID_VAL                        */
#define NRXIDMSK       13              /* tag XID_MSK                        */
#define NRDEPTH        15              /* tag DEPTH                          */
#define NRVALEXT       16              /* tag VALUEEXT                       */
#define NRMKPEXT       18              /* tag MASKEXT                        */
#define NRMKNEXT       20              /* tag MASKNEGEXT                     */
 
#define MSK_CRC         0x00000003     /* Masque de registre pour tag CRC    */
#define MSK_USEFSEC     0x00000100     /* Masque de registre pour tag USEFSEC*/
#define MSK_DEPTH       0x000001FF     /* Masque de registre pour tag DEPTH  */
#define MSK_TID         0xFF000000     /* Masque de registre pour tag TID    */
#define MSK_TIDEXT      0x0001FFFF     /* Masque de registre pour tag TIDEXT */


/* Valeurs des registres TAG_ENUM ...........................................*/

#define MVIDEODEC       0              /* Valeur registre pour mode VIDEODEC */
#define MAUDIODEC       1              /* Valeur registre pour mode AUDIODEC */
#define MPACKET         2              /* Valeur registre pour mode PACKET   */
#define MSECTION        3              /* Valeur registre pour mode SECTION  */
#define MSECTION_CHANGE 4              /* Valeur registre pour mode SECTION C*/
#define MTABLE          5              /* Valeur registre pour mode TABLE    */
#define MTABLE_CHANGE   6              /* Valeur registre pour mode TABLE CHA*/
#define MMASK           7              /* Valeur registre pour mode MASK     */
#define MMASK_ALL       8              /* Valeur registre pour mode MASK ALL */
#define MMASK_TOGGLE    9              /* Valeur registre pour mode MASK TOGG*/
#define MVBI           10              /* Valeur registre pour mode VBI      */
#define MTTX           11              /* Valeur registre pour mode TTX      */
#define MSTL           12              /* Valeur registre pour mode STL      */
#define MTABLES        13              /* Valeur registre pour mode TABLES   */
#define MEITS          14              /* Valeur registre pour mode EITS     */
#define MALA_ONESH     15              /* Valeur registre pour mode ALA_ONESH*/
#define MALA_TOGGL     16              /* Valeur registre pour mode ALA_TOGGL*/
#define MALA_CONT      17              /* Valeur registre pour mode ALA_CONT */
#define MPVR_VID       18              /* Valeur registre pour mode PVR_VID  */
#define MPVR_AUD       19              /* Valeur registre pour mode PVR_AUD  */
#define MPVR_VBI       20              /* Valeur registre pour mode PVR_VBI  */
#define MPVR_STL       21              /* Valeur registre pour mode PVR_STL  */
#define MPVR_SEC       22              /* Valeur registre pour mode PVR_SECT.*/
#define MPVR_PSM       23              /* Valeur registre pour mode PVR_PSM  */
#define MPVR_EITPF     24              /* Valeur registre pour mode PVR_SECT.*/
#define MDEC_VBI       25              /* Valeur registre pour mode DEC_VBI. */
#define MDEC_TTX       26              /* Valeur registre pour mode DEC_TTX. */
#define MTRP           27              /* Valeur registre pour mode TRP.     */
#define MPCR           28              /* Valeur registre pour mode PCR.     */
#define MPVR_PCR       29              /* Valeur registre pour mode PVR_PCR. */
#define MSGT           30              /* Valeur registre pour mode SGT.     */
#define MSSF_ALL       31              /* Valeur registre pour mode SSF_ALL. */
#define MTABLE_REF     32              /* Valeur registre pour mode TABLE_REF*/
#define MHLS           33              /* Valeur registre pour mode HLS.     */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/drv_smc.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres ...........................................*/

#define NBRSMCATR      9               /* Nombre de registres pour l'ATR.    */

/* Numero des registres .....................................................*/

#define NRSMCTYPE      0               /* TYPE                               */
#define NRSMCNAME      1               /* NAME                               */
#define NRSMCSTATE     4               /* STATE                              */
#define NRSMCRVV       5               /* RST VCC VPP                        */
#define NRSMCFCLK      6               /* FCLK                               */
#define NRSMCBAUDS     7               /* BAUDS                              */
#define NRSMCATR       8               /* ATR                                */
#define NRSMCRCIP      (NRSMCATR+NBRSMCATR) /* RESET_TYPE CONVENTION I P     */
#define NRSMCFDN       (NRSMCRCIP +1)  /* F D N                              */
#define NRSMCRCVTO     (NRSMCFDN  +1)  /* RCVTO                              */
#define NRSMCRESET     (NRSMCRCVTO+1)  /* RESET_CMD                          */
#define NRSMCTRY       (NRSMCRESET+1)  /* TRY_NB                             */
#define NBREGSMC       (NRSMCTRY  +1)  /* Nombre total de registres.         */


/* Valeurs des registres TAG_ENUM ...........................................*/

#define SMCST_EMPTY   0                /* Slot du lecteur est vide           */
#define SMCST_PTS     1                /* Echange PTS en cours               */
#define SMCST_T0      2                /* Initialise en T=0                  */
#define SMCST_T1      3                /* Initialise en T=1                  */
#define SMCST_MUTE    4                /* Echec de l'initialisation          */
#define SMCST_ERROR   5                /* Echec apres une initialisation OK  */

/* Numero des tags ..........................................................*/

#define NTSMCSTATE    2                /* Numero du tag "STATE"              */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/drv_pcm.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRPCMTYPE     1               /* TYPE, au max  3 cars + '\0'        */
#define NBRPCMNAME     4               /* NAME, au max 15 cars + '\0'        */
#define NBRPCMMANU     8               /* MANU, au max 31 cars + '\0'        */
#define NBRPCMPROD     8               /* PROD, au max 31 cars + '\0'        */
#define NBRPCMATASN    6               /* SERIALNB     20 cars + '\0'        */
#define NBRPCMATAFW    3               /* FW version    8 cars + '\0'        */
#define NBRPCMATAMOD   11              /* MODEL        40 cars + '\0'        */
#define NBRPCMATAUID   2               /* Unique ID, MEM 8 octets            */


/* Numero des registres .....................................................*/

#define NRPCMTYPE      0                              /* TAG_STR   0         */
#define NRPCMNAME     (NRPCMTYPE    +NBRPCMTYPE   )   /* TAG_STR   1         */
#define NRPCMSTATE    (NRPCMNAME    +NBRPCMNAME   )   /* TAG_ENUM  5         */
#define NRPCMRESET    (NRPCMSTATE   +0            )   /* TAG_ENUM  5         */
#define NRPCMMANU     (NRPCMRESET   +1            )   /* TAG_STR   6         */
#define NRPCMPROD     (NRPCMMANU    +NBRPCMMANU   )   /* TAG_STR   14        */
#define NRPCMMANFID   (NRPCMPROD    +NBRPCMPROD   )   /* TAG_ENUM  22        */
#define NBPCMREG      (NRPCMMANFID  +1            )   /* Nombre de registres */


/* Valeurs des registres TAG_ENUM ...........................................*/

#define PCMST_EMPTY    0                              /* TAG_ENUM "STATE"    */
#define PCMST_BUSY    (PCMST_EMPTY  +1)               /* TAG_ENUM "STATE"    */
#define PCMST_DVBCI   (PCMST_BUSY   +1)               /* TAG_ENUM "STATE"    */
#define PCMST_DISK    (PCMST_DVBCI  +1)               /* TAG_ENUM "STATE"    */
#define PCMST_UNK     (PCMST_DISK   +1)               /* TAG_ENUM "STATE"    */
#define PCMST_CIPLUS  (PCMST_UNK    +1)               /* TAG_ENUM "STATE"    */
#define PCMST_CIPMS   (PCMST_CIPLUS +1)               /* TAG_ENUM "STATE"    */
#define PCMST_INIT    (PCMST_CIPMS  +1)               /* TAG_ENUM "STATE"    */


/* Numero des tags ..........................................................*/

#define NTPCMTYPE      0                              /* "TYPE"              */
#define NTPCMNAME     (NTPCMTYPE    +1)               /* "NAME"              */
#define NTPCMSTATE    (NTPCMNAME    +1)               /* "STATE"             */
#define NTPCMRESET    (NTPCMSTATE   +1)               /* "RESET"             */
#define NTPCMMANU     (NTPCMRESET   +1)               /* "MANU"              */
#define NTPCMPROD     (NTPCMMANU    +1)               /* "PROD"              */
#define NTPCMMANFID   (NTPCMPROD    +1)               /* "MANFID"            */
#define NTPCMCARDID   (NTPCMMANFID  +1)               /* "CARDID"            */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvpro/pro_ci.c"                    *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRCITYPE         1            /* TYPE    , au max   3 cars + '\0'   */
#define NBRCINAME         4            /* NAME    , au max  15 cars + '\0'   */
#define NBRCIANSWER      11            /* ANSWER  , au max  43 cars + '\0'   */
#define NBRCIAPPNAME      8            /* APP     , au max  31 cars + '\0'   */
#define NBRCICASYSID      9            /* SYSID   , au max 8+1 tranches CAId */
#define NBRCITITLE       16            /* TITLE   , au max  63 cars + '\0'   */
#define NBRCISUBTITLE    16            /* SUBTITLE, au max  63 cars + '\0'   */
#define NBRCICONTENT     64            /* CONTENT , au max 255 cars + '\0'   */
#define NBRCIFOOTER      16            /* FOOTER  , au max  63 cars + '\0'   */
#define NBRCIKEYODD       8            /* KEYODD  , AES:16 oct CCK 16 oct CIV*/
#define NBRCIKEYEVEN      8            /* KEYEVEN , DES, 8 oct CCK, puis 0   */
#define NBRCIURIPRG       3            /* URIPRG  , PRGnum (2oct)suivi de URI*/
#define NBRCICOUNTRY      1            /* COUNTRY , code ISO pays            */
#define NBRCILANGUAGE     1            /* LANGUAGE, code ISO langue          */
#define NBRCITUNE         1            /* TUNE    , Ad des parametres        */
#define NBRCIAPMOPT       8            /* OPTION  , pour Application MMI (32)*/
#define NBRCIAPM         32            /* LOAD    , pour Application MMI(128)*/
#define NBRCIAPMFILE     32            /* FILE    , pour Application MMI(128)*/
#define NBRCIAPMDAT       1            /* DATA    , pour Application MMI (1) */
#define NBRCILICM         1            /* LICM    , Licence du module (ptr)  */
#define NBRCILICH         1            /* LICH    , Licence lue par le Host  */
#define NBRCILICSTAT      1            /* LICSTAT , Licence status+playcount */
#define NBRCIREC          1            /* REC     , Recording info           */
#define NBRCIPLA          1            /* PLA     , Playback info            */
#define NBRCIPINCAP       1            /* PINCAP  , PIN capabilities/Status  */
#define NBRCIPINM         1            /* PINM    , PIN info du module       */
#define NBRCIPINH         1            /* PINH    , retour des PIN info      */

/* Numero des registres .....................................................*/

#define NRCITYPE       0                              /*   0 TAG_STR         */
#define NRCINAME      (NRCITYPE     +NBRCITYPE     )  /*   1 TAG_STR         */
#define NRCIENUM      (NRCINAME     +NBRCINAME     )  /*   5 TAG_ENUM/INT    */
#define NRCIANSWER    (NRCIENUM     +1             )  /*   6 TAG_STR         */
#define NRCIAPPNAME   (NRCIANSWER   +NBRCIANSWER   )  /*  17 TAG_STR         */
#define NRCICASYSID   (NRCIAPPNAME  +NBRCIAPPNAME  )  /*  25 TAG_MEM         */
#define NRCITITLE     (NRCICASYSID  +NBRCICASYSID  )  /*  34 TAG_STR         */
#define NRCISUBTITLE  (NRCITITLE    +NBRCITITLE    )  /*  50 TAG_STR         */
#define NRCICONTENT   (NRCISUBTITLE +NBRCISUBTITLE )  /*  66 TAG_STR deporte */
#define NRCIFOOTER    (NRCICONTENT  + 1            )  /*  67 TAG_STR         */
#define NRCIKEYODD    (NRCIFOOTER   +NBRCIFOOTER   )  /*  83 TAG_MEM         */
#define NRCIKEYEVEN   (NRCIKEYODD   +NBRCIKEYODD   )  /*  91 TAG_MEM         */
#define NRCIURIPRG    (NRCIKEYEVEN  +NBRCIKEYEVEN  )  /*  99 TAG_MEM         */
#define NRCICOUNTRY   (NRCIURIPRG   +NBRCIURIPRG   )  /* 102 TAG_STR         */
#define NRCILANGUAGE  (NRCICOUNTRY  +NBRCICOUNTRY  )  /* 103 TAG_STR         */
#define NRCITUNE      (NRCILANGUAGE +NBRCILANGUAGE )  /* 104 TAG_INT         */
#define NRCIAPMOPT    (NRCITUNE     +NBRCITUNE     )  /* 105 TAG_STR         */
#define NRCIAPM       (NRCIAPMOPT   +NBRCIAPMOPT   )  /* 113 TAG_STR         */
#define NRCIAPMFILE   (NRCIAPM      +NBRCIAPM      )  /* 145 TAG_STR         */
#define NRCIAPMDAT    (NRCIAPMFILE  +NBRCIAPMFILE  )  /* 177 TAG_INT         */
#define NRCIENUM2     (NRCIAPMDAT   +NBRCIAPMDAT   )  /* 178 TAG_INT         */
#define NRCILICM      (NRCIENUM2    +1             )  /* 179 TAG_INT         */
#define NRCILICH      (NRCILICM     +NBRCILICM     )  /* 180 TAG_INT         */
#define NRCILICSTAT   (NRCILICH     +NBRCILICH     )  /* 181 TAG_INT         */
#define NRCIREC       (NRCILICSTAT  +NBRCILICSTAT  )  /* 182 TAG_INT         */
#define NRCIPLA       (NRCIREC      +NBRCIREC      )  /* 183 TAG_INT         */
#define NRCIPINCAP    (NRCIPLA      +NBRCIPLA      )  /* 184 TAG_INT         */
#define NRCIPINM      (NRCIPINCAP   +NBRCIPINCAP   )  /* 185 TAG_INT         */
#define NRCIPINH      (NRCIPINM     +NBRCIPINM     )  /* 186 TAG_INT         */
#define NRCIOPCMD     (NRCIPINH     +NBRCIPINH     )  /* 187 TAG_INT         */
#define NRCIOPUP      (NRCIOPCMD    +1             )  /* 188 TAG_INT         */
#define NRCIOPDW      (NRCIOPUP     +1             )  /* 189 TAG_INT         */
#define NRCIOPNUM     (NRCIOPDW     +1             )  /* 190 TAG_INT         */
#define NRCILSCDESC0  (NRCIOPNUM    +1             )  /* 191 TAG_INT         */
#define NRCILSCCMD0   (NRCILSCDESC0 +1             )  /* 192 TAG_INT         */
#define NRCILSCRX0    (NRCILSCCMD0  +1             )  /* 193 TAG_INT         */
#define NRCILSCTX0    (NRCILSCRX0   +1             )  /* 194 TAG_INT         */
#define NRCITUNESTA   (NRCILSCTX0   +1             )  /* 195 TAG_INT/ENUM    */
#define NRCICICAMBRID (NRCITUNESTA  +1             )  /* 196 TAG_INT         */
#define NRCISRM       (NRCICICAMBRID+1             )  /* 197 TAG_INT         */
#define NRCIINFO      (NRCISRM      +1             )  /* 198 TAG_INT         */
#define NRCILSCFREEBUF (NRCIINFO    +1             )  /* 199 TAG_INT         */

#define NBCIREG       (NRCILSCFREEBUF +1           )  /* 200 Nb de registres */

/* Valeurs des registres TAG_ENUM ...........................................*/

#define CIMENU_EXIT    0                              /* 0 ENUM "MENU"       */
#define CIMENU_ENTER  (CIMENU_EXIT  +1)               /* 1 ENUM "MENU"       */

#define CIIHM_NONE     0                              /* 0 ENUM "IHMTYPE"    */
#define CIIHM_MENU    (CIIHM_NONE   +1)               /* 1 ENUM "IHMTYPE"    */
#define CIIHM_LIST    (CIIHM_MENU   +1)               /* 2 ENUM "IHMTYPE"    */
#define CIIHM_QUERY   (CIIHM_LIST   +1)               /* 3 ENUM "IHMTYPE"    */
#define CIIHM_PIN     (CIIHM_QUERY  +1)               /* 4 ENUM "IHMTYPE"    */

#define CICFU_REQNONE   0                             /* 0 ENUM "CFUREQ"     */
#define CICFU_REQIMM   (CICFU_REQNONE +1)             /* 1 ENUM "CFUREQ"     */
#define CICFU_REQDELAY (CICFU_REQIMM  +1)             /* 2 ENUM "CFUREQ"     */

#define CITYPE_CI       0                             /* 0 ENUM "CITYPE" CI  */
#define CITYPE_CIPLUS  (CITYPE_CI   +1)               /* 1 ENUM "CITYPE" CI+ */

/* Numero des tags ..........................................................*/

#define NTCITYPE       0                              /*  0 "TYPE"           */
#define NTCINAME      (NTCITYPE     +1)               /*  1 "NAME"           */
#define NTCIAPP       (NTCINAME     +1)               /*  2 "APPNAME"        */
#define NTCICANB      (NTCIAPP      +1)               /*  3 "CANB"           */
#define NTCICAID      (NTCICANB     +1)               /*  4 "CASYSID"        */
#define NTCIMENU      (NTCICAID     +1)               /*  5 "MENU"           */
#define NTCIIHM       (NTCIMENU     +1)               /*  6 "IHM"            */
#define NTCICHOICE    (NTCIIHM      +1)               /*  7 "CHOICE"         */
#define NTCIANSWL     (NTCICHOICE   +1)               /*  8 "ANSWLEN"        */
#define NTCIANSW      (NTCIANSWL    +1)               /*  9 "ANSWER"         */
#define NTCITITLE     (NTCIANSW     +1)               /* 10 "TITLE"          */
#define NTCISUBT      (NTCITITLE    +1)               /* 11 "SUBTITLE"       */
#define NTCICONT      (NTCISUBT     +1)               /* 12 "CONTENT"        */
#define NTCIFOOTER    (NTCICONT     +1)               /* 13 "FOOTER"         */
#define NTCICFUREQ    (NTCIFOOTER   +1)               /* 14 "CFUREQ"         */
#define NTCICFUPROG   (NTCICFUREQ   +1)               /* 15 "CFUPROG"        */
#define NTCICITYPE    (NTCICFUPROG  +1)               /* 16 "CITYPE"         */
#define NTCIKEYODD    (NTCICITYPE   +1)               /* 17 "KEYODD"         */
#define NTCIKEYEVEN   (NTCIKEYODD   +1)               /* 18 "KEYEVEN"        */
#define NTCIURIPRG    (NTCIKEYEVEN  +1)               /* 19 "URIPRG"         */
#define NTCICOUNTRY   (NTCIURIPRG   +1)               /* 20 "COUNTRY"        */
#define NTCILANGUAGE  (NTCICOUNTRY  +1)               /* 21 "LANGUAGE"       */
#define NTCITUNE      (NTCILANGUAGE +1)               /* 22 "TUNE"           */
#define NTCIAPMOPT    (NTCITUNE     +1)               /* 23 "APMOPT"         */
#define NTCIAPM       (NTCIAPMOPT   +1)               /* 24 "APM"            */
#define NTCIAPMFILE   (NTCIAPM      +1)               /* 25 "APMFILE"        */
#define NTCIAPMDAT    (NTCIAPMFILE  +1)               /* 26 "APMDAT"         */
#define NTCINEWCX     (NTCIAPMDAT   +1)               /* 27 "NEWCX"          */
#define NTCICFURESP   (NTCINEWCX    +1)               /* 28 "CFURESP"        */
#define NTCILICM      (NTCICFURESP  +1)               /* 29 "LICM"           */
#define NTCILICH      (NTCILICM     +1)               /* 30 "LICH"           */
#define NTCILICSTAT   (NTCILICH     +1)               /* 31 "LICSTAT"        */
#define NTCIPLACNT    (NTCILICSTAT  +1)               /* 32 "PLACNT"         */
#define NTCIRECSTAT   (NTCIPLACNT   +1)               /* 33 "RECSTAT"        */
#define NTCIRECNUM    (NTCIRECSTAT  +1)               /* 34 "RECNUM"         */
#define NTCIRECMODE   (NTCIRECNUM   +1)               /* 35 "RECMODE"        */
#define NTCIRECCMD    (NTCIRECMODE  +1)               /* 36 "RECCMD"         */
#define NTCIPLACMD    (NTCIRECCMD   +1)               /* 37 "PLACMD"         */
#define NTCIPINCAP    (NTCIPLACMD   +1)               /* 38 "PINCAP"         */
#define NTCIPRATING   (NTCIPINCAP   +1)               /* 39 "PRATING"        */
#define NTCIPINSTAT   (NTCIPRATING  +1)               /* 40 "PINSTAT"        */
#define NTCIPINM      (NTCIPINSTAT  +1)               /* 41 "PINM"           */
#define NTCIPINH      (NTCIPINM     +1)               /* 42 "PINH"           */
#define NTCIOPCMDH    (NTCIPINH     +1)               /* 43 "OPCMDH"         */
#define NTCIOPSTATH   (NTCIOPCMDH   +1)               /* 44 "OPSTATH"        */
#define NTCIOPCMDM    (NTCIOPSTATH  +1)               /* 45 "OPCMDM"         */
#define NTCIOPSTATM   (NTCIOPCMDM   +1)               /* 46 "OPSTATM"        */
#define NTCIOPUP      (NTCIOPSTATM  +1)               /* 47 "OPUP"           */
#define NTCIOPDW      (NTCIOPUP     +1)               /* 48 "OPDW"           */
#define NTCILSCDESC0  (NTCIOPDW     +1)               /* 49 "LSCDESC"        */
#define NTCILSCCMD0   (NTCILSCDESC0 +1)               /* 50 "LSCCMD"         */
#define NTCILSCSTAT0  (NTCILSCCMD0  +1)               /* 51 "LSCSTAT"        */
#define NTCILSCRX0    (NTCILSCSTAT0 +1)               /* 52 "LSCRX"          */
#define NTCILSCTX0    (NTCILSCRX0   +1)               /* 53 "LSCTX"          */
#define NTCITUNESTA   (NTCILSCTX0   +1)               /* 54 "TUNESTA"        */
#define NTCITUNEREL   (NTCITUNESTA  +1)               /* 55 "TUNEREL"        */
#define NTCIOPNUM     (NTCITUNEREL  +1)               /* 56 "OPNUM"          */
#define NTCIIHMOPEN   (NTCIOPNUM    +1)               /* 57 "IHMOPEN"        */
#define NTCILSCRXTO0  (NTCIIHMOPEN  +1)               /* 58 "LSCRXTO"        */
#define NTCICICAMBRID (NTCILSCRXTO0 +1)               /* 59 "CICAMBRID"      */
#define NTCIAUTHSTATE (NTCICICAMBRID+1)               /* 60 "AUTHSTATE"      */
#define NTCIRESETPCM  (NTCIAUTHSTATE+1)               /* 61 "RESETPCM"       */
#define NTCITUNESESS  (NTCIRESETPCM +1)               /* 62 "TUNESESS"       */
#define NTCISRM       (NTCITUNESESS +1)               /* 63 "SRM"            */
#define NTCIINFO      (NTCISRM      +1)               /* 64 "INFO"           */
#define NTCILSCFREEBUF (NTCIINFO    +1)               /* 65 "LSCFREEBUF"     */

/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/drv_vid.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere et memoire .........*/

#define NBRTVID                      1 /* TYPE    , au max   3 cars + '\0'   */
#define NBRNVID                      1 /* NAME    , au max   3 cars + '\0'   */
#define NBRH264SPS                   8 /* H264_SPS (8*4)=32 octets           */
#define NBRH264PPS                   8 /* H264_PPS (8*4)=32 octets           */

/* Numero des registres .....................................................*/

#define VIDRTYPE                               0 /* TYPE      (0 )           */
#define VIDRNAME     (VIDRTYPE     + 1         ) /* NAME      (1 )           */
#define VIDRCMD      (VIDRNAME     + 1         ) /* CMD       (2 )           */
#define VIDRSOURCE   (VIDRCMD      + 1         ) /* SOURCE    (3 )           */
#define VIDRPCRSRC   (VIDRSOURCE   + 1         ) /* PCRSRC    (4 )           */
#define VIDRDECCFG   (VIDRPCRSRC   + 1         ) /* DECCFG    (5 )           */
#define VIDRMFFDUR   (VIDRDECCFG   + 1         ) /* MFFDUR    (6 )           */
#define VIDRSPEED    (VIDRMFFDUR   + 1         ) /* SPEED     (7 )           */
#define VIDRSFREQ    (VIDRSPEED    + 1         ) /* SFREQ     (8 )           */
#define VIDRDSPCFG   (VIDRSFREQ    + 1         ) /* DSPCFG    (9 )           */
#define VIDRDPS      (VIDRDSPCFG   + 1         ) /* DPS       (10)           */
#define VIDRSZPS     (VIDRDPS      + 1         ) /* SZPS      (11)           */
#define VIDRSIZE     (VIDRSZPS     + 1         ) /* SIZE      (12)           */
#define VIDRSTATE    (VIDRSIZE     + 1         ) /* STATE     (13)           */
#define VIDRDSP      (VIDRSTATE    + 1         ) /* DSP       (14)           */
#define VIDRDFCX     (VIDRDSP      + 1         ) /* DFCX      (15)           */
#define VIDRDFCY     (VIDRDFCX     + 1         ) /* DFCY      (16)           */
#define VIDRPICT_NUM (VIDRDFCY     + 1         ) /* PICT_NUM  (17)           */
#define VIDRPICT_FST (VIDRPICT_NUM + 1         ) /* PICT_FST  (18)           */
#define VIDRRSPEED   (VIDRPICT_FST + 1         ) /* RSPEED    (19)           */
#define VIDRDSTATE   (VIDRRSPEED   + 1         ) /* DSTATE    (20)           */
#define VIDRH264SPS  (VIDRDSTATE   + 1         ) /* H264_SPS  (21)           */
#define VIDRH264PPS  (VIDRH264SPS  + NBRH264SPS) /* H264_PPS  (29)           */
#define VIDRDVID     (VIDRH264PPS  + NBRH264PPS) /* DVID      (37)           */
#define VIDRSZVID    (VIDRDVID     + 1         ) /* SZVID     (38)           */
#define VIDRDDISP    (VIDRSZVID    + 1         ) /* DDISP     (39)           */
#define VIDRSZDISP   (VIDRDDISP    + 1         ) /* SZDISP    (40)           */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/drv_aud.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRTYPEAUD         1           /* TYPE    , au max   3 cars + '\0'   */
#define NBRNAMEAUD         1           /* NAME    , au max   3 cars + '\0'   */


/* Numero des registres .....................................................*/

#define  AUDRTYPE          0           /* TYPE                               */
#define  AUDRNAME          1           /* NAME                               */
#define  AUDRSOURCE        2           /* SOURCE                             */
#define  AUDRPCRSRC        3           /* PCRSRC                             */
#define  AUDRDECCFG        4           /* DECCFG                             */
#define  AUDRESSTRID       5           /* ESSTRID                            */
#define  AUDRVOLUME        6           /* VOLUME                             */
#define  AUDRVOLNBLEV      7           /* VOLNBLEV                           */
#define  AUDRDECMCFG       8           /* DECMCFG                            */
#define  AUDRDECACFG       9           /* DECMCFG                            */
#define  AUDRJOBCFG       10           /* JOBCFG                             */
#define  AUDRSTATE        11           /* STATE                              */
#define  AUDRSFREQ        12           /* SFREQ                              */
#define  AUDRBRATE        13           /* BITRATE                            */
#define  AUDRMPSTATUS     14           /* MPSTATUS                           */
#define  AUDRAC3STATUS    15           /* AC3STATUS                          */
#define  AUDRAC3STATUS2   16           /* AC3STATUS2                         */
#define  AUDRAC3SETTINGS  17           /* AC3SETTINGS                        */
#define  AUDRAACSTATUS    18           /* AACSTATUS                          */
#define  AUDROUTPUT       19           /* OUTPUT                             */
#define  AUDRHDMI         20           /* HDMI                               */
#define  AUDRSWITCH       21           /* SWITCH                             */
#define  AUDRLEV_TAB      22           /* LEV_TAB                            */
#define  AUDRHDMI_CODINGS 54           /* HDMI_CODINGS                       */

/* Statut des decodeurs video/audio (valeurs du tag STATUS) .................*/

#define DECODER_STOP       0  /* Decodeur arrete                             */
#define DECODER_START      1  /* Decodeur demarre, pas de donnees mais pas   */
                              /* besoin de le dire                           */
#define BBF_ALMOST_FULL    2  /* Decodeur demarre proche de l'overflow       */
#define BBF_ALMOST_EMPTY   3  /* Decodeur demarre proche du buffer vide      */
#define BBF_EMPTY          4  /* Decodeur demarre buffer vide                */
#define DECODER_ERROR      5  /* Decodeur en erreur                          */
#define DECODER_RESET      6  /* Decodeur fin de reset                       */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/srv_stl.c"                   *
 *****************************************************************************/

/* Numero des registres .....................................................*/

#define  STLRTYPE          0           /* TYPE                               */
#define  STLRNAME          1           /* NAME                               */
#define  STLRCMD           3           /* CMD                                */
#define  STLRSOURCE        4           /* SOURCE                             */
#define  STLRPCRSRC        5           /* PCRSRC                             */
#define  STLRPAGES         6           /* MAIN_PAGE, ANC_PAGE                */


/*****************************************************************************
 * Numeros des registres de "$vmio/drvcar/drv_swi.c"                         *
 *****************************************************************************/

#define NRSWITYPE       0              /* adr chaine TYPE                    */
#define NRSWINAME       1              /* adr chaine NAME                    */
#define NRSWICAPS       2              /* Capacites du SWITCH                */
#define NRSWIRGB        3              /* Valeurs Min/Max RGB                */
#define NRSWIAUDTV      4              /* Reglages AUD_TV MS_TV              */
#define NRSWIVIDTV      5              /* Reglages PRGB_TV RGBLEV_TV Px_TV   */
#define NRSWIVCR        6              /* Reglages AUD_VCR MS_VCR Px_VCR     */
#define NRSWIINP8_TV    7              /* Lecture etat P8 TV                 */
#define NRSWIINP8_VCR   8              /* Lecture etat P8 VCR                */
#define NRSWIPOWER      9              /* Mode normal ou basse consommation  */
#define NRSWITOGGLES   10              /* Subtitles,OSDINCVBS ON/OFF         */
#define NRSWIVIDSYNC   11              /* Lecture synchro video              */
#define NRSWIAUDLEV    12              /* Reglage volume                     */
#define NRSWIVOLNBLEV  13              /* Nombre de niveaux de volume        */
#define NRSWILEV_TAB   14              /* Attenuations des niveaux de volume */


/*****************************************************************************
 * Numeros des registres de "$vmio/drvcar/drv_eth.c"                         *
 *****************************************************************************/

#define ETHRTYPE            0          /* Adresse chaine TYPE                */
#define ETHRNAME            1          /* Adresse chaine NAME                */
#define ETHRSTATE           2          /* Etat du cable reseau.              */
#define ETHRADDRESS         3          /* Adresse MAC de l'interface reseau  */
#define ETHRNODE1           4          /* Tunnel node 1                      */
#define ETHRNODE2           5          /* Tunnel node 2                      */
#define ETHRNODE3           6          /* Tunnel node 3                      */
#define ETHRNODE4           7          /* Tunnel node 4                      */
#define ETHRNODE5           8          /* Tunnel node 5                      */
#define ETHRNODE6           9          /* Tunnel node 6                      */
#define ETHRNODE7           10         /* Tunnel node 7                      */
#define ETHRNODE8           11         /* Tunnel node 8                      */
#define ETHRNODE9           12         /* Tunnel node 9                      */
#define ETHRNODE10          13         /* Tunnel node 10                     */
#define ETHRNODE11          14         /* Tunnel node 11                     */
#define ETHRNODE12          15         /* Tunnel node 12                     */
#define ETHRNODE13          16         /* Tunnel node 13                     */
#define ETHRNODE14          17         /* Tunnel node 14                     */
#define ETHRNODE15          18         /* Tunnel node 15                     */
 
#define ETHNBREGS           19         /* Nombre total de registres.         */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvblk/dsk_ata.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRTYPEATA       1             /* TYPE fait au max 3 car + '\0'      */
#define NBRCTRNAME       8             /* NAME fait au max 31 car + '\0'     */
#define NBRDISKNAME      11            /* LIST fait au max 40 car + '\0'     */

 
/* Numero des registres .....................................................*/

#define ATARTYPE         0             /* Adresse chaine "ATA"               */
#define ATARCTRNAME      1             /* Adresse chaine nom controleur      */
#define ATARSTATE        9             /* Etat d'un disque                   */
#define ATARSZSECTOR     10            /* Taille d'un secteur en octets      */
#define ATARDISKNAME     11            /* Adresse chaine nom disque          */
#define ATARTEMP         22            /* Temperature actuelle du disque     */
#define ATARPOWER        23            /* Mode alimentation du disque        */
#define ATARDELAY        24            /* Delai de mise en standby (secondes)*/
#define ATARCIPHER       25            /* Mode de chiffrement du disque      */
#define ATARFSTLBAMSB    26            /* MSB du LBA du 1er secteur          */
#define ATARFSTLBALSB    27            /* LSB du LBA du 1er secteur          */
#define ATARNBLBAMSB     28            /* MSB du nombre de secteurs          */
#define ATARNBLBALSB     29            /* LSB du nombre de secteurs          */
#define ATARCMD          30            /* Commande manipulation de partitions*/


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvblk/dsk_nand.c"                  *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/


#define NBRTYPENAND      2             /* TYPE fait au max 4 car + '\0'      */
#define NBRNAMENAND      8             /* NAME fait au max 31 car + '\0'     */
#define NBRNANDDISKNAME  11            /*                                    */

 
/* Numero des registres .....................................................*/

#define NANDRTYPE         0            /* Adresse chaine "NAND"              */
#define NANDRNAME         1            /* Adresse chaine nom de la flash     */
#define NANDRSTATE        2            /* A virer ...                        */
#define NANDRDISKNAME     3            /* A virer ...                        */
#define NANDRIDMAN        4            /* Identifiant "manufacturer".        */
#define NANDRIDDEV        5            /* Identifiant "device".              */
#define NANDRTDATA        6            /* Nb oct donnees user  par secteur   */
#define NANDRTSPARE       7            /* Nb oct donnees spare par secteur   */
#define NANDRSECPERBLK    8            /* Nombre de secteurs par bloc        */
#define NANDRNBBLK        9            /* Nombre total de blocs              */
#define NANDRFSTLBAMSB    10           /* MSB 1er secteur partition.         */
#define NANDRFSTLBALSB    11           /* LSB 1er secteur partition.         */
#define NANDRNBLBAMSB     12           /* MSB nombre de secteurs ds part.    */
#define NANDRNBLBALSB     13           /* LSB nombre de secteurs ds part.    */
#define NANDRCMD          14           /* Commande sur table des part.       */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvsys/dsk_usb.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRUMSTYPE     1               /* TYPE: UMS au max  3 cars + '\0'    */
#define NBRUMSNAME     1               /* NAME: ""  au max  3 cars + '\0'    */

/* Numero des registres .....................................................*/

#define NRUMSTYPE      0                              /* TAG_STR             */
#define NRUMSNAME     (NRUMSTYPE    +NBRUMSTYPE    )  /* TAG_STR             */
#define NRUMSETAT     (NRUMSNAME    +NBRUMSNAME    )  /* TAG_ENUM            */
#define NRUMSHOSTCTR  (NRUMSETAT    +1             )  /* TAG_INT             */
#define NRUMSPORTNB   (NRUMSHOSTCTR +1             )  /* TAG_INT             */
#define NRUMSFSTLBAM  (NRUMSPORTNB  +1             )  /* TAG_INT             */
#define NRUMSFSTLBAL  (NRUMSFSTLBAM +1             )  /* TAG_INT             */
#define NRUMSNBLBAM   (NRUMSFSTLBAL +1             )  /* TAG_INT             */
#define NRUMSNBLBAL   (NRUMSNBLBAM  +1             )  /* TAG_INT             */
#define NRUMSCMD      (NRUMSNBLBAL  +1             )  /* TAG_ENUM            */
#define NBUMSREG      (NRUMSCMD     +1             )  /* Nombre de registres */

#define OFUMSPART      NRUMSFSTLBAM                   /* Offset regs. part.  */
#define NBUMSPART     (NBUMSREG     - OFUMSPART    )  /* Nombre regs. part.  */

/* Valeurs des registres TAG_ENUM ...........................................*/

#define UMSCMD_REMOVE  0                              /* TAG_ENUM "CMD"      */
#define UMSCMD_ADD    (UMSCMD_REMOVE+1)               /* TAG_ENUM "CMD"      */

#define UMSETAT_UNPLUG 0                              /* TAG_ENUM "ETAT"     */
#define UMSETAT_CLS   (UMSETAT_UNPLUG+1)              /* TAG_ENUM "ETAT"     */
#define UMSETAT_RDY   (UMSETAT_CLS   +1)              /* TAG_ENUM "ETAT"     */
#define UMSETAT_OTHER (UMSETAT_RDY   +1)              /* TAG_ENUM "ETAT"     */

/* Numero des tags ..........................................................*/
 
#define NTUMSTYPE      0                              /* "TYPE"              */
#define NTUMSNAME     (NTUMSTYPE    +1)               /* "NAME"              */
#define NTUMSETAT     (NTUMSNAME    +1)               /* "UMSETAT"           */
#define NTUMSHOSTCTR  (NTUMSETAT    +1)               /* "HOSTCTR"           */
#define NTUMSPORTNB   (NTUMSHOSTCTR +1)               /* "PORTNB"            */
#define NTUMSFSTLBAM  (NTUMSPORTNB  +1)               /* "FSTLBAMSB"         */
#define NTUMSFSTLBAL  (NTUMSFSTLBAM +1)               /* "FSTLBALSB"         */
#define NTUMSNBLBAM   (NTUMSFSTLBAL +1)               /* "NBLBAMSB"          */
#define NTUMSNBLBAL   (NTUMSNBLBAM  +1)               /* "NBLBALSB"          */
#define NTUMSCMD      (NTUMSNBLBAL  +1)               /* "CMD"               */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvpro/pro_dsm.c"                   *
 *****************************************************************************/

/* Taille (en mots) des registres de chaine de caractere ....................*/

#define NBRDSMTYPE        1            /* TYPE    , au max   3 cars + '\0'   */

/* Numero des registres .....................................................*/

#define NRDSMTYPE      0                              /* TYPE                */
#define NRDSMPMT      (NRDSMTYPE    +NBRDSMTYPE    )  /* PMT                 */
#define NRDSMPARAM    (NRDSMPMT     +1             )  /* MOUNT, NBFLT        */
#define NRDSMTAG      (NRDSMPARAM   +1             )  /* TAG                 */
#define NRDSMCARID    (NRDSMTAG     +1             )  /* CARID               */
#define NRDSMPID      (NRDSMCARID   +1             )  /* BOOTPID, TRGPID     */

/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvcar/srv_str.c"                   *
 *****************************************************************************/

/* Numero des registres .....................................................*/

#define NRSTRTYPE         0            /* Adresse chaine type                */
#define NRSTRNAME         1            /* Adresse chaine version du driver   */
#define NRDEVICE          2            /* Numeros des devices du pipe-line   */
#define NRDMX2DEC         3            /* Connexion DMX --> DEC              */
#define NRFIFODEC         4            /* Numero FIFO pour decodeurs source  */
#define NRVCODING         5            /* Reglage du codage des donnees video*/
#define NRACODING         6            /* Reglage du codage des donnees audio*/
#define NRVIDSFREQ        7            /* Reglage sampling rate video        */
#define NRAUDSFREQ        8            /* Reglage sampling rate audio        */
#define NRNUMOUT          9            /* Reglage de la sortie audio NUMOUT  */
#define NRCNLNAME         10           /* Nom chaine TV qu'on enregistre     */
#define NRRECNAME         11           /* Nom a donner a l'enregistrement    */
#define NRRECATTR         12           /* Attributs initiaux de l'enregistrem*/
#define NRRECVOL          13           /* Nom du volume sur lequel ecrire    */
#define NRRECPATH         14           /* Chemin absolu repertoire REC       */
#define NRRECFORMAT       15           /* Format d'enregistrement            */
#define NRCIPHER          16           /* Activation chiffrement audio/video */
#define NRSTRTTL          17           /* Time to live pour enregistrements  */
#define NRRECCMD          18           /* Commande element REC               */
#define NRRECTIM          19           /* Timestamp dernier paquet enregistre*/
#define NRRECSTATE        20           /* Etat courant element REC           */
#define NRRECFSERR        21           /* Code d'erreur file system REC      */
#define NRPLAVOL          22           /* Nom du volume sur lequel lire      */
#define NRPLAPATH         23           /* Chemin absolu repertoire PLAY      */
#define NRPLATYPE         24           /* Type du fichier joue               */
#define NRPLANUMREF       25           /* Numero 1er fichier du repertoire   */
#define NRPLAFORMAT       26           /* Format de lecture                  */
#define NRPLACMD          27           /* Commande element PLAY              */
#define NRPLADECCTR       28           /* Pilotage des decodeurs source      */
#define NRPLALIST         29           /* Liste des flots a lire             */
#define NRPLASTLPAGE      30           /* Numero page principale ss-titres   */
#define NRPLASTLANC       31           /* Numero page partagee ss-titres     */
#define NRPLAVCODING      32           /* Reglage du codage des donnees video*/
#define NRPLAACODING      33           /* Reglage du codage des donnees audio*/
#define NRPLAVSFREQ       34           /* Reglage sampling rate video        */
#define NRPLAASFREQ       35           /* Reglage sampling rate audio        */
#define NRPLANUMOUT       36           /* Reglage de la sortie audio NUMOUT  */
#define NRPLADRVSEGID     37           /* Identifiant segment courant du drv */
#define NRPLAAPPSEGID     38           /* Identifiant segment connu de l'app */
#define NRPLATIM          39           /* Timestamp dernier paquet lu        */
#define NRPLAMRK          40           /* Id. dernier marqueur rencontre     */
#define NRPLAXTIM         41           /* Timestamp ou il faut se deplacer   */
#define NRPLAXMRK         42           /* Id. mrk ou il faut se deplacer     */
#define NRPLAMODE         43           /* Mode de lecture                    */
#define NRPLASPEED        44           /* Vitesse de lecture                 */
#define NRPLASTATE        45           /* Etat courant element PLAY          */
#define NRSTRDEADL        46           /* Donnees non perimees ou perimees   */
#define NRPLAFSERR        47           /* Code d'erreur file system PLAY     */
#define NRTSBPATH         48           /* Chemin absolu repertoire TSB       */
#define NRTSBDIM          49           /* Taille du TSB en secondes          */
#define NRTSBSIZE         50           /* Taille du TSB en secondes          */
#define NRTSBCMD          51           /* Commande element TSB               */
#define NRTSBDECCTR       52           /* Pilotage des decodeurs source      */
#define NRTSBLIST         53           /* Liste des flots a lire             */
#define NRTSBSTLPAGE      54           /* Numero page principale ss-titres   */
#define NRTSBSTLANC       55           /* Numero page partagee ss-titres     */
#define NRTSBVCODING      56           /* Reglage du codage des donnees video*/
#define NRTSBACODING      57           /* Reglage du codage des donnees audio*/
#define NRTSBVSFREQ       58           /* Reglage sampling rate video        */
#define NRTSBASFREQ       59           /* Reglage sampling rate audio        */
#define NRTSBNUMOUT       60           /* Reglage de la sortie audio NUMOUT  */
#define NRTSBDRVSEGID     61           /* Identifiant segment courant du drv */
#define NRTSBAPPSEGID     62           /* Identifiant segment connu de l'app */
#define NRTSBTIM          63           /* Timestamp dernier paquet lu        */
#define NRTSBMRK          64           /* Id. dernier marqueur rencontre     */
#define NRTSBXTIM         65           /* Timestamp ou il faut se deplacer   */
#define NRTSBXMRK         66           /* Id. mrk ou il faut se deplacer     */
#define NRTSBMRKOUT       67           /* Id. dernier marqueur degage du TSB */
#define NRTSBMODE         68           /* Mode de lecture                    */
#define NRTSBSPEED        69           /* Vitesse de lecture                 */
#define NRTSBBEGIN        70           /* Timestamp plus ancien fichier TSB  */
#define NRTSBSTATE        71           /* Etat courant de l'element TSB      */
#define NRTSBFSERR        72           /* Code d'erreur file system TSB      */
#define NRCOPYMRK1        73           /* Marqueur debut de la copie         */
#define NRCOPYMRK2        74           /* marqueur fin de la copie           */
#define NRCOPYPERCENT     75           /* Pourcentage d'avancement copie     */
#define NRSTRTIMEREF      76           /* Val courante base de temps lecture */

#define NRMP4STATUS       77           /* Statut recherche des index MP4     */
#define NRMP4DURATION     78           /* Duree totale de la presentation    */
#define NRMP4VIDCODING    79           /* Codage video dans le fichier       */
#define NRMP4VIDSFREQ     80           /* Frequence echantillonnage video    */
#define NRMP4AUDCODING    81           /* Codage audio dans le fichier       */
#define NRMP4AUDSFREQ     82           /* Frequence echantillonnage audio    */

#define NRSTRCILICENCE    83           /* Licence  CI plus v1.3              */
#define NRSTRCIPIN        84           /* Info Pin CI Plus v1.3              */

#define NRSTRHLSXML       85           /* Generation of XML files            */


/* Registres non exposes a l'exterieur ......................................*/
#define NRDECODERRUN      87           /* Indique si decodeur source demarre */
#define NRPLATYPPOS       88           /* Type positionnement demande player */
#define NRTSBTYPPOS       89           /* Type positionnement demande TSB    */
#define NRPLALISTGOT      90           /* Masque tags fournis par l'appli    */
#define NRTSBLISTGOT      91           /* Masque tags fournis par l'appli    */

#define NBREG_STR         92

/* Valeurs des registres TAG_ENUM ...........................................*/

#define RECCMD_IDLE         0          /* REC_CMD = IDLE                     */
#define RECCMD_RUN          1          /* REC_CMD = RUN                      */

#define PLACMD_IDLE         0          /* PLA_CMD = IDLE                     */
#define PLACMD_RUN          1          /* PLA_CMD = RUN                      */
#define PLACMD_STEP         2          /* PLA_CMD = STEP                     */
 
#define TSBCMD_IDLE         0          /* TSB_CMD = IDLE                     */
#define TSBCMD_RUN          1          /* TSB_CMD = RUN                      */
#define TSBCMD_RECORD       2          /* TSB_CMD = RECORD                   */
#define TSBCMD_SUSPEND      3          /* TSB_CMD = SUSPEND                  */
#define TSBCMD_CONTINUE     4          /* TSB_CMD = CONTINUE                 */
#define TSBCMD_DROP         5          /* TSB_CMD = DROP                     */
#define TSBCMD_RECOVER      6          /* TSB_CMD = RECOVER                  */
#define TSBCMD_RECOVERP     7          /* TSB_CMD = RECOVERP                 */
#define TSBCMD_PLAY         8          /* TSB_CMD = PLAY                     */
#define TSBCMD_STOP         9          /* TSB_CMD = STOP                     */
#define TSBCMD_STEP         10         /* TSB_CMD = STEP                     */
#define TSBCMD_COPY         11         /* TSB_CMD = COPY                     */
 
#define FORMAT_PES_TV25     0          /* FORMAT = PES_TV25                  */
#define FORMAT_PES_TV30     1          /* FORMAT = PES_TV30                  */
#define FORMAT_PES_RADIO    2          /* FORMAT = PES_RADIO                 */
#define FORMAT_TRP          3          /* FORMAT = TRP                       */
#define FORMAT_HLS          4          /* FORMAT = HLS                       */
#define FORMAT_MPG          5          /* FORMAT = MPG                       */
#define FORMAT_FLV          6          /* FORMAT = FLV                       */
#define FORMAT_AVI          7          /* FORMAT = AVI                       */
#define FORMAT_MP4          8          /* FORMAT = MP4                       */
#define FORMAT_MP3          9          /* FORMAT = MP3                       */
 
#define REC_IDLE            0          /* REC_STATE = IDLE                   */
#define REC_RUNNING         1          /* REC_STATE = RUNNING                */
#define REC_COPYING         2          /* REC_STATE = COPYING                */
#define REC_DSKFULL         3          /* REC_STATE = DSKFULL                */
#define REC_DIRFULL         4          /* REC_STATE = DIRFULL                */
#define REC_FILEFULL        5          /* REC_STATE = FILEFULL               */
#define REC_IOERR           6          /* REC_STATE = IOERR                  */
#define REC_ALLOCERR        7          /* REC_STATE = ALLOCERR               */

#define PLA_IDLE            0          /* PLA_STATE = IDLE                   */
#define PLA_SEARCHING       1          /* PLA_STATE = SEARCHING              */
#define PLA_AT_BEGIN        2          /* PLA_STATE = AT_BEGIN               */
#define PLA_RUNNING         3          /* PLA_STATE = RUNNING                */
#define PLA_AT_END          4          /* PLA_STATE = AT_END                 */
#define PLA_CORRUPTED       5          /* PLA_STATE = CORRUPTED              */
#define PLA_IOERR           6          /* PLA_STATE = IOERR                  */
#define PLA_ALLOCERR        7          /* PLA_STATE = ALLOCERR               */
 
#define TSB_IDLE            0          /* TSB_STATE = IDLE                   */
#define TSB_RECORDING       1          /* TSB_STATE = RECORDING              */
#define TSB_SEARCHING       2          /* TSB_STATE = SEARCHING              */
#define TSB_AT_BEGIN        3          /* TSB_STATE = AT_BEGIN               */
#define TSB_CRITIC          4          /* TSB_STATE = CRITIC                 */
#define TSB_SHIFTED         5          /* TSB_STATE = SHIFTED                */
#define TSB_AT_END          6          /* TSB_STATE = AT_END                 */
#define TSB_DSKFULL         7          /* TSB_STATE = DSKFULL                */
#define TSB_FILEFULL        8          /* TSB_STATE = FILEFULL               */
#define TSB_CORRUPTED       9          /* TSB_STATE = CORRUPTED              */
#define TSB_IOERR           10         /* TSB_STATE = IOERR                  */
#define TSB_ALLOCERR        11         /* TSB_STATE = ALLOCERR               */


/*****************************************************************************
 * Numeros des registres de "$vmio/drvcar/drv_sec.c"                         *
 *****************************************************************************/

#define NBRKEY                     8   /* Nombre de registres pour tag KEY   */
#define NBRRND                   129   /* Nombre de registres pour tag RND   */
#define NBRCWK                     4   /* Nombre de registres pour tag CWK   */
#define NBRNUID                    2   /* Nombre de registres pour tags NUID */

#define NRSECTYPE                  0   /* Numero registre pour tag TYPE      */
#define NRSECNAME (NRSECTYPE+      1)  /* Numero registre pour tag NAME      */
#define NRINFO    (NRSECNAME+      1)  /* Numero registre pour tags IS_xxx   */
#define NRNUID    (NRINFO   +      1)  /* Numero registre pour tag NUID0     */
#define NRCHKNBR  (NRNUID   +NBRNUID)  /* Numero registre pour tag CHKNBR    */
#define NRCASNB   (NRCHKNBR +      1)  /* Numero registre pour tag CASSNB    */
#define NRCACRC   (NRCASNB  +      1)  /* Numero registre pour tag CACRC     */
#define NRKEY     (NRCACRC  +      1)  /* Numero 1er registre pour tag KEY   */
#define NRRNDLG   (NRKEY    + NBRKEY)  /* Numero registre pour tag RNDLG     */
#define NRRND     (NRRNDLG  +      1)  /* Numero 1er registre pour tag RND   */
#define NRKDMASRC (NRRND    + NBRRND)  /* Numero registre pour tag ADR_SRC   */
#define NRKDMADST (NRKDMASRC+      1)  /* Numero registre pour tag ADR_DST   */
#define NRKDMANB  (NRKDMADST+      1)  /* Numero reg. tags NB_SRC et NB_DST  */
#define NRKDMACMD (NRKDMANB +      1)  /* Numero reg. tags METHOD et MODE.   */
#define NRSCWMODE (NRKDMACMD+      1)  /* Numero reg. tag SCW_MODE.          */
#define NRCWK     (NRSCWMODE+      1)  /* Numero 1er reg. tag CWK.           */
#define NRFUSE    (NRCWK    + NBRCWK)  /* Numero reg. tag FUSE.              */
#define NRKEYDEF  (NRFUSE   +      1)  /* N reg. tags KEY_NB KEY_NUM KEY_TYPE*/
#define NBSECREG  (NRKEYDEF +      1)  /* Nombre total de registres.         */

#define MSK_IS_NUID     0x00000001
#define MSK_IS_CW       0x00000002
#define MSK_IS_RND      0x00000004
#define MSK_IS_SCS      0x00000008
#define MSK_IS_SK       0x00000010
#define MSK_IS_TDES     0x00000020
#define MSK_IS_TDES_CFG 0x00000040
#define MSK_IS_AES      0x00000080
#define MSK_IS_AES_CFG  0x00000100
#define MSK_ALGO        0x00000003
#define MSK_CHAIN       0x00000300
#define MSK_SENS        0x00010000
#define MSK_SCW_MODE    0x00000003
#define MSK_FUSE        0x00000001
#define MSK_KEY_NB      0x0000000F
#define MSK_KEY_NUM     0x0000FF00
#define MSK_KEY_TYPE    0x00010000

#define SFT_ALGO                 0
#define SFT_CHAIN                8
#define SFT_SENS                16
#define SFT_KEY_NB               0         
#define SFT_KEY_NUM              8         
#define SFT_KEY_TYPE            16        

#define FUSE_SCW        0x00000001     /* Valeur SCW du tag FUSE.            */


/*****************************************************************************
 * Numeros des registres de "$vmio/drvcar/drv_path.c"                        *
 *****************************************************************************/

#define NRPTHTYPE       0              /* tag TYPE                           */
#define NRPTHNAME       1              /* tag NAME            ( 2 registres) */
#define NRNBFDT         3              /* tag NB_FRE NB_DVBPATH NB_TINPATH   */
#define NRACMD          4              /* tag APPLYCMD                       */
#define NRPTHTFRE       5              /* tag FRE                            */
#define NRDVBPATH       6              /* tag DVB_PATH                       */
#define NRDVBISWI       7              /* tag DVB_INSWI                      */
#define NRDVBOSWI       8              /* tag DVB_OUTSWI                     */
#define NRDVBCMD        9              /* tag DVB_CMD                        */
#define NRTINPATH       10             /* tag TIN_PATH                       */
#define NRTINSWI        11             /* tag TIN_INSWI                      */
#define NRTINCMD        12             /* tag TIN_CMD                        */
#define NRPTHPOW        13             /* tag POWGRP POWSTA                  */

#define PTH_NBDREG      (NRACMD  +1)   /* Nombre de registres lies au device.*/
#define PTH_NBREG       (NRPTHPOW+1)   /* Nombre total de registres          */


/*****************************************************************************
 * Numeros des registres, tags de "$vmio/drvpro/srv_telnet.c"                *
 *****************************************************************************/

#define TELNET_REG                        12 /* Nombre de registres TELNET   */

#define NRTYPE                             0 /* Type du service              */
#define NRNAMETEL                          2 /* Nom du service               */
#define NRREMIP                            4 /* Adresse IP du serveur (CALL) */
#define NRREMPORT                          5 /* Port distant (CALL)          */
#define NRLOCIP                            6 /* Adresse IP locale (LISTEN)   */
#define NRLOCPORT                          7 /* Port local (LISTEN)          */
#define NRCMDTEL                           8 /* Option binaire               */
#define NRBIN                              9 /* Option binaire               */
#define NRECHO                            10 /* Option echo                  */
#define NRNOGA                            11 /* Option supp Go Ahead         */


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of RDRV_H.                     */
