/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/stdtyp.l"              */
/*  Standard definitions and constants.                                      */
/*                                                                           */
/*  Author : JYA                        Last modification : 20/07/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================

         Hyperpanel OS standard definitions and constants.

*/


#ifndef STDTYP_L                       /* Debut compilation conditionnelle.  */
#define STDTYP_L                       /* Identificateur de cet include.     */

#include <vm.h>                        /* Constantes de la VM.               */


/* Instruction blocs ........................................................*/

#define BEGIN            {             /* Debut de bloc d'instructions.      */
#define END              }             /* Fin de bloc d'instructions.        */
#define END_IF           }             /* Fin de structure logique "if".     */
#define END_THEN         }             /* Fin de structure logique "then".   */
#define END_ELSE         }             /* Fin de structure logique "else".   */
#define END_SW           }             /* Fin de structure logique "switch". */
#define END_FOR          }             /* Fin de structure logique "for".    */
#define END_WHILE        }             /* fin de structure logique "while".  */
#define END_UNTIL        }             /* Fin de structure logique "until".  */
#define END_PROC         }             /* Fin de structure logique "proced". */

                                       /* Procedures et variables externes.  */
#define PROCEDURE                      /* Declaration de procedure.          */

                                       /* Operateurs logiques.               */
#define OR               |             /* Operateur "ou".                    */
#define AND              &             /* Operateur "et".                    */
#define XOR              ^             /* Operateur "ou exclusif".           */
#define LOGOR            ||            /* Operateur "ou" logique.            */
#define LOGAND           &&            /* Operateur "et" logique.            */
#define NOT              !             /* Negation logique.                  */

                                       /* Operateurs de comparaisons.        */
#define EQ               ==            /* Operateur "egal a".                */
#define NE               !=            /* Operateur "different de".          */
#define GT               >             /* Operateur "superieur a".           */
#define LT               <             /* Operateur "inferieur a".           */
#define GE               >=            /* Operateur "superieur ou egal a".   */
#define LE               <=            /* Operateur "inferieur ou egal a".   */


/* Standard types ...........................................................*/

typedef int              IWORD ;       /* Type mot machine.                  */
typedef unsigned int     UIWORD;       /* Type mot machine.                  */

#define CHAR             HYPER_CHAR    /* Entier signe      sur 1 octet.     */
#define UCHAR            HYPER_UCHAR   /* Entier non signe  sur 1 octet.     */
#define HBYTE            HYPER_UCHAR   /* Entier non signe  sur 1 octet.     */
#define SHORT            HYPER_SHORT   /* Entier signe      sur 2 octets.    */
#define USHORT           HYPER_USHORT  /* Entier non signe  sur 2 octets.    */
#define HHALF            HYPER_USHORT  /* Entier non signe  sur 2 octets.    */
#define INT              HYPER_INT     /* Entier signe      sur 4 octets.    */
#define UINT             HYPER_UINT    /* Entier non signe  sur 4 octets.    */
#define LONG             HYPER_LONG    /* Entier signe      sur 4/8 octets.  */
#define ULONG            HYPER_ULONG   /* Entier non signe  sur 4/8 octets.  */
#define HWORD            HYPER_UINT    /* Entier non signe  sur 4 octets.    */
#define FLOAT            HYPER_FLOAT   /* Reel              sur 4 octets.    */
#define DOUBLE           HYPER_DOUBLE  /* Reel              sur 8 octets.    */
#define OID              INT           /* oid (object identifier)            */

#define UNUSED_PARAM(x)  (void)(x)     /* Suppress Unused Parameter warnings.*/

typedef struct                         /* Nombre reel flottant 64 bits       */
  {                                    /* ---------------------------------- */
    UINT            m1               ; /* b31     = bit de signe             */
                                       /* b30..20 = 11 bits d'exposant       */
                                       /* b19..0  = 20 bits pd fort mantisse */
    UINT            m2               ; /* b31..0  = 32 bits pf faible mant   */
  }                 FL64             ; /* ---------------------------------- */


/* Standard constantes ......................................................*/

#define IW0             (IWORD)0       /* La constante 0 format mot machine. */
#define IW1             (IWORD)1       /* La constante 1 format mot machine. */
#define IWN             (IWORD)-1      /* La constante -1 format mot machine.*/
#define HMAXUCHAR       (INT)0xFF      /* Valeur max. pour CHAR  non signe.  */
#define HMAXUSHORT      (INT)0xFFFF    /* Valeur max. pour SHORT non signe.  */
#define HMAXUINT             0xFFFFFFFF/* Valeur max. pour INT   non signe.  */
#define HMAXCHAR        (INT)0x7F      /* Valeur max. pour CHAR  signe.      */
#define HMAXSHORT       (INT)0x7FFF    /* Valeur max. pour SHORT signe.      */
#define HMAXINT         (INT)0x7FFFFFFF/* Valeur max. pour INT   signe.      */
#define HMAXLONG        HYPER_MAXLONG  /* Valeur max. pour LONG  signe.      */
#define HMAXULONG       HYPER_MAXULONG /* Valeur max. pour LONG  non signe.  */
#define CHLG            '\n'           /* Code ascii du line-feed.           */
#define STDTXT          (INT)1024      /* Longueur d'un buffer texte.        */
#define STDLINE         (INT)256       /* Longueur d'une ligne de caracteres.*/

#define ICACHE_ENABLE   0x01           /* Configurer la cache instruction.   */
#define DCACHE_ENABLE   0x02           /* Configurer la cache donnee     .   */
                                       

/* Standard type sizes ......................................................*/

#define ZC     (INT)(sizeof(CHAR))     /* Taille d'un CHAR.                  */
#define ZS     (INT)(sizeof(SHORT))    /* Taille d'un SHORT.                 */
#define ZI     (INT)(sizeof(INT))      /* Taille d'un INTEGER.               */
#define ZL     (INT)(sizeof(LONG))     /* Taille d'un LONG.                  */
#define ZF     (INT)(sizeof(FLOAT))    /* Taille d'un FLOAT.                 */
#define ZD     (INT)(sizeof(DOUBLE))   /* Taille d'un DOUBLE.                */
#define ZW     (INT)(sizeof(IWORD))    /* Taille d'un IWORD.                 */
#define LGAD   (INT)(sizeof(INT*))     /* Longueur d'un champ adresse.       */
#define NBNS   (INT)5                  /* Nombre d'INT pour buffers noms     */


/* Macros for CHAR array handling ...........................................*/

/* Usage : INS_UINT_format(x)                                                */
/*           format    INTEL, MOTO, HYP                                      */
/*           x         l'entier a inserer dans l'ordre donne par format      */
/*                     dans le tableau                                       */

#define INS_UINT_MOTO(x)   (((UINT)(x) >> 24) & 0xFF),\
                           (((UINT)(x) >> 16) & 0xFF),\
                           (((UINT)(x) >>  8) & 0xFF),\
                           (((UINT)(x) >>  0) & 0xFF)
#define INS_UINT_INTEL(x)  (((UINT)(x) >>  0) & 0xFF),\
                           (((UINT)(x) >>  8) & 0xFF),\
                           (((UINT)(x) >> 16) & 0xFF),\
                           (((UINT)(x) >> 24) & 0xFF)
#ifdef _BIG_ENDIAN_
#define INS_UINT_HYP(x)    (((UINT)(x) >> 24) & 0xFF),\
                           (((UINT)(x) >> 16) & 0xFF),\
                           (((UINT)(x) >>  8) & 0xFF),\
                           (((UINT)(x) >>  0) & 0xFF)
#endif

#ifdef _LITTLE_ENDIAN_
#define INS_UINT_HYP(x)    (((UINT)(x) >>  0) & 0xFF),\
                           (((UINT)(x) >>  8) & 0xFF),\
                           (((UINT)(x) >> 16) & 0xFF),\
                           (((UINT)(x) >> 24) & 0xFF)
#endif


/*  Macros for warning messages .............................................*/

#define Warning0(a)              { warning0((INT)a);                       }
#define Warning1(a,b)            { warning1((INT)a,(IWORD)(b));            }
#define Warning2(a,b,c)          { warning2((INT)a,(IWORD)(b),(IWORD)(c)); }
#define Warning3(a,b,c,d)        { warning3((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d));            }
#define Warning4(a,b,c,d,e)      { warning4((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d),(IWORD)(e)); }
#define Warning5(a,b,c,d,e,f)    { warning5((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d),(IWORD)(e),  \
                                                   (IWORD)(f));            }

#define Warningt0(t,a)           { if (hmsg_flag&t) Warning0(a)            }
#define Warningt1(t,a,b)         { if (hmsg_flag&t) Warning1(a,b)          }
#define Warningt2(t,a,b,c)       { if (hmsg_flag&t) Warning2(a,b,c)        }
#define Warningt3(t,a,b,c,d)     { if (hmsg_flag&t) Warning3(a,b,c,d)      }
#define Warningt4(t,a,b,c,d,e)   { if (hmsg_flag&t) Warning4(a,b,c,d,e)    }
#define Warningt5(t,a,b,c,d,e,f) { if (hmsg_flag&t) Warning5(a,b,c,d,e,f)  }


/*  Macros for error messages ...............................................*/

#define Escap                    { retour = -1;                goto error; }
#define Setret(a)                { *ret   =  a;                goto error; }
#define Abort0(a)                { Warningt0(MERR,a)           Escap       }
#define Abort1(a,b)              { Warningt1(MERR,a,b)         Escap       }
#define Abort2(a,b,c)            { Warningt2(MERR,a,b,c)       Escap       }
#define Abort3(a,b,c,d)          { Warningt3(MERR,a,b,c,d)     Escap       }
#define Abort4(a,b,c,d,e)        { Warningt4(MERR,a,b,c,d,e)   Escap       }
#define Abort5(a,b,c,d,e,f)      { Warningt5(MERR,a,b,c,d,e,f) Escap       }
#define Error0(a)                { Warningt0(MERR,a)           Setret(a)   }
#define Error1(a,b)              { Warningt1(MERR,a,b)         Setret(a)   }
#define Error2(a,b,c)            { Warningt2(MERR,a,b,c)       Setret(a)   }
#define Error3(a,b,c,d)          { Warningt3(MERR,a,b,c,d)     Setret(a)   }
#define Error4(a,b,c,d,e)        { Warningt4(MERR,a,b,c,d,e)   Setret(a)   }
#define Error5(a,b,c,d,e,f)      { Warningt5(MERR,a,b,c,d,e,f) Setret(a)   }
#define War0(a)                  { Warningt0(MWAR,a)           Setret(a)   }


/*  Message and exit without "Escsap" .......................................*/

#define GotoError0(m)            { Warningt0(MERR,(m))         goto error; }
#define GotoError1(m,s)          { Warningt1(MERR,(m),(s))     goto error; }


/*  Other macros ............................................................*/

#define Abs(a)          (((a)>=0)?(a):(-(a)))              /* Valeur absolue.*/

#ifndef nearint                                            /* Entier proche. */
#define nearint(a)      (((a)>=0)?(INT)((a)+0.5):(INT)((a)-0.5)) /*          */
#endif                                                     /*                */

#ifndef intround                                           /* Entier arrondi.*/
#define intround(a,b)   ((a+(a%b))/b)                      /*                */
#endif                                                     /*                */

#ifndef of7                                                /* Offset d'un    */
#define of7(type,champ) ((UINT)&(((type*)0)->champ))       /* champ dans une */
#endif                                                     /* structure.     */

#ifndef Min                                                /* Valeur min.    */
#define Min(a,b)        (((a)<=(b))?(a):(b))               /*                */
#endif                                                     /*                */

#ifndef Max                                                /* Valeur max.    */
#define Max(a,b)        (((a)>=(b))?(a):(b))               /*                */
#endif                                                     /*                */

#ifndef pow2                                               /* Mise au carre. */
#define pow2(a)         ((a)*(a))                          /*                */
#endif                                                     /*                */

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif


/* Libraries fonctions ......................................................*/

#define Strlen(a)        (INT)strlen((char*)(a))
#define Strcpy(a,b)      strcpy((char*)(a),(char*)(b))
#define Strcmp(a,b)      strcmp((char*)(a),(char*)(b))

#define Memcpy(a,b,c)    memcpy((void*)(a),(void*)(b),(unsigned int)(c))
#define Memcmp(a,b,c)    memcmp((void*)(a),(void*)(b),(unsigned int)(c))
#define Memset(a,b,c)    memset((void*)(a),(int)(b),(unsigned int)(c))
#define Memccpy(a,b,c,d) memccpy((void*)(a),(void*)(b),(int)(c),\
                                              (unsigned int)(d))
#define Memmove(a,b,c)   memmove((void*)(a),(void*)(b),(unsigned int)(c))

#define Strcat(a,b)      strcat((char*)(a),(char*)(b))
#define Strncpy(a,b,c)   strncpy((char*)(a),(char*)(b),(unsigned int)(c))
#define Stricmp(a,b)     stricmp((char*)(a),(char*)(b))
#define Strncmp(a,b,c)   strncmp((char*)(a),(char*)(b),(unsigned int)(c))
#define Strnicmp(a,b,c)  strnicmp((char*)(a),(char*)(b),(unsigned int)(c))
#define Strncat(a,b,c)   strncat((char*)(a),(char*)(b),(unsigned int)(c))
#define Strchr(a,b)      (CHAR*)strchr((char*)(a),(int)b)
#define Strrchr(a,b)     (CHAR*)strrchr((char*)(a),(int)b)
#define Strstr(a,b)      (CHAR*)strstr((char*)(a),(char*)(b))
#define Strtol(a,b,c)    (INT)strtol((char*)(a),(char**)(b),(int)(c))

#define divfnt(a,b)      (((a)/(b)) + (((a) % (b)) != 0))

#endif                                 /* Fin de compilation conditionnelle  */
