/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmiodbg.h"             */
/*  Constants and definitions for I/O container test.                        */
/*                                                                           */
/*  Author : RG                         Last modification : 04/11/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations =======================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef VMIODBG_H                      /* If this file is not already        */
#define VMIODBG_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/


/* Driver interruption drv_int.c ............................................*/

typedef struct                         /* Informations stockees par la       */
  BEGIN                                /* procedure "note_it"                */
    INT             temps            ; /* Valeur du temps a l'appel.         */
    INT             level            ; /* Niveau de l'interruption           */
    INT             step             ; /* Etape du traitement                */
    INT             par1,par2        ; /* Parametres de l'etape              */
  END               Note_it          ; /*                                    */

typedef struct                         /* Mesure de temps d'un traitement    */
  BEGIN                                /* ---------------------------------- */
    UINT            dura             ; /* Duree du traitement                */
    IWORD           (*proc)()        ; /* Adresse de la procedure C          */
  END               Dura_hand_it     ; /* ---------------------------------- */
                 
typedef struct                         /* Mesure d'une section critique      */
  BEGIN                                /* ---------------------------------- */
    UINT            dura             ; /* Duree de la section critique.      */
    IWORD           (*proc)()        ; /* Adresse de la procedure C appelante*/
  END               Dura_crit_it     ; /* ---------------------------------- */
                   
                   
/* Driver DMA drv_dma.c .....................................................*/

typedef struct                         /* Informations stockees par "note_d" */
  BEGIN                                /* ---------------------------------- */
    INT             temps            ; /* Valeur du temps a l'appel.         */
    INT             code             ; /* Code de l'action.                  */
    INT             job              ; /* Numero du job concerne.            */
    INT             cnl              ; /* Numero de canal logique.           */
    INT             p1               ; /* Parametre dependant du code        */
    INT             p2               ; /* Parametre dependant du code        */
    INT             p3               ; /* Parametre dependant du code        */
  END               Note_dma         ; /* ---------------------------------- */


/* VMIO vmio.c ..............................................................*/

typedef struct                         /* Informations stockees par "err_s"  */
  BEGIN                                /* ---------------------------------- */
    INT             err              ; /* FAPLEINE, EERNEVT, ...             */
    USHORT          numauto_s        ; /* Numero de l'automate courant       */
    USHORT          etat_s           ; /* Etat de l'automate                 */
    USHORT          voielog_s        ; /* Numero de voie logique             */
    USHORT          code_s           ; /* Code de l'evenement                */
    INT             reserve_s        ; /* Champ reserve de l'evenement       */
  END               Err_evt_s        ; /* ---------------------------------- */

typedef struct                         /* Informations stockees par "note_s  */
  BEGIN                                /* ---------------------------------- */
    INT             temps            ; /* Valeur du temps a l'appel.         */
    INT             code             ; /* EVT_ACT_S,EVT_ADD_S,...            */
    INT             n                ; /* Numero de la file                  */
    INT             nbevt            ; /* Nombre d'evenements de la fle      */
    INT             param4           ; /* Parametre dependant du code        */
    INT             param5           ; /* Parametre dependant du code        */
    INT             param6           ; /* Parametre dependant du code        */
    INT             param7           ; /* Parametre dependant du code        */
  END               Note_evt_s       ; /* ---------------------------------- */

typedef struct                         /* Mesure de temps passe en file      */
  BEGIN                                /* ---------------------------------- */
    UINT            dura             ; /* Temps passe dans la file           */
    USHORT          aut              ; /* Champ "aut"  de l'evenememnt       */
    USHORT          code             ; /* Champ "code" de l'evenement        */
  END               Dura_fa_s        ; /* ---------------------------------- */
 
typedef struct                         /* Mesure de temps d'un traitement    */
  BEGIN                                /* ---------------------------------- */
    UINT            dura             ; /* Duree du traitement                */
    USHORT          code             ; /* Code de l'evenement cause          */
    IWORD           (*proc)()        ; /* Adresse de la procedure C          */
    INT             par              ; /* Parametre d'appel de la procedure  */
    UINT            cpt              ; /* Compteur                           */
  END               Dura_trt_s       ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of VMIODBG_H.                  */
