/*****************************************************************************/
/*                                                                           */
/*  HyperPanel VMOS                     File "stm32/board_stm32py.c"         */
/*  Micro Python Board PYBv1.1 description (STM32F405RGT6).                  */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/11/2018       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/

 
/*  Some explanations ========================================================


         1  Purpose of this module

         2  User manual
            2.1  Contents of the present file
            2.2  Static const data overall schematic
            2.3  System drivers
            2.4  Character drivers
            2.5  Block drivers
            2.6  Volume drivers
            2.7  Protocols
            2.8  Services
            2.9  Interrupt handlers

         3  Include files and external references

         4  Internal constants of the module
            4.1  Timers reference table "ref_tim32"
            4.2  DMA related numbers
            4.3  GPIO reference table "gpio_pindef"
            4.4  DAC related numbers

         5  Exported (public) variables
            5.1  preinit_stm32py program for boot_stm32.s
            5.2  Timer numbers

         6  System drivers definitions
            6.1  SYSTEM Driver
            6.2  BOARD driver
            6.3  MEMORY driver
            6.4  INTERRUPT driver
            6.5  TIME driver
            6.6  VMIO driver
            6.7  GPIO driver
            6.8  I2C driver
            6.9  ASY driver
            6.10 RCU driver
            6.11 DMA driver
            6.12 RESOLVE driver

         7  Character drivers definitions
            7.1  ASY driver
            7.2  LED driver
            7.3  GPIO driver
            7.4  KBDITF driver
            7.5  DAC driver
            7.6  ADC driver

         8  Block drivers definitions
         9  Volume drivers definitions
         10 Protocols definitions
         11 Services definitions



    1    Purpose of this module
    ___________________________


         Ce module contient les donnees de configuration, les parametres des
drivers VMIO de la plateforme Discovery kit F407VG. Il contient:

         - Les parametres des drivers systemes

         - Les parametres des drivers caracteres

         - Les parametres des drivers blocs

         - Les parametres des drivers volumes

         - Les parametres des protocoles

         - Les tables qui regroupent les adresses des donnees precedentes

         - Les quelques procedures C specifiques de ce board.



    2    User manual
    ________________


    2.1  Contents of the present file

         Une carte particuliere est decrite par une arborescence de structures
C, dont la structure "racine" est toujours une structure de type "Board_link".
Les structures "Board_link" fournies par le present modules se trouvent au
point "11.1 Boards".


    2.2  Static const data overall schematic

         Le schema qui suit est une copie du schema que l'on trouve au point
"2.9.1  Generalites - Schema general" de "drv_itf1.c". Pour plus
d'explications, on se reportera donc a "drv_itf1.c".

#DIAGRAM ----------------------------------------------------------------------

INT            Board_link *                           Description des boards
num_board      boards[]
+--------+     +--------+
|        | -+  |        |
+--------+  |  +--------+
            |  |        |      Board_link
            |  +--------+      +------------+
            +->| ADBOARD| -+-> | board_name |-----+
               +--------+  |   | drv_link   |     |
               |        |  |   +------------+     |
               +--------+  |   Une structure      |
               |        |  |   Board_link         |
               +--------+  |                      |
               |        |  |                      |
               +--------+  |                      |
    Tableau d'adresses de  |                      |
    structures Board_link  |                      |
                           |                      |
   Board_link* +--------+  |                      |
        board  | ADBOARD| -+                      |
               +--------+                         |
                                                  |
                                                  |
   +----------------------------------------------+
===|===========================================================================
   |   Drv_link * []                                 Descriptions des drivers
   |   +-----------+                                 systemes
   +-> |           |     Sdrv_link          Sdrv_desc
       +-----------+     +-----------+      +----------+
       |           | --> | sdrv_desc | ---> | drv_name |
       +-----------+     | dev_link  | -+   | typdrv   |
       |           |     | nb_dev    |  |   | init     | --> proc C
       +-----------+     +-----------+  |   | end      | --> proc C
       |           |      Une structure |   +----------+
       +-----------+      Sdrv_link     |
       |           |                    |   Dev_link[]
       +-----------+                    |   +----------+      +-----+
       |           |                    +-> | dev_name |      |     | -> proc C
       +-----------+                        | dev_desc | ---->|     | -> proc C
       |           |                        | dev_def  | ---+ |     | -> proc C
       +-----------+                        | drv_def  | -+ | |     | -> proc C
       |           |                        +----------+  | | |     | -> proc C
       +-----------+                        | dev_name +  | | |     | -> proc C
       |           |                        | dev_desc |  | | |     | -> proc C
       +-----------+                        | dev_def  |  | | +-----+
       |           |                        | drv_def  |  | |
       +-----------+                        +----------+  | |
       |           |                        | dev_name |  | | +-----+
       +-----------+                        | dev_desc |  | +>|     |
       |           |                        | dev_def  |  |   |     |
       +-----------+                        | drv_def  |  |   +-----+
       |           |                        +----------+  |
       +-----------+                                      |   +-----+
       |           |                                      +-->|     |
       +-----------+                                          |     |
       |           |                                          +-----+
====== +-----------+ ==========================================================
       |           |                                  Descriptions des
       +-----------+                                  drivers/protocoles/
       |           |                                  services/drivers volumes
       +-----------+
       |           |     Drv_link          Drv_desc
       +-----------+     +----------+      +----------+
       |           | --> | drv_desc | ---> | drv_name |
       +-----------+     | dev_link | -+   | typdrv   |
       |           |     | nb_dev   |  |   | aut      |
       +-----------+     +----------+  |   | trans    | --> Table transitions
       |           |     Une structure |   | lim      |
       +-----------+     Drv_link      |   | etat     |
       |           |                   |   | reqevt   | --> proc C
       +-----------+                   |   | respevt  | --> proc C
       |     0     |                   |   | init     | --> proc C
       +-----------+                   |   | nbvl     |
                                       |   | nbcon    |
                                       |   +----------+
  Tableau d'adresses                   |
  de structures Drv_link               |   Dev_link[]
                                       |   +----------+
                                       +-> | dev_name |      +-----+
                                           | dev_desc | ---->|     | -> proc C
                                           | dev_def  | ---+ |     | -> proc C
                                           | drv_def  | -+ | |     | -> proc C
                                           +----------+  | | |     | -> proc C
                                           | dev_name |  | | +-----+
                                           | dev_desc |  | |
                                           | dev_def  |  | |
                                           | drv_def  |  | | +-----+
                                           +----------+  | +>|     |
                                           | dev_name |  |   |     |
                                           | dev_desc |  |   +-----+
                                           | dev_def  |  |
                                           | drv_def  |  |
                                           +----------+  |
                                                         |   +-----+
                                                         +-->|     |
                                                             |     |
                                                             +-----+

#END --------------------------------------------------------------------------


    2.3  System drivers

         La table des drivers VMIO est une liste, un tableau d'adresses de
structures Sdrv_link/Drv_link. Pour les drivers systemes, ce sont des
structures "Sdrv_link" qui sont utilisees. Les structures de type "Sdrv_link"
sont dependantes des boards. On va donc trouver ces structures dans le present
fichier. Chaque structure de type "Sdrv_link" contient un pointeur vers une
structure de type "Sdrv_desc".  Ces dernieres ne sont pas definies dans le
present module, on on trouve ici les references vers ces structures externes "
Sdrv_desc".


         Une structure "Sdrv_desc" est l'interface que tout driver systeme
fournit. Cette structure contient quatre champs. L'adresse d'une chaine, qui
est le nom de ce driver, un entier "typdrv" qui vaut 0 pour tous les drivers
systemes, l'adresse "init" d'une procedure d'initialisation et l'adresse "end"
d'une procedure de terminaison. Chaque source C qui constitue un driver systeme
contient sa propre structure de type "Sdrv_desc" .


         Les structures de type "Dev_link" sont dependantes des boards, et
sont donc definies dans le present fichier. Le champ "dev_desc" de ces
structures peut parfois pointer sur des donnees definies a l'exterieur du
present fichier, le plus souvent dans le source d'un driver de "bas niveau".
On trouve ici les references externes des donnees qui sont pointees par un
champ "dev_desc" d'une structure de type "Dev_link".


    2.4  Character drivers


         La table des drivers VMIO est une liste, un tableau d'adresses de
structures Sdrv_link/Drv_link. Pour les drivers caracteres, ce sont des
structures "Drv_link" qui sont utilisees. Les structures de type "Drv_link"
sont dependantes des boards. On va donc trouver ces structures dans le present
fichier. Chaque structure de type "Drv_link" contient un pointeur vers une
structure de type "Drv_desc". Ces dernieres ne sont pas definies dans le
present module, on on trouve ici les references vers ces structures externes
"Drv_desc".


         Une structure "Drv_desc" est l'interface que tout driver caractere,
tout protocole et tout service fournit. Les deux premiers champs sont les
memes que ceux d'une structure "Sdrv_desc", mais la structure "Drv_desc"
contient beaucoup plus de champs que "Sdrv_desc". Chaque source C qui constitue
un driver caractere contient sa propre structure de type "Sdrv_desc" .


         Les structures de type "Dev_link" sont dependantes des boards, et
sont donc definies dans le present fichier. Le champ "dev_desc" de ces
structures peut parfois pointer sur des donnees definies a l'exterieur du
present fichier, le plus souvent dans le source d'un driver de "bas niveau".
On trouve ici les references externes des donnees qui sont pointees par un
champ "dev_desc" d'une structure de type "Dev_link".


    2.5  Block drivers


    2.6  Volume drivers

         La table des drivers du VMIO, qui est une liste d'adresses de
structures de type Sdrv_link/Drv_link, contient un pointeur vers une structure
de type Drv_link pour chaque driver de volume. Dans cette structure, qui
contient trois champs (drv_desc, dev_link et nb_dev) seul le premier est
utilise dans le cas d'une driver de volume. Ce champ "drv_desc" est l'adresse
d'une structure de type "Drv_desc", qui est normalement definie dans le
source C qui constitue le driver de volume. On trouve donc ici les references
externes aux structures "Drv_desc" des drivers de volumes.


    2.7  Protocols

         La table des drivers du VMIO, qui est une liste d'adresses de
structures de type Sdrv_link/Drv_link, contient un pointeur vers une structure
de type Drv_link pour protocole. Comme pour un driver de volume, seul le
premier des trois champs (drv_desc, dev_link et nb_dev) est utilise dans le cas
d'un protocole. Ce champ "drv_desc" est l'adresse d'une structure de type
"Drv_desc", qui est normalement definie dans le source C qui constitue le
driver de volume. On trouve donc ici les references externes aux structures
"Drv_desc" des drivers de volumes.


    2.8  Services


    2.9  Interrupt handlers

          On trouvera en 5.4 une table nommee "ithand", qui contient les
adresses de tous les handlers d'interruptions. On trouve ici les references
externes a ces procedures handlers d'interruptions.


         Les valeurs effectives des parametres de chacune des procedures
doivent etre installes par la procedure appelee.



    3    Include files and external references
    __________________________________________
*/

#include <stdtyp.l>                    /* Definitions et constantes Cojyp.   */
#include <moteur.h>                    /* S_evt                              */
#include <vmio.h>                      /* moteur_s evtto_s                   */
#include <drv.h>                       /* Structures interface haut-niveau   */
#include <drvmio.h>                    /* Structures drivers VMIO            */

#include <automaton.h>                 /* Automaton definitions.             */
#include <engine.h>                    /* Constants for automatong engine.   */
#include <drv_itf.h>                   /* Procedures drv_dps_*               */
#include <memoire.h>                   /* Structure Alloc_head               */
#include <telecom.h>                   /* Constants and definitions for tags.*/

#include <vmiodbg.h>                   /* Note_*, ...  debug vmio            */
#include <emu32.h>                     /* E32_ctx                            */
#include <genio.h>                     /* Const_ctx                          */
#include <keycode.h>                   /* Mnemoniques codes de touches.      */
#include <req_lib.h>                   /*                                    */

#include <drv_adc.h>                   /* ADC                                */
#include <drv_asy.h>                   /* ASY                                */
#include <drv_dac.h>                   /* DAC                                */
#include <drv_gpio.h>                  /* GPIO                               */
#include <drv_i2c.h>                   /* Eep_devdef                         */
#include <drv_int.h>                   /* Driver INT                         */
#include <drv_rcu.h>                   /* RCU_IR                             */
#include <drv_srf.h>                   /* Srf_def, Srf_ctx                   */
/* Temporary add begin *******************************************************/
#define NDRV_DMA_C
/* Temporary add end *********************************************************/
#include <drv_dma.h>                   /* Description of the DMA structures  */
/* Temporary add begin *******************************************************/
#undef NDRV_DMA_C
/* Temporary add end *********************************************************/

#include <stm32.h>                     /* Gpio_def                           */
#include <boot_stm32.h>                /* preinit_stm32py instruction codes  */
#include <adc_stm32.h>                 /* handlerADC_stm32                   */
#include <asy_stm32.h>                 /* handlerASY_stm32                   */
#include <dac_stm32.h>                 /* handlerDAC_stm32                   */
#include <gpio_stm32.h>                /* handlerGPIO_stm32                  */
#include <i2c_stm32.h>                 /* handlerI2C_stm32                   */
#include <sys_stm32.h>                 /* sdrv_sysm3                         */
#include <tim_stm32.h>                 /* handlerASY_stm32                   */
#include <dma_stm32.h>                 /* handlerDMA_stm32                   */

#define STM32F405RG                    /* Needed by <defs_stm32.h>           */
#include <defs_stm32.h>                /* XYZ_IRD defines'                   */


/*
    4    Internal constants of the module
    _____________________________________


    4.1  Timers reference table "ref_tim32"
    ---------------------------------------

    The "ref_tim32[]" table is directly referenced by the "tim_stm32.c" module
    This table is used by the "get_ref_tim_stm32" function from "tim_stm32.c"
    that takes in input a "uname" string character and that returns the
    corresponding "ldev" and "chan" values. The STM32F405 has 14 timers 
    numbered from 1 to 14. We have 5 differents variants that are called
    "Advanced", "Basic" and General purpose (1/2/3). Amongst the General
    purpose (1), we have 2 32 bits timers and 2 16 bits timers

    1    16 bits Advanced
    2    32 bits General purpose (1)
    3    16 bits General purpose (1)
    4    16 bits General purpose (1)
    5    32 bits General purpose (1)
    6    16 bits Basic
    7    16 bits Basic
    8    16 bits Advanced
    9    16 bits General purpose (2)
    10   16 bits General purpose (3)
    11   16 bits General purpose (3)
    12   16 bits General purpose (2)
    13   16 bits General purpose (3)
    14   16 bits General purpose (3)
*/

Ref_tim32 ref_tim32[] =                /* Timers reference table             */
  BEGIN                                /* ---------------------------------- */
/*  uname             ldev    chan        Used by                            */
/*  --------------    ----    ----        ---------------------------------- */
   "INFRARED_TIM"  ,   2   ,   3   ,   /* tim_stm32, RCU timer and input ch. */
   "UHFRCV_TIM"    ,   2   ,   4   ,   /* tim_stm32, RCU timer and input ch. */
   "SYS_TIM"       ,   5   ,  255  ,   /* tim_arm.c                          */
   "FLT_TIM"       ,   6   ,  255  ,   /* tim_stm32, digital filter timer    */
   "UHFSND_TIM"    ,   9   , 255   ,   /* tim_stm32, UHF transmitter         */
   "ASY0_TIM"      ,  10   ,  255  ,   /* asy_stm32.c USART6                 */
   "ASY1_TIM"      ,  11   ,  255  ,   /* asy_stm32.c USART4                 */
   "ASY2_TIM"      ,  12   ,  255  ,   /* asy_stm32.c USART2                 */
   "DAC0_TIM"      ,   6   ,  255  ,   /* dac_stm32.c DAC1                   */
   "ADC0_TIM"      ,   7   ,  255  ,   /* adc_stm32.c ADC1                   */
   HNULL           ,   0   ,    0  ,   /* End of table mark                  */
  END                                ; /* ---------------------------------- */

#define SYS_TIM                 5      /* tim_arm.c                          */
#define FLT_TIM                 6      /* tim_stm32.c                        */
#define DAC1_TIM                6      /* dac_stm32.c  DAC1                  */
#define ADC1_TIM                7      /* adc_stm32.c  ADC1                  */


/*  4.2  DMA related numbers
    ------------------------
*/

#define USART4_DMA              1     /* DMA number                          */
#define USART4_TX_CHDMA         4     /* USART4 TX channel                   */
#define USART4_RX_CHDMA         4     /* USART4 RX channel                   */
#define USART4_TX_STDMA         4     /* USART4 TX stream                    */
#define USART4_RX_STDMA         2     /* USART4 RX stream                    */

#define DAC1_DMA                1     /* DMA device for DAC1                 */
#define DAC1_CHDMA              7     /* DMA channel for DAC1                */
#define DAC1_STDMA              5     /* DMA stream for DAC1                 */

#define ADC1_DMA                1     /* DMA device for ADC1                 */
#define ADC1_ACCU_STDMA        10     /* DMA stream for ADC1 accurate channel*/
#define ADC1_STDMA              0     /* DMA stream for ADC1 channel         */


/*  4.3  GPIO reference table "pindef"
    ----------------------------------
*/

/* Hardware definitions .....................................................

Group     PY paint  Box paint  Pin#  ST name    Signal     Commentary
--------  --------  ---------  ----  ---------  ---------  -----------------
JTAG      P5          -        46    PA13       JTMS       PY red led
          P4          -        49    PA14       JTCK       PY green led
          P3          -        50    PA15       JTDI       PY yellow led
          X17        X17       55    PB3        JTDO       PY USR button
          P2          -        56    PB4        NJTRST     PY blue led

ASY0      TX6        Y1        37    PC6        UART6_TX   ASY0_TX
          RX6        Y2        38    PC7        UART_RX    ASY0_RX

ASY1      X1         X1        14    PA0        UART4_TX   ASY1_TX
          X2         X2        15    PA1        UART4_RX   ASY1_RX
          X7         X7        22    PA6        UART4_RTS  ASY1_RTS
          X8         X8        23    PA7        UART4_CTS  ASY1_CTS

ASY2      X3         X3        16    PA2        UART2_TX   ASY2_TX
          X4         X4        17    PA3        UART2_RX   ASY2_RX

I2C       SCL1       X9        58    PB6        I2C1_SCL   I2C_SCL
          SDA1       X10       59    PB7        I2C1_SDA   I2C_SDA

IRed      Y9         Y9        29    PB10       TIM2_CH3   INFRARED input
UHF in    Y10        Y10       30    PB11       TIM2_CH4   UHF input

LED       Y3         Y3        61    PB8        GPIO out   LEDR
          Y4         Y4        62    PB9        GPIO out   LEDG
          Y5         Y5        33    PB12       GPIO out   LEDY

BUT       Y6         Y6        34    PB13       GPIO in    BUT1
          Y11        Y11       26    PB0        GPIO in    BUT2
          Y12        Y12       27    PB1        GPIO in    BUT3
          X18        X18        2    PC13       GPIO in    BUTUSR
          RST        RST        7    NRST       reset      RST

GPIO      X5         X5        20    PA4        GPIO/DAC1  GPIO1
          X6         X6        21    PA5        GPIO/DAC2  GPIO2
          X11        X11       24    PC4        GPIO/ADC1  GPIO3
          X12        X12       25    PC5        GPIO/ADC2  GPIO4
          Y7         Y7        35    PB14       GPIO       GPIO5
          Y8         Y8        36    PB15       GPIO       GPIO6

BOOT      P1         -         60    BOOT0      BOOT0      Jumper JP2
          -          -         28    PB2/BOOT1  PB2/BOOT   Grounded with 100K
*/

Gpio_pindef gpio_pindef[] =            /* Specific init.                     */
  BEGIN                                /* ---------------------------------- */
/*sname   uname   hidden func      PU/PD  PP/OD  outcfg                      */
/* JTAG signals .............................................................*/
 "PA13", "JTMS"    , 1, "SYS"    , GP_PU, GP_PP, 0     , /* PA13 JTMS        */
 "PA14", "JTCK"    , 1, "SYS"    , GP_PD, GP_PP, 0     , /* PA14 JTCK        */
 "PA15", "JTDI"    , 1, "SYS"    , GP_PU, GP_PP, 0     , /* PA15 JTDI        */
 "PB3" , "JTDO"    , 1, "SYS"    , GP_NP, GP_PP, 0     , /* PB3  JTDO        */
 "PB4" , "NJTRST"  , 1, "SYS"    , GP_PU, GP_PP, 0     , /* PB4  NJTRST      */

/* ASY0 is UART6 ............................................................*/
 "PC6" , "ASY0_TX" , 1, "UART456", GP_PU, GP_PP, 0     , /* PC6  UART6-TX    */
 "PC7" , "ASY0_RX" , 1, "UART456", GP_PU, GP_PP, 0     , /* PC7  UART6-RX    */

/* ASY1 is UART4 ............................................................*/
 "PA0" , "ASY1_TX" , 1, "UART456", GP_PU, GP_PP, 0     , /* PA0  UART4-TX    */
 "PA1" , "ASY1_RX" , 1, "UART456", GP_PU, GP_PP, 0     , /* PA1  UART4-RX    */
 "PA6" , "ASY1_RTS", 1, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PA6  UART3-RTS   */
 "PA7" , "ASY1_CTS", 1, "GPIO"   , GP_PD, GP_PP, GP_LOW, /* PA7  UART3-CTS   */

/* ASY2 is USART2 ...........................................................*/
 "PA2" , "ASY2_TX" , 1, "UART123", GP_PU, GP_PP, 0     , /* PA2  UART2-TX    */
 "PA3" , "ASY2_RX" , 1, "UART123", GP_PU, GP_PP, 0     , /* PA3  UART2-RX    */

/* I2C is I2C1 ..............................................................*/
 "PB6" , "I2C_SCL" , 1, "I2C"    , GP_NP, GP_PP, 0     , /* PB6  I2C1-SCL    */
 "PB7" , "I2C_SDA" , 1, "I2C"    , GP_NP, GP_OD, 0     , /* PB7  I2C1-SDA    */

/* RCU is TIM2_CH3 ..........................................................*/
 "PB10", "INFRARED", 1, "TIM12"  , GP_PU, GP_PP, 0     , /* PB10 TIM2_CH3 RCU*/
 "PB11", "UHFRCV"  , 1, "TIM12"  , GP_PD, GP_PP, 0     , /* PB11 TIM2_CH4 RCU*/

/* LEDs are PB8/PB9/PB12.....................................................*/
 "PB8" , "LEDR"    , 1, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PB8  LED1 RED    */
 "PB9" , "LEDG"    , 1, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PB9  LED2 GREEN  */
 "PB12", "LEDY"    , 1, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PB12 LED3 YELLOW */

/* BUTTONs are PB13/PB0/PB1/PC13 ............................................*/
 "PB13", "BUT1"    , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PB13 BUTTON 1    */
 "PB0" , "BUT2"    , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PB0  BUTTON 2    */
 "PB1" , "BUT3"    , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PB1  BUTTON 3    */
 "PC13", "BUTUSR"  , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PC13 BUTTON USR  */

/* GPIOs are PA4/PA5/PC4/PC5/PB14/PB15 ......................................*/
 "PA4" , "GPIO1"   , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PA4  GPIO 1      */
 "PA5" , "GPIO2"   , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PA5  GPIO 2      */
 "PC4" , "GPIO3"   , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PC4  GPIO 3      */
 "PC5" , "GPIO4"   , 0, "GPIO"   , GP_NP, GP_PP, GP_HIZ, /* PC5  GPIO 4      */
 "PB14", "GPIO5"   , 0, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PB14 GPIO 5      */
 "PB15", "GPIO6"   , 0, "GPIO"   , GP_NP, GP_PP, GP_LOW, /* PB15 GPIO 6      */

  HNULL                              , /* END OF TABLE                       */
  END                                ; /* ---------------------------------- */

/* LEDs are PB8/PB9/PB12 ....................................................*/
#define LEDR_PIO           2,   8     /* PIO number for LEDR RED        PB8  */
#define LEDG_PIO           2,   9     /* PIO number for LEDG GREEN      PB9  */
#define LEDY_PIO           2,   12    /* PIO number for LEDY YELLOW     PB12 */


/*  4.4  DAC related numbers
    ------------------------
*/


/*
    5    Exported (public) variables
    ________________________________


    5.1  preinit_stm32py program for boot_stm32.s
    ---------------------------------------------

    The USR button is connected on PC13, there is no external pull up or pull 
    down. The pin has to be configured as an input with a weak pull-up. The 
    signal is high when the button is not pressed, the bit value read in the 
    input register is 1. When the button os pressed, the signal goes low and 
    we read a 0 in the data input register.

    The "preinit_stm32py" program read the USR button and if the button is not 
    pressed (PC13 state is 1) a 0,5 millisecond positive pulse is send ON PC6
    and the program terminates. If the USR button is pressed then a positive 
    pulse of 1 millisecond on PC6 (This will be the TX6 signal for the USART6) 
    and then a stop on breakpoint is executed

    So if we press the USR button and then, without releasing the USR button,
    we press the RST button then the boot will break. This provides us 
    the opportunity to use the GDB CTRL-C command, set break-points, issue 
    a GDB "c" command and then continue.

    What this "preinit_stm32py" does it as follows :

      - Enable the clock for GPIO bank C. The RCC base address is 0x4002 3800 
        and GPIO bank C uses bit 2. We set bit 2 of register RCC_AHB1EN.
      - We activate PC13 pull-up. We have to set bits b8..9 of register
        GPIOC_PUPDR to 1.
      - Set the pin mode of PC6 to 01 General purpose output, set bits b12..13 
        of GPIOC_MODE to 01
      - Then we wait for 10000 usec
      - We read in the accumulator the USR button state, so PC13 state, bit 13
        of register GPIOC_ID.
      - If the read value is  0, button is pressed 
            . Set GPIO PC6 to 1/high. We set bit 6 of GPIOC_BSRR
            . Wait for 1 msec, so 100 counts of 10 usec
            . Set GPIO PC6 to 0/low. We set bit 22 of GPIOC_BSRR
            . Breakpoint stop
            . After beeing restarted with GDB (c), exit from preinit
        else
            . Set GPIO PC6 to 1/high. We set bit 6 of GPIOC_BSRR
            . Wait for 1 msec, so 100 counts of 10 usec
            . Set GPIO PC6 to 0/low. We set bit 22 of GPIOC_BSRR
            . Exit from preinit
       
    We do not mofify GPIOD_OTYP value since after reset it is push-pull.

*/

const UINT cnt10usec = 39     ; /* See WAIT10US in boot_stm32.c              */

#define RCC_AHB1EN  0x40023830  /* Enable clock for AHB1 peripheral bus dev  */
#define GPIOC_MODE  0x40020800  /* GPIO C Mode register                      */
#define GPIOC_PUPDR 0x4002080C  /* GPIO B pull-up/pull-down register         */
#define GPIOC_ID    0x40020810  /* GPIO B Input data register                */
#define GPIOC_BSRR  0x40020818  /* GPIO C bit set/reset register             */

const UINT preinit_stm32py[][5] =
  BEGIN
/*  Enable clock for the GPIO C device ......................................*/
/*  0 */   PRE_POKE      , RCC_AHB1EN , 0x00000004 , 1 << 2     , PEMPTY     ,
/*  1 */   PRE_WAIT10US  , 1          , PEMPTY     , PEMPTY     , PEMPTY     ,

/*  Enable the pull-up resistor for PC13.....................................*/
/*  2 */   PRE_POKE      , GPIOC_PUPDR, 0x0C000000 , 1 << 26    , PEMPTY     ,

/*  Select GPIO output for PC6 (ASY TX) .....................................*/
/*  3 */   PRE_POKE      , GPIOC_MODE , 0x00003000 , 1 << 12    , PEMPTY     ,

/*  Check if USR is pressed, if not (if state is 1) GOTO 12 .................*/
/*  4 */   PRE_WAIT10US  , 100        , PEMPTY     , PEMPTY     , PEMPTY     ,
/*  5 */   PRE_PEEK      , GPIOC_ID   , 0x00002000 , PEMPTY     , PEMPTY     ,
/*  6 */   PRE_BEQ       , 0x00002000 , 0x00002000 , 12         , PEMPTY     ,

/* USR button is pressed. Send a 1 msec pulse and stop on BKPT ..............*/
/*  7 */   PRE_POKE      , GPIOC_BSRR , 0x00000000 , 1 << 6     , PEMPTY     ,
/*  8 */   PRE_WAIT10US  , 100        , PEMPTY     , PEMPTY     , PEMPTY     ,
/*  9 */   PRE_POKE      , GPIOC_BSRR , 0x00000000 , 1 << 22    , PEMPTY     ,
/* 10 */   PRE_BKPT      , 0x11111111 , PEMPTY     , PEMPTY     , PEMPTY     ,
/* 11 */   PRE_B         , 15         , PEMPTY     , PEMPTY     , PEMPTY     ,

/* USR button is not pressed. Send a 0,5 msec pulse an exit .................*/
/* 12 */   PRE_POKE      , GPIOC_BSRR , 0x00000000 , 1 << 6     , PEMPTY     ,
/* 13 */   PRE_WAIT10US  , 50         , PEMPTY     , PEMPTY     , PEMPTY     ,
/* 14 */   PRE_POKE      , GPIOC_BSRR , 0x00000000 , 1 << 22    , PEMPTY     ,

/* 15 */   PRE_ENDPGM    , PEMPTY     , PEMPTY     , PEMPTY     , PEMPTY     ,
  END;

/*
    5.2  Timer numbers
    ------------------
*/

const INT sys_tim = SYS_TIM          ; /* tim_arm.c                          */
const INT flt_tim = FLT_TIM          ; /* tim_stm32.c                        */


/*
+++ Poubelle
*/

IWORD PROCEDURE sdrv_empty(drv) IWORD drv; { return(0); }

static IWORD board_init                (void                                 );
static void  infoLED                   (IWORD,const CHAR**,IWORD*,
                                        const CHAR***,
                                        const UCHAR**,const UCHAR**          );
static void  setLED                    (IWORD,IWORD,IWORD                    );



/*
    6    System drivers definition
    ______________________________


    6.1  SYSTEM Driver
    ------------------

         Le driver systeme configure les horloges du CPU et du bus.
*/

static const UINT sysdef[] =           /* Parametres du systeme              */
  BEGIN                                /* ---------------------------------- */

    168000000                       ,  /* SYSCLK                             */
    168000000                       ,  /* HCLK    (Horloge bus AHB)          */
     42000000                       ,  /* PCLK1   (Horloge bus APB1)         */
     84000000                       ,  /* PCLK2   (Horloge bus APB2)         */

    0x00608007                      ,  /* Config ahb1en : DMA1&2 (3<<21),    */
                                       /* flash (1<<15), GPIOA,B&C (7<<0)    */
    0x00060000                      ,  /* Config apb1en : USART2&3 (3<<17)   */
    0x00000000                      ,  /* Config apb2en                      */

/* @TODO Remplacer ces champs par un appel de la proc sys_ena + supprimer    */
/* l'etape 5 de la proc init_sysm4.                                          */

  END                                ; /* ---------------------------------- */

static const Dev_link dev_sys =        /* Device SYSTEM                      */
  BEGIN                                /* ---------------------------------- */
    "SYS_F405"                      ,  /* Nom du device                      */
    (const void *)HNULL             ,  /* Description du device              */
    sysdef                          ,  /* Parametres du systeme              */
    (const void *)HNULL             ,  /* Pas de parametres du driver        */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_sys    = /* Driver SYSTEM                      */
  BEGIN                                /* ---------------------------------- */
    &sdrv_sysm4                     ,  /* Ad. interface  du  driver          */
    &dev_sys                        ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'SYSTEM'                  */
  END                                ; /* ---------------------------------- */



/*
    6.2  BOARD driver
    -----------------

         On ne fait rien du tout dans le board.

*/

static const Sdrv_desc sdrv_board =    /* Driver systeme de ce board.        */
  BEGIN                                /* ---------------------------------- */
    "BOARD_MICRO-PYTHON"            ,  /* Nom du driver.                     */
    DRV_SYS                         ,  /* Type du driver.                    */
    board_init                      ,  /* Procedure d'initialisation         */
    sdrv_empty                      ,  /* Procedure de terminaison           */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_board =  /* Driver BOARD Rev0                  */
  BEGIN                                /* ---------------------------------- */
    &sdrv_board                     ,  /* Ad. interface  du  driver          */
    HNULL                           ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'BOARD'                   */
  END                                ; /* ---------------------------------- */



/*
    6.3  MEMORY driver
    ------------------



    6.3.1  VMOS allocations
    .......................

         La zone memoire du VMOS est celle qu'il utilise pour toutes les
allocations dynamiques du VMOS. 

*/

static const Ram_def_vmos ram_def_vmos  ; /* Parametrage memoire VMOS        */


/*
    6.3.2  Services
    ...............

         Il n'y a pas de services pour l'instant.
*/

static const Ram_def_srv ram_def_srv   ;  /* Parametrage memoires services   */



/*
    6.3.3  Telecom buffers
    ......................
*/

/*****************************************************************************/
/* RAM pour buffer telecoms : 24 Ko                                          */
/* La valeur de cette adresse est renseignee par le linker!                  */
/*****************************************************************************/

extern UINT      ad_buf_part0           ; /* Importe depuis le linker        */
extern UINT      lg_buf_part0           ; /* Importe depuis le linker        */
extern UINT      end_buf_part0          ; /* Importe depuis le linker        */


const Ram_def_part ram_def_part[] =       /* Parametrage memoire Telecom     */
  BEGIN                                   /* ------------------------------- */

/*****************************************************************************
 * 1ERE et DERNIERE PARTITION                                                *
 *****************************************************************************/
    BEGIN                                 /* Definition de partition ------- */

    1                                  ,  /* Un seul segment memoire         */
    /* PJB : Pourquoi cette valeur? .........................................*/
    4576                               ,  /* Taille d'un bigbuff             */

    (UCHAR*)&  ad_buf_part0             , /* Ad. de debut du segment         */
    (UCHAR*)& end_buf_part0 - 1         , /* Ad. de fin   du segment         */
    (UINT  )&  lg_buf_part0             , /* Taille en octet du segment.     */
    (USHORT)1                             /* Segment en cache.               */

    END                                , /* -------------------------------- */

  END                                   ; /* ------------------------------- */


/*
    6.3.4 Definition table
    ......................
*/

const Ram_def_dev ram_def_dev        ; /* Importe memoires des devices       */

const Ram_def ram_def =                /* Parametrage de la RAM              */
  BEGIN                                /* ---------------------------------- */
    HNULL                           ,  /* Memoire pages teletexte            */
    &ram_def_vmos                   ,  /* Memoire allocations VMOS           */
    &ram_def_srv                    ,  /* Memoire services                   */
    &ram_def_dev                    ,  /* Memoire devices                    */
    ram_def_part                    ,  /* Memoire partitions buffers         */
    1                               ,  /* Deux partitions de buffers         */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_ram =        /* Configurations memoire RAM         */
  BEGIN                                /* ---------------------------------- */
    "RAM"                           ,  /* Nom du device                      */
    HNULL                           ,  /* Sur systeme embarque               */
    (void *)&ram_def                ,  /* Parametrage de la RAM              */
    HNULL                           ,  /* Une seule configuration de RAM     */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_mem =    /* Driver RAM                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_ram                       ,  /* Ad. interface  du  driver          */
    &dev_ram                        ,  /* Ad. interface  des devices         */
    1                               ,  /* 1 device 'RAM'                     */
  END                                ; /* ---------------------------------- */


/*
    6.4 INTERRUPT driver
    --------------------
*/

/* Handlers ................................................................ */

#define H_MOTEURS       0
#define H_TIME          1
#define H_EVTTOS        2
#define H_GPIO00        3
#define H_GPIO01        4
#define H_GPIO02        5
#define H_GPIO03        6
#define H_GPIO04        7
#define H_GPIO05        8
#define H_GPIO06        9
#define H_GPIO07       10
#define H_GPIO08       11
#define H_GPIO09       12
#define H_GPIO10       13
#define H_GPIO11       14
#define H_GPIO12       15
#define H_GPIO13       16
#define H_GPIO14       17
#define H_GPIO15       18
#define H_I2C_EV       19
#define H_I2C_ERR      20
#define H_ASY0         21
#define H_ASY1         22
#define H_ASY2         23
#define H_ASY0_DMATX   24
#define H_ASY0_DMARX   25
#define H_ASY0_TIM     26
#define H_ASY1_TIM     27
#define H_ASY2_TIM     28
#define H_RCURCV       29
#define H_RCUSND       30
#define H_DAC          31
#define H_DAC_DMA      32
#define H_ADC          33

/* Handlers chaining ....................................................... */

static const Ithand_t ithand[] =                        /* Chainage handlers */
  BEGIN                                                 /* ----------------- */
    0xFFFFFFFF             , moteur_s          ,  0, 0, /*  0 : H_MOTEURS    */
                                                        /*                   */
    (H_EVTTOS<<24)|0xFFFFFF, handlerTIM_armv7m ,  0, 0, /*  1 : H_TIME       */
    0xFFFFFFFF             , evtto_s           ,  0, 0, /*  2 : H_EVTTOS     */
                                                        /*                   */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  0, 0, /*  3 : H_GPIO00     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  1, 0, /*  4 : H_GPIO01     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  2, 0, /*  5 : H_GPIO02     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  3, 0, /*  6 : H_GPIO03     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  4, 0, /*  7 : H_GPIO04     */
                                                        /*                   */
    (H_GPIO06<<24)|0xFFFFFF, handlerGPIO_stm32 ,  5, 0, /*  8 : H_GPIO05     */
    (H_GPIO07<<24)|0xFFFFFF, handlerGPIO_stm32 ,  6, 0, /*  9 : H_GPIO06     */
    (H_GPIO08<<24)|0xFFFFFF, handlerGPIO_stm32 ,  7, 0, /* 10 : H_GPIO07     */
    (H_GPIO09<<24)|0xFFFFFF, handlerGPIO_stm32 ,  8, 0, /* 11 : H_GPIO08     */
    0xFFFFFFFF             , handlerGPIO_stm32 ,  9, 0, /* 12 : H_GPIO09     */
                                                        /*                   */
    (H_GPIO11<<24)|0xFFFFFF, handlerGPIO_stm32 , 10, 0, /* 13 : H_GPIO10     */
    (H_GPIO12<<24)|0xFFFFFF, handlerGPIO_stm32 , 11, 0, /* 14 : H_GPIO11     */
    (H_GPIO13<<24)|0xFFFFFF, handlerGPIO_stm32 , 12, 0, /* 15 : H_GPIO12     */
    (H_GPIO14<<24)|0xFFFFFF, handlerGPIO_stm32 , 13, 0, /* 16 : H_GPIO13     */
    (H_GPIO15<<24)|0xFFFFFF, handlerGPIO_stm32 , 14, 0, /* 17 : H_GPIO14     */
    0xFFFFFFFF             , handlerGPIO_stm32 , 15, 0, /* 18 : H_GPIO15     */
                                                        /*                   */
    0xFFFFFFFF             , handlerevI2C_stm32,  0, 0, /* 19 : H_I2C_EV     */
    0xFFFFFFFF             , handlererrI2C_stm32, 0, 0, /* 20 : H_I2C_ERR    */
    0xFFFFFFFF             , handlerASY_stm32  ,  0, 0, /* 21 : H_ASY0       */
    0xFFFFFFFF             , handlerASY_stm32  ,  1, 0, /* 22 : H_ASY1       */
    0xFFFFFFFF             , handlerASY_stm32  ,  2, 0, /* 23 : H_ASY2       */
    0xFFFFFFFF             , handlerDMA_stm32  , USART4_DMA, USART4_TX_STDMA,
                                                        /* 24 : H_ASY0_DMATX */
    0xFFFFFFFF             , handlerDMA_stm32  , USART4_DMA, USART4_RX_STDMA,
                                                        /* 25 : H_ASY0_DMARX */
    0xFFFFFFFF             , handlerTIM_stm32  , 10, 0, /* 26 : H_ASY0_TIM   */
    0xFFFFFFFF             , handlerTIM_stm32  , 11, 0, /* 27 : H_ASY1_TIM   */
    0xFFFFFFFF             , handlerTIM_stm32  , 12, 0, /* 28 : H_ASY2_TIM   */
                                                        /*                   */
    0xFFFFFFFF             , handlerRCU_stm32  ,  0, 0, /* 29 : H_RCURCV     */
    0xFFFFFFFF             , handlerSND_stm32  ,  1, 0, /* 30 : H_RCUSND     */
    0xFFFFFFFF             , handlerDAC_stm32  ,  0, 0, /* 31 : H_DAC        */
    0xFFFFFFFF             , handlerdmaDAC_stm32, 0, 0, /* 32 : H_DAC_DMA    */
    0xFFFFFFFF             , handlerADC_stm32  ,  0, 0, /* 33 : H_ADC        */
                                                        /*                   */
    0                                                   /* MARQUEUR DE FIN   */
  END                                                 ; /* ----------------- */

/* Raccorde les tetes de chainage des handlers ............................. */

static const Ithead_t ithead[] =       /* Association code d'IT <-> handler  */
  BEGIN                                /* ---------------------------------- */

/* Priorite 0, on y raccorde les fils d'IT device inutilisees .............. */
#define L_ITPRI0  0

/* Priorite 1, ce fil est commande par logiciel ............................ */
#define L_ITPRI1  L_ITPRI0 + 1

    0      , 1 , H_MOTEURS           , /* IRQ 0 , interruption logicielle    */

/* Priorite 2 .............................................................. */
#define L_ITPRI2  L_ITPRI1 + 8

   UART6_IRQ       , 1 , H_ASY0      , /* IRQ71 , USART6                     */
   UART4_IRQ       , 1 , H_ASY1      , /* IRQ52 , USART4                     */
   UART2_IRQ       , 1 , H_ASY2      , /* IRQ38 , USART2                     */
   DMA1_STREAM4_IRQ, 1 , H_ASY0_DMATX, /* IRQ14 , DMA1 stream 4 : USART4 TX  */
   DMA1_STREAM2_IRQ, 1 , H_ASY0_DMARX, /* IRQ12 , DMA1 stream 2 : USART4 RX  */
   TIM10_IRQ       , 1 , H_ASY0_TIM  , /* IRQ25 , timer utilise par l'USART6 */
   TIM11_IRQ       , 1 , H_ASY1_TIM  , /* IRQ26 , timer utilise par l'USART4 */
   TIM12_IRQ       , 1 , H_ASY2_TIM  , /* IRQ43 , timer utilise par l'USART2 */

/* Priorite 3 .............................................................. */
#define L_ITPRI3  L_ITPRI2 + 7

   EXTI0_IRQ       , 1 , H_GPIO00    , /* IRQ 6 , GPIO 0                     */
   EXTI1_IRQ       , 1 , H_GPIO01    , /* IRQ 7 , GPIO 1                     */
   EXTI2_IRQ       , 1 , H_GPIO02    , /* IRQ 8 , GPIO 2                     */
   EXTI3_IRQ       , 1 , H_GPIO03    , /* IRQ 9 , GPIO 3                     */
   EXTI4_IRQ       , 1 , H_GPIO04    , /* IRQ10 , GPIO 4                     */
   EXTI9_5_IRQ     , 1 , H_GPIO05    , /* IRQ23 , GPIO 5 a GPIO 9            */
   EXTI15_10_IRQ   , 1 , H_GPIO10    , /* IRQ40 , GPIO 10 a GPIO 15          */

/* Priorite 4 .............................................................. */
#define L_ITPRI4  L_ITPRI3 + 2

   TIM2_IRQ        , 1 , H_RCURCV    , /* IRQ28 , TIMER 2 : RCU              */
   TIM9_IRQ        , 1 , H_RCUSND    , /* IRQ24 , TIMER 9 : RCU SND          */

/* Priorite 5 .............................................................. */
#define L_ITPRI5  L_ITPRI4 + 3

   TIM6_IRQ        , 1 , H_DAC       , /* IRQ54 , TIMER 6 : DAC              */
   DMA1_CH5_IRQ    , 1 , H_DAC_DMA   , /* IRQ12 , DMA1 stream 5 : DAC        */
   ADC1_IRQ        , 1 , H_ADC       , /* IRQ18 , ADC                        */

/* Priorite 6 .............................................................. */
#define L_ITPRI6  L_ITPRI5 + 3

    -1             , 1 , H_TIME      , /* SysTick, decompteur systeme.       */
   I2C1_EV_IRQ     , 1 , H_I2C_EV    , /* IRQ31 , I2C1 event                 */
   I2C1_ER_IRQ     , 1 , H_I2C_ERR   , /* IRQ32 , I2C1 error                 */

/* Priorite 7 .............................................................. */
#define L_ITPRI7  L_ITPRI6 + 0

  END                                ; /* ---------------------------------- */


static const IWORD itidx[] =           /* Priorites des interruptions        */
  BEGIN                                /* ---------------------------------- */
    0        , L_ITPRI0 , L_ITPRI1 ,   /* Delimitation des priorites dans    */
    L_ITPRI2 , L_ITPRI3 , L_ITPRI4 ,   /* la table "it_head"                 */
    L_ITPRI5 , L_ITPRI6 , L_ITPRI7 ,   /*                                    */
  END                                ; /* ---------------------------------- */


/* Definition du driver Interruption ....................................... */

static const Itdev_t itdev =           /* Definition locale pour INT.        */
  BEGIN                                /* ---------------------------------- */
    itidx                           ,  /* Priorites des interruptions        */
    ithead                          ,  /* Association code d'IT <-> handler  */
    ithand                          ,  /* Handlers d'interruption            */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_int =        /* Device INTERRUPT                   */
  BEGIN                                /* ---------------------------------- */
    "INT_F407"                      ,  /* Nom du device                      */
    (const void *)&int_desc_nvic    ,  /* Controleur d'interruption NVIC     */
    (const void *)&itdev            ,  /* Ad. structure parametres  device   */
    (const void *)HNULL             ,  /* Pas de parametres driver/dev       */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_int =    /* Driver INTERRUPT                   */
  BEGIN                                /* ---------------------------------- */
    &sdrv_int                       ,  /* Ad. interface  du  driver          */
    &dev_int                        ,  /* Ad. interfaces des devices         */
    1                               ,  /* 1 device 'INT'                     */
  END                                ; /* ---------------------------------- */



/*  6.5 TIME driver
    ---------------
*/

static const Dev_link dev_tim =        /* Device TIME                        */
  BEGIN                                /* ---------------------------------- */
    "TIM_F405"                       , /* Nom du device                      */
    (const void *)HNULL              , /* Pas de description device          */
    (const void *)HNULL              , /* Pas de parametres du peripherique  */
    (const void *)HNULL              , /* Pas de parametres du systeme       */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_tim =    /* Driver TIME                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_timarmv7m                  , /* Ad. interface  du driver           */
    &dev_tim                         , /* Ad. interfaces des devices         */
    1                                , /* 1 device 'TIME'                    */
  END                                ; /* ---------------------------------- */



/*  6.6  VMIO driver
    ----------------
*/

static const Sdrv_link sdrv_l_vmio =   /* Driver systeme VMIO                */
  BEGIN                                /* ---------------------------------- */
    &sdrv_vmio                      ,  /* Driver systeme generique           */
    HNULL                           ,  /* Pas de configuration               */
    0                               ,  /* Aucun device                       */
 END                                 ; /* ---------------------------------- */


/*
    6.7  GPIO driver
    ----------------

         On fait le choix de considerer qu'il n'y a qu'un device GPIO. On
pourrait aussi bien dire qu'il y en a 8 (1 par banque), mais ca n'apporte
aucun benefice.

*/

/* Contextes GPIO ...........................................................*/

#define GPIO_NBCTL          1          /* Only one device.                   */
#define GPIO_NBPIN        144          /* Number of pins.                    */
#define GPIO_NBVL     (1+1+48)         /* Total number of logical channels.  */


static const Dev_link dev_gpio =       /* Device GPIO                        */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                          ,  /* Nom du device                      */
    (void*)&gpio_desc_stm32         ,  /* Interface bas-niveau.              */
    (void*)&gpio_pindef             ,  /* Description board                  */
    HNULL                           ,  /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_gpio =   /* Driver GPIO                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_gpio                      ,  /* Driver systeme                     */
    &dev_gpio                       ,  /* Configuration des devices GPIO     */
    GPIO_NBCTL                      ,  /* 1 device GPIO                      */
  END                                ; /* ---------------------------------- */



/*  6.8  I2C driver
    ---------------
*/

/* Contextes I2C ............................................................*/

#define NUM_I2C              1         /* Number of used I2Cs                */

const UCHAR     i2c_ctxmax = 3       ; /* Nombre de contextes disponibles.   */
I2c_ctx         i2c_ctx[NUM_I2C]     ; /* Contextes.                         */
USHORT volatile i2c_etat[3]          ; /* Etat de voie logique               */

static const
    I2c_stm32_def i2c_def[NUM_I2C]   = /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* ST I2C1 ****************************/
    100000                           , /* SCL frequency                      */
  END                                ; /* ---------------------------------- */

static const
    I2c_drvdef i2c_drvdef[NUM_I2C]   = /* Device definition                  */
  BEGIN                                /* ---------------------------------- */
    10                               , /* Duree attente entre 2 requetes     */
  END                                ; /* ---------------------------------- */

static const Dev_link sdev_i2c[]     = /* Device I2C                         */
  BEGIN                                /* ---------------------------------- */
    "I2C1"                           , /* Nom du device                      */
    &i2c_desc_stm32                  , /* Description du device I2C          */
    (void*)&i2c_def[0]               , /* Description du device I2C          */
    (void*)&i2c_drvdef[0]            , /* Description du device I2C          */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_i2c    = /* Driver I2C                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_i2c                        , /* Ad. interface  du  driver          */
    sdev_i2c                         , /* Ad. interfaces des devices         */
    NUM_I2C                          , /* Number of used I2Cs                */
  END                                ; /* ---------------------------------- */



/*  6.9  ASY driver
    ---------------
*/

/* Contextes ASY ............................................................*/

#define NUM_ASY              3         /* Number of used UARTs               */
#define BUF_SIZE            64         /* Circular buffer size               */
#define NUM_OF_SEG           2         /* Number of segments of circular buff*/

/* Temporary add begin *******************************************************/
extern const Sdrv_desc sdrv_dma2        ; /* DMA       system drv interface  */
/* Temporary add end *********************************************************/

static const
    Asy_stm32_def asy_def[NUM_ASY] =   /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */
                                       /*                                    */
    6                                , /*** USART6 ***************************/
    "ASY0_TIM"                       , /* Timer number = 10                  */
    0xFF                             , /* DMA number (not used)              */
    0xFF                             , /* USART6 TX stream (not used)        */
    0xFF                             , /* USART6 RX stream (not used)        */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    HNULL          , 0               , /* CTS                                */
    HNULL          , 0               , /* DSR                                */
    HNULL          , 0               , /* RI                                 */
    HNULL          , 0               , /* DCD                                */
    HNULL          , 0               , /* DTR                                */
    HNULL          , 0               , /* RTS                                */
    "ASY0_RX"      , 0               , /* RXB     USART6 RX (PC7)            */
    "ASY0_TX"      , 0               , /* TXB     USART6 TX (PC6)            */
                                       /*                                    */
    4                                , /*** UART4 ****************************/
    "ASY1_TIM"                       , /* Timer number = 11                  */
    USART4_DMA                       , /* DMA number = 1                     */
    USART4_TX_STDMA                  , /* DMA TX stream number = 4           */
    USART4_RX_STDMA                  , /* DMA RX stream number = 2           */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    "ASY1_CTS"     , 1               , /* CTS     UART4 CTS (PA7)            */
    HNULL          , 0               , /* DSR                                */
    HNULL          , 0               , /* RI                                 */
    HNULL          , 0               , /* DCD                                */
    HNULL          , 0               , /* DTR                                */
    "ASY1_RTS"     , 1               , /* RTS     UART4 RTS (PA6)            */
    "ASY1_RX"      , 0               , /* RXB     UART4 RX  (PA1)            */
    "ASY1_TX"      , 0               , /* TXB     UART4 TX  (PA0)            */
                                       /*                                    */
    2                                , /*** USART2 ***************************/
    "ASY2_TIM"                       , /* Timer number = 12                  */
    0xFF                             , /* DMA number (not used)              */
    0xFF                             , /* USART2 TX stream (not used)        */
    0xFF                             , /* USART2 RX stream (not used)        */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    HNULL          , 0               , /* CTS                                */
    HNULL          , 0               , /* DSR                                */
    HNULL          , 0               , /* RI                                 */
    HNULL          , 0               , /* DCD                                */
    HNULL          , 0               , /* DTR                                */
    HNULL          , 0               , /* RTS                                */
    "ASY2_RX"      , 0               , /* RXB     USART2 RX (PA3)            */
    "ASY2_TX"      , 0               , /* TXB     USART2 TX (PA2)            */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const
    Asy_drvdef asy_drvdef[NUM_ASY] =   /* Device definition                  */
  BEGIN                                /* ---------------------------------- */
                                       /* ST USART 6                         */
    0                                , /* Numero de contexte de bas niveau   */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
                                       /* ---------------------------------- */
                                       /* ST UART 4                          */
    1                                , /* Numero de contexte de bas niveau   */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    OTH_DMA |                          /* DMA supporte en hardware           */
    OTH_CTS                          , /* CTS supporte en hardware           */
                                       /* ---------------------------------- */
                                       /* ST USART 2                         */
    2                                , /* Numero de contexte de bas niveau   */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static UCHAR circ_buffer[BUF_SIZE]   ; /* Circular buffer                    */

const Circ_buf circ_buf[NUM_ASY] =     /* Circular buffer definition         */
  BEGIN                                /* ---------------------------------- */
                                       /*                                    */
    &circ_buffer[0]                  , /* Circular buffer base address       */
    BUF_SIZE / NUM_OF_SEG            , /* Segment size in bytes              */
    NUM_OF_SEG                       , /* Total number of segments           */
    1                                , /* Number of next segment to write    */
    0                                , /* Number of next segment to be read  */
    NUM_OF_SEG                       , /* Number of used segments            */
                                       /*                                    */
    HNULL                            , /* Circular buffer base address       */
    0                                , /* Segment size in bytes              */
    0                                , /* Total number of segments           */
    0                                , /* Number of next segment to write    */
    0                                , /* Number of next segment to be read  */
    0                                , /* Number of used segments            */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const Dev_link  sdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "ASY USART6"                     , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[0]               , /* Description du device UART         */
    (void*)&asy_drvdef[0]            , /* Description du device UART         */
                                       /*                                    */
    "ASY UART4"                      , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[1]               , /* Description du device UART         */
    (void*)&asy_drvdef[1]            , /* Description du device UART         */
                                       /*                                    */
    "ASY USART2"                     , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[2]               , /* Description du device UART         */
    (void*)&asy_drvdef[2]            , /* Description du device UART         */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_asy   =  /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_asy                        , /* Ad. interface  du  driver          */
    sdev_asy                         , /* Ad. interfaces des devices         */
    NUM_ASY                          , /* Nb de devices 'ASY' utilises       */
  END                                ; /* ---------------------------------- */



/*  6.10 RCU driver
    ---------------
*/

extern const Rcu_desc rcu_desc_stm32 ;

static const Rcu_drv_def rcurcv_high = /* High level RCU parameters          */
  BEGIN                                /* ---------------------------------- */
    0                                , /* typdev    = receiver               */
    0                                , /* definput  = first                  */
    2                                , /* nbinput   = 2 inputs               */
                                       /* input 0  ......................... */
    RCU_IR                           , /* link      = infra-red              */
    1                                , /* signal    = inverted               */
    "ECEL JU090601"                  , /* defrcu    = ECEL JU090601          */
    56000                            , /* freq      = Vishay LED receiver    */
    0                                , /* banduhf   = unused, must be 0      */
                                       /* input 1  ......................... */
    RCU_UHF                          , /* link      = UHF                    */
    0                                , /* signal    = non inverted           */
    "DIO 54591"                      , /* defrcu    = DIO 54591              */
    0                                , /* freq      = ??                     */
    433920000                        , /* banduhf   = 433.92 Mhz             */
  END                                ; /* ---------------------------------- */

static const Rcu_drv_def rcusnd_high = /* High level RCU parameters          */
  BEGIN                                /* ---------------------------------- */
    1                                , /* typdev    = transmitter            */
    0                                , /* definput  = first                  */
    2                                , /* nboutput  = 2 outputs              */
                                       /* output 0  ........................ */
    RCU_UHF                          , /* link      = UHF                    */
    0                                , /* signal    = non inverted           */
    "RAW"                            , /* defrcu    = RAW                    */
    56000                            , /* freq      = Vishay LED receiver    */
    433920000                        , /* banduhf   = unused, must be 0      */
                                       /* output 1  ........................ */
    RCU_IR                           , /* link      = IR                     */
    1                                , /* signal    = inverted               */
    "RAW"                            , /* defrcu    = ??                     */
    0                                , /* freq      = ??                     */
    0                                , /* banduhf   = vide                   */
  END                                ; /* ---------------------------------- */

static const CHAR* const rcurcv_low[]= /* Low-level RCU parameters           */
  BEGIN                                /* ---------------------------------- */
    "INFRARED_TIM"                   , /* Timer user name for input 0        */
    "UHFRCV_TIM"                     , /* Timer user name for input 1        */
  END                                ; /* ---------------------------------- */

static const CHAR* const rcusnd_low[]= /* Low-level RCU parameters           */
  BEGIN                                /* ---------------------------------- */
    "UHFSND_TIM"                     , /* Timer user name                    */
    "GPIO5"                          , /* GPIO  user name for output 0       */
    "GPIO6"                          , /* GPIO  user name for output 1       */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_rcu[2] =     /* Description specifique             */
  BEGIN                                /* ---------------------------------- */
    "RCV_STM32"                      , /* Nom du device                      */
    (void *)&rcu_desc_stm32          , /* Low-level procedures addresses     */
    (void *)&rcurcv_low              , /* Low-level parameter = input names  */
    (void *)&rcurcv_high             , /* High-level parameters              */

    "SND_STM32"                      , /* Nom du device                      */
    (void *)&rcu_desc_stm32          , /* Low-level procedures addresses     */
    (void *)&rcusnd_low              , /* Low-level parameter = output names */
    (void *)&rcusnd_high             , /* High-level parameters              */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_rcu =    /* Driver RCU                         */
  BEGIN                                /* ---------------------------------- */
    &sdrv_rcu                        , /* Driver systeme generique           */
    dev_rcu                          , /* Configuration                      */
    2                                , /* 2 devices, 1 receiver, 1 transmit  */
  END                                ; /* ---------------------------------- */



/*  6.11 DMA driver
    ---------------
*/

#define NUM_DMA              2         /* Number of DMAs                     */

static const Dma_stm32_def dma_def[NUM_DMA] = /* Low level device definition */
  BEGIN                                /* ---------------------------------- */
                                       /* DMA1 ----------------------------- */
    0xFF                             , /* Stream 0 not used                  */
    0xFF                             , /* Stream 1 not used                  */
    USART4_RX_CHDMA                  , /* Stream 2: UART4 RX                 */
    0xFF                             , /* Stream 3 not used                  */
    USART4_TX_CHDMA                  , /* Stream 4: UART4 TX                 */
    DAC1_CHDMA                       , /* Stream 5: DAC1                     */
    0xFF                             , /* Stream 6 not used                  */
    0xFF                             , /* Stream 7 not used                  */
                                       /* DMA2 ----------------------------- */
    0xFF                             , /* Stream 0 not used                  */
    0xFF                             , /* Stream 1 not used                  */
    0xFF                             , /* Stream 2 not used                  */
    0xFF                             , /* Stream 3 not used                  */
    0xFF                             , /* Stream 4 not used                  */
    0xFF                             , /* Stream 5 not used                  */
    0xFF                             , /* Stream 6 not used                  */
    0xFF                             , /* Stream 7 not used                  */
  END                                ; /* ---------------------------------- */

static const Dma_drvdef dma_drvdef[NUM_DMA] = /* Device definition           */
  BEGIN                                /* ---------------------------------- */
                                       /* DMA1 ----------------------------- */
    0xFF                             , /* Device index for low level driver  */
    0xFF                             , /* Number of channels                 */
    0xFF                             , /* 0:General Purpose 1:Specialized    */
    HNULL                            , /* Maximum numbers of allowed alloc's */
    0xFF                             , /* Available options bitmaps          */
    0xFF                             , /* Memcpy/DMA usage limit             */
                                       /* DMA2 ----------------------------- */
    0xFF                             , /* Device index for low level driver  */
    0xFF                             , /* Number of channels                 */
    0xFF                             , /* 0:General Purpose 1:Specialized    */
    HNULL                            , /* Maximum numbers of allowed alloc's */
    0xFF                             , /* Available options bitmaps          */
    0xFF                             , /* Memcpy/DMA usage limit             */
  END                                ; /* ---------------------------------- */

static const Dev_link  sdev_dma[] =    /* Device DMA                         */
  BEGIN                                /* ---------------------------------- */
    "DMA1"                           , /* Nom du device                      */
    &dma_desc_stm32                  , /* Description du device DMA          */
    (void*)&dma_def[0]               , /* Description du device DMA          */
    (void*)&dma_drvdef[0]            , /* Description du device DMA          */
                                       /*                                    */
    "DMA2"                           , /* Nom du device                      */
    &dma_desc_stm32                  , /* Description du device DMA          */
    (void*)&dma_def[1]               , /* Description du device DMA          */
    (void*)&dma_drvdef[1]            , /* Description du device DMA          */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sdrv_l_dma   =  /* Driver DMA                         */
  BEGIN                                /* ---------------------------------- */
//    &sdrv_dma                        , /* Ad. interface  du  driver          */
    &sdrv_dma2                       , /* Ad. interface  du  driver          */
    sdev_dma                         , /* Ad. interfaces des devices         */
    NUM_DMA                          , /* 2 devices 'DMA'                    */
  END                                ; /* ---------------------------------- */


/*  6.12 RESOLVE driver
    -------------------

    Ce driver systeme ne pilote pas de hardware, mais sert a la resolution de
nom d'automate pour l'initialisation des drivers caracteres.
*/

static const Sdrv_link sdrv_l_rslv =   /* Driver RESOLVE                     */
  BEGIN                                /* ---------------------------------- */
    &sdrv_rslv                      ,  /* Ad. interface du driver            */
    HNULL                           ,  /* Ad. interfaces des devices         */
    0                               ,  /* Pas de device pour RESOLVE         */
  END                                ; /* ---------------------------------- */




/*
    7    Character drivers definitions
    __________________________________

*/

#define BOARD_XYZ_DATA_C               /* Driver space/data                  */

/*  7.1  ASY driver
    ---------------
*/

/* Contextes UART ...........................................................*/

#include <drv_asy.h>                   /* asy_nbiod asy_nbvl asy_etat        */

static const Dev_link  cdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[0]               , /* Description du device UART         */
    (void*)&asy_drvdef[0]            , /* Description du device UART         */
                                       /*                                    */
    "DEV1"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[1]               , /* Description du device UART         */
    (void*)&asy_drvdef[1]            , /* Description du device UART         */
                                       /*                                    */
    "DEV2"                           , /* Nom du device                      */
    &asy_desc_stm32                  , /* Description du device UART         */
    (void*)&asy_def[2]               , /* Description du device UART         */
    (void*)&asy_drvdef[2]            , /* Description du device UART         */
  END                                ; /* ---------------------------------- */

static const Drv_link  cdrv_l_asy =    /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_asy                    , /* Description du driver              */
    cdev_asy                         , /* Description des devices            */
    NUM_ASY                          , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */


/*  7.2  LED driver
    ---------------
*/


/* Contextes LED ........................................................... */

#define LEDCTRL_NB   1                 /* Count of LED controllers           */

#define LED_NB0      3                 /* Controller 0  LED number           */
#define LED_NB1      0                 /* Controller 1  LED number           */
#define LED_NB2      0                 /* Controller 2  LED number           */
#define LED_NB3      0                 /* Controller 3  LED number           */
#define LED_NBMAX    LED_NB0           /* Maximum of 4 previous              */

#include <drv_led.h>                   /* led_nbiod, led_nbvl, led_etat      */

USHORT led_tsubcnt [LEDCTRL_NB] = { LED_NB0 } ;

static const Led_desc led_desc =       /* Description du bas-niveau          */
  BEGIN                                /* ---------------------------------- */
    infoLED                          , /*                                    */
    setLED                           , /*                                    */
  END                                ; /* ---------------------------------- */

static const Dev_link cdev_led =       /* Device LED STM32                   */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    &led_desc                        , /* Description device LED 0           */
    HNULL                            , /* Pas de parametres                  */
    HNULL                            , /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_led =     /* Driver LED STM32                   */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_led                    , /* Description du driver              */
    &cdev_led                        , /* Description des devices            */
    LEDCTRL_NB                       , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */

/* Description des LEDs .................................................... */

static const CHAR *lednames[LED_NBTOT]=/* Noms des LEDs                      */
  BEGIN                                /* ---------------------------------- */
    "RED", "GREEN", "YELLOW"         , /* Noms des LEDs par couleurs         */
  END                                ; /* ---------------------------------- */

static const UCHAR ledcol[LED_NBTOT] = /* Couleurs des LEDs                  */
  BEGIN                                /* ---------------------------------- */
    C_RED, C_GREEN , C_YELLOW        , /* LEDs tri-colores.                  */
  END                                ; /* ---------------------------------- */

static const UCHAR ledsta[LED_NBTOT] = /* Etats des LEDs                     */
  BEGIN                                /* ---------------------------------- */
    ST_OFF, ST_OFF, ST_OFF           , /* Toutes les LEDs sont eteintes.     */
  END                                ; /* ---------------------------------- */

static const UCHAR ledpio[LED_NBTOT][2] = /* Table des GPIOs de controle LEDs*/
  BEGIN                                /* ---------------------------------- */
    LEDR_PIO                         , /* LEDR sur PB8                       */
    LEDG_PIO                         , /* LEDG sur PB9                       */
    LEDY_PIO                         , /* LEDY sur PB12                      */
  END                                ; /* ---------------------------------- */


/*  7.3  GPIO driver
    ----------------
*/

#define GPIO_NBIOD 1                   /* Must be LE than GPIO_NBCTL         */

#include <drv_gpio.h>                  /* led_nbiod, led_nbvl, led_etat      */

USHORT gpio_tsubcnt [GPIO_NBIOD] = { GPIO_NBPIN } ;

static const Drv_link cdrv_l_gpio =    /* Driver GPIO                        */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_gpio                  ,  /* Driver systeme                     */
    &dev_gpio                       ,  /* Configuration des devices GPIO     */
    GPIO_NBIOD                      ,  /* 1 device GPIO                      */
  END                                ; /* ---------------------------------- */


/*  7.4  KBDITF driver
    ------------------
*/

Itf_ctx kbditf_itfctx[1+1] = { 0 } ;

static const Dev_link dev_kbditf[1] =  /* Device KBDITF                      */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    HNULL                            , /* Description device                 */
    HNULL                            , /* Pas de parametres                  */
    HNULL                            , /* Pas de description driver          */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_kbditf  = /* Driver KBDITF                      */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_kbditf                 , /* Driver KBDITF                      */
    dev_kbditf                       , /* Configuration                      */
    1                                , /* 1 device                           */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */


/*  7.5  DAC driver
    ---------------
*/

/* Contextes DAC ........................................................... */

Dac_ctx      dac_ctx     [1]          ; /* Contextes des DACs.               */
USHORT       dac_states  [2]          ; /* Logical channels states.          */
const USHORT dac_nbiod =  1           ; /* Number of visble devices.         */
const USHORT dac_nbvl  =  2           ; /* Number of logical channels.       */

Itf_ctx dac_itfctx[1+1] = { 0 } ;

static const Dac_drvdef dac_drvdef[1]= /* High-level device definition       */
  BEGIN                                /* ---------------------------------- */
    0                                , /* Low-level device index.            */
    2                                , /* Number of channels.                */
    DAC_8BIT | DAC_12BIT | DAC_16BIT , /* Device supports 8, 12, 16 bits.    */
    DAC_8BIT | DAC_12BIT | DAC_16BIT , /* DMA    supports 8, 12, 16 bits.    */
    33                               , /* Voltage reference: 3.3V.           */
  END                                ; /* ---------------------------------- */

static const Dac_stm32_def dac_def[1]= /* Low-level device parameters        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* Type: mono: 0, stereo: 1           */
    1                                , /* DAC identifier.                    */
    DAC1_TIM                         , /* Timer identifier.                  */
    DAC1_DMA                         , /* DMA 1                              */
    DAC1_STDMA                       , /* Stream 5                           */
  END                                ; /* ---------------------------------- */


static const Dev_link cdev_dac =       /* Device DAC STM32                   */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
   // &mono_dac_desc_stm32           , /* Description device DAC 0, mono mode*/
    &dac_desc_stm32_stereo           , /* Description device DAC 0, mono mode*/
    &dac_def[0]                      , /* Parametres devices                 */
    &dac_drvdef[0]                   , /* Parametres driver/dev              */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_dac =     /* Driver DAC STM32                   */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_dac                    , /* Description du driver              */
    &cdev_dac                        , /* Description des devices            */
    1                                , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */



/*  7.6  ADC driver
    ---------------
*/

/* Contextes ADC ........................................................... */

Adc_ctx      adc_ctx     [1]          ; /* Contextes des ADCs.                */
USHORT       adc_states  [2]          ; /* Logical channels states.           */
const USHORT adc_nbiod =  1           ; /* Number of visble devices.          */
const USHORT adc_nbvl  =  2           ; /* Number of logical channels.        */

Itf_ctx adc_itfctx[1+1] = { 0 } ;

static const Adc_drvdef adc_drvdef[1]= /* High-level device definition       */
  BEGIN                                /* ---------------------------------- */
    0                                , /* lctx        Low-level device index */
    (1<<8) | (1<<12)                 , /* supp_depth  supports 8 and 12 bits */
    0                                , /* supp_dma    DMA not supported yet  */
  END                                ; /* ---------------------------------- */

static const Adc_stm32_def adc_def[1]= /* Low-level device parameters        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* adc_num                            */
    4                                , /* tim_num   Tim number accurate chan */
    ADC1_DMA                         , /* dma_num                            */
    0x9 /* TIM4_TRGO */              , /* jext_trig Trigger ext injected chan*/
    11                               , /* accu_ch   accurate chanel number   */
    ADC1_STDMA                       , /* dma_ch = 0                         */
//    4                                , /* Stream 4                           */
  END                                ; /* ---------------------------------- */

static const Dev_link cdev_adc =       /* Device ADC STM32                   */
  BEGIN                                /* ---------------------------------- */
    "DEV0"                           , /* Nom du device                      */
    &adc_desc_stm32                  , /* Description device ADC 0           */
    &adc_def[0]                      , /* Parametres devices                 */
    &adc_drvdef[0]                   , /* Parametres driver/dev              */
  END                                ; /* ---------------------------------- */

static const Drv_link cdrv_l_adc =     /* Driver ADC STM32                   */
  BEGIN                                /* ---------------------------------- */
    &drv_desc_adc                    , /* Description du driver              */
    &cdev_adc                        , /* Description des devices            */
    1                                , /* Nombre de devices                  */
    HNULL                              /* Interface pipe                     */
  END                                ; /* ---------------------------------- */




/*
    8    Block drivers definitions
    ______________________________


    9    Volume drivers definitions
    _____________________________


    10   Protocols definitions
    __________________________


    11   Services definitions
    _________________________


    12   Variables de l'interface (exportes)
    ________________________________________


    11.1 Board
    __________
*/

static const Drv_link * const drv_f407[] =       /* Liste des drivers VMIO   */
  BEGIN                                          /* ------------------------ */

/* Liste des drivers systeme ............................................... */

    (const Drv_link *)&sdrv_l_sys             ,  /*  0 SYSTEM                */
    (const Drv_link *)&sdrv_l_board           ,  /*  1 BOARD                 */
    (const Drv_link *)&sdrv_l_mem             ,  /*  2 MEMORY                */
    (const Drv_link *)&sdrv_l_int             ,  /*  3 INTERRUPT             */
    (const Drv_link *)&sdrv_l_tim             ,  /*  4 TIME                  */
    (const Drv_link *)&sdrv_l_vmio            ,  /*  5 VMIO                  */
    (const Drv_link *)&sdrv_l_gpio            ,  /*  6 GPIO                  */
    (const Drv_link *)&sdrv_l_dma             ,  /*  7 DMA                   */
    (const Drv_link *)&sdrv_l_i2c             ,  /*  8 I2C                   */
    (const Drv_link *)&sdrv_l_asy             ,  /*  9 ASY                   */
    (const Drv_link *)&sdrv_l_rcu             ,  /* 10 RCU                   */
    (const Drv_link *)&sdrv_l_rslv            ,  /* 11 RESOLVE (avant carac) */

/* Liste des drivers caracteres ............................................ */

//    (const Drv_link *)&cdrv_l_i2c             ,  /* 12 I2C                   */
    (const Drv_link *)&cdrv_l_asy             ,  /* 13 ASY                   */
    (const Drv_link *)&cdrv_l_gpio            ,  /* 14 GPIO                  */
    (const Drv_link *)&cdrv_l_led             ,  /* 15 LED                   */
    (const Drv_link *)&cdrv_l_kbditf          ,  /* 16 KBDITF                */
    (const Drv_link *)&cdrv_l_dac             ,  /* 17 DAC                   */
    (const Drv_link *)&cdrv_l_adc             ,  /* 18 ADC                   */

/* Liste des drivers bloc .................................................. */
/* Liste des drivers de volume ............................................. */
/* Liste des protocoles .................................................... */
/* Liste des services ...................................................... */
/* Marque de fin de liste .................................................. */

    HNULL                                     ,  /* FIN DE LISTE             */
  END                                          ; /* ------------------------ */

Board_link board_f407 =                /* Board Discovery kit F407VG         */
  BEGIN                                /* ---------------------------------- */
    "F407"                          ,  /* Nom du board                       */
    drv_f407                        ,  /* Liste des drivers                  */
  END                                ; /* ---------------------------------- */


Board_link *boards[] =                 /* Definitions des boards             */
 BEGIN                                 /* ------------ |MC  |CPU| RAM|Flash| */
  &board_f407                       ,  /* 0 STM32F407  |F407|  0|192K|   1M| */
 END                                 ; /* ---------------------------------+ */


/* ATTENTION : Cette table utilise comme indice le major number, qui est
   suppose etre un indice dans "vmos_driver" UNIQUEMENT (cf board_xyz.c)
   En attendant que je nettoie cette salete, il faut penser a verifier que
   "drv_itf_ctx" est bien coherente avec "vmod_driver".
*/

Drv_itf_ctx drv_itf_ctx[] =
  BEGIN
/*   itfctx          sub_vl        nbiod        nbmodd   sessid_cond_act[]   */
/*   -------------   -----------   ----------   ------   -----------------   */
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,
  {  asy_itfctx    , HNULL       , NUM_ASY    , 1      , {0}              } ,
  {  led_itfctx    , led_sub_vl  , LEDCTRL_NB , 1      , {0}              } ,
  {  gpio_itfctx   , gpio_sub_vl , GPIO_NBCTL , 1      , {0}              } ,
  {  kbditf_itfctx , HNULL       , 1          , 1      , {0}              } ,
  {  dac_itfctx    , HNULL       , 1          , 1      , {0}              } ,
  {  adc_itfctx    , HNULL       , 1          , 1      , {0}              } ,
  END                                ;


/*
    11.2 Tables de debug
    ____________________

         Afin d'aider a l'identification de bugs qui pourraient se produire
apres un temps de fonctionnement assez long, un certain nombre de tables de
debug sont renseignees par les differents modules du VMIO. Ces tables peuvent
avoir des tailles tres consequentes et il y a des cas ou on souhaite les
reduire au minimum (loader) et d'autre ou veut les augmenter au maximum (debug
d'un pb sur une platteforme donnee). On definit donc ces tables dans les
differents fichiers boards.

*/

/* Driver interruption "int_f407.c" ........................................ */

const IWORD max_note_it =   50     ; /* Taille de la table de debug          */
Note_it     tnote_it     [  50]    ; /* Buffer circulaire de notes           */


/* Driver dma "dma_f407.c" ................................................. */

const IWORD max_note_dma = 10      ; /* Taille de la table de debug          */
Note_dma    tnote_dma     [10]     ; /* Buffer circulaire de notes           */


/* VMIO "vmio.c" ........................................................... */

const IWORD max_trace_evt_s = 10   ; /* Taille de la table de debug          */
Err_evt_s   terr_evt_s       [10]  ; /* Buffer circulaire de notes           */

const IWORD max_note_evt_s =  20   ; /* Taille de la table de debug          */
Note_evt_s  tnote_evt_s      [20]  ; /* Buffer circulaire de notes           */



/*
    12   Procedures des drivers systemes/caracteres du board
    ________________________________________________________
*/



/*  Procedure board_init --------------------------------------------------- */
/*
    But : Initialiser les parametres globaux du board.
 
    Auteur : AL                              Derniere modification : 13/11/2017

...............................................................................
 
    Exemple d'appel : board_init()
 
    En entree : -
 
    En sortie : -
 
    Notes : Cette procedure est specifique au board.

            NB : Le driver "board" est le 2eme dans l'ordre d'initialisation
                 des drivers systeme, apres le driver "sys" qui initialise
                 les pointeurs sur les registres des devices.
 
...............................................................................
*/
static IWORD PROCEDURE board_init()

  BEGIN                                /* Debut de cette procedure.          */

/*  Declaration des variables locales, internes de cette procedure ..........*/

/*  Debut du code ...........................................................*/

#ifdef debug                           /* Debut compilation conditionnelle.  */
    procdeb("board_init")            ; /* Message.                           */
#endif                                 /* Fin compilation conditionnelle.    */

/*****************************************************************************
 * Etape 1 : On active l'usage des mises-en-sommeil du processeur par des    *
 * -------   appels a "vm_sleep".                                            *
 *****************************************************************************/

//  set_idle(1)                      ; /* On active la fonction.             */


/*****************************************************************************
 * Etape 2 : Si necessaire pour debugguer le GPIO, on met les pins JTAG dans *
 * -------   le mode adapte. Sinon, on est un peu coince.                    *
 *****************************************************************************/


#ifdef debug                           /* Debut compilation conditionnelle.  */
    procfin("board_init")            ; /* Message.                           */
#endif                                 /* Fin compilation conditionnelle.    */

    return (0)                       ; /* Code de retour de la procedure.    */

  END_PROC




/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    But : Ne rien faire quand on nous propose de dormir.

    Auteur : PJB                             Derniere modification : 15/10/2015

...............................................................................

    Exemple d'appel : vm_sleep(par)

    En entree : par      IWORD      Parametre : 0 pour ignore, 1 pour dormir.

    En sortie : procval  IWORD      0.

    Notes : On ne fait jamais rien ici.

...............................................................................
*/
IWORD PROCEDURE vm_sleep(IWORD par)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    if (par) asm("WFI")              ; /* Si par = 1, on utilise WFI.        */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return 0                         ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */



/*  Procedure infoLED -------------------------------------------------------*/
/*
    But : Remonter les informations du controleur.

    Auteur : AL                              Derniere modification : 22/11/2017

...............................................................................

    Exemple d'appel : infoLED(dev,&name,&nbleds,&colors,&states)

    En entree : dev      IWORD      Numero du device (controleur de leds)

    En sortie : name     const CHAR**  Adresse de  la chaine contenant le nom
                                         du circuit
                nbleds   IWORD*        Nombre de leds raccordees au device
                colors   const UCHAR** Tableau (de taille "nbleds") donnant la
                                         couleur de chaque led raccordee au
                                         device
                states   const UCHAR** Tableau (de taille "nbleds") donnant
                                         l'etat initial de chaque led raccordee
                                         au device


    Notes : Cette procedure est specifique au board.

...............................................................................
*/
static void PROCEDURE infoLED(IWORD dev, const CHAR **name, IWORD *nbleds,
                              const CHAR ***names, const UCHAR **colors,
                              const UCHAR **states                         )
                               

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("infoLED")               ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    *name   = "Discovery kit F407VG" ; /* Nom du board.                      */
    *nbleds = LED_NBTOT              ; /* Nombre de LEDs pilotables.         */
    *names  = lednames               ; /* Table des couleurs.                */
    *colors = ledcol                 ; /* Table des couleurs.                */
    *states = ledsta                 ; /* Table des etats.                   */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("infoLED")               ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return                           ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */




/*  Procedure setLED --------------------------------------------------------*/
/*
    But : Modifier l'etat d'une LED.

    Auteur : AL                              Derniere modification : 22/11/2017

...............................................................................

    Exemple d'appel : setLED(dev, ssc, etat)

    En entree : dev      IWORD      Numero du device (controleur de leds)
                sscanal  IWORD      Numero du sous-canal (led)
                etat     IWORD      Allume (1) ou eteint (0)

    En sortie : -

    Notes : Cette procedure est specifique au board.

...............................................................................
*/
static void PROCEDURE setLED(IWORD dev, IWORD ssc, IWORD etat)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("setLED")                ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    set_output_stm32(ledpio[ssc][0],
                     ledpio[ssc][1],
                     etat) ;

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("setLED")                ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return                           ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */



#if 0

/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    But : Realiser une mise en veille.

    Auteur : PJB                             Derniere modification : 07/10/2015

...............................................................................

    Exemple d'appel : vm_sleep(val)

    En entree : val      IWORD      Parametre de la veille :
                                    - SLEEP_NONE  (0), pas de veille
                                    - SLEEP_CPU   (1), arret de l'horloge CPU
                                    - SLEEP_MOST  (2), arret de presque tout
                                    - SLEEP_SHTDWN(3), arret complet

    En sortie : procval  IWORD      0 si OK, -1 sinon

    Notes :

...............................................................................
*/
IWORD vm_sleep(IWORD) __attribute__((section(".text.ram")));
IWORD PROCEDURE vm_sleep(IWORD val)

  BEGIN                                /* Debut de cette procedure           */

/* Declaration des variables locales, internes a cette procedure ............*/

    IWORD          ret = -1          ; /* Code de retour de la procedure.    */

extern volatile Rcc_ctrl* const rcc  ; /*                                    */
extern volatile Scs     * const scs  ; /*                                    */
extern volatile UINT    * const pwr  ; /*                                    */

#define SLEEP_NONE 0
#define SLEEP_CPU  1
#define SLEEP_MOST 2

/* Debut du code ............................................................*/

#ifdef debug                           /* Debut compilation conditionnelle   */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    switch (val)                       /* On saute a l'etape selon la valeur */
                                       /* de "val".                          */
      BEGIN                            /* ---------------------------------- */

/*****************************************************************************
 * Etape 0 : Aucune veille                                                   *
 * -------   Si val = SLEEP_NONE (=0) ou que c'est une valeur farfelue, on   *
 *           ne fait rien.                                                   *
 *****************************************************************************/
    default : vhalt()                ; /* Valeur inattendue : on vient voir  */
    ret = -1                         ; /*                                    */

    case SLEEP_NONE  :                 /*                                    */
    goto exit                        ; /*                                    */


/*****************************************************************************
 * Etape 1 : Veille du CPU seul                                              *
 * -------   Si val = SLEEP_CPU (=1), on met le CPU en attente. Cela         *
 *           correspond au mode sleep du STM32L1xx. Pour entrer dans ce      *
 *           mode, il suffit d'attendre une interruption : WFI               *
 *           Si on n'a pas desactive le compteur de milisecondes, on va etre *
 *           reveille bientot.                                               *
 *           Pour rappel : STM32L152RE @ 2Mhz se reveille en 7us vs 1ms au   *
 *           plus.                                                           *
 *           Attention : le mode low-power sleep peut sembler interessant    *
 *           (consommation 4 fois moindre), mais le temps de reveil est bien *
 *           plus grand. Il vaut mieux faire l'effort de trouver les vrais   *
 *           temps de veille et passer en SLEEP_MOST.                        *
 *****************************************************************************/
    case SLEEP_CPU   :                 /*                                    */

    scs->sc = 0                      ; /* Not about to deepsleep             */
    asm("WFI")                       ; /* On se met en attente d'une IT.     */

    goto exit                        ; /*                                    */

/*****************************************************************************
 * Etape 2 : Veille du CPU et des peripheriques (sauf le minimum pour etre   *
 * -------   reveille quand meme). On garde aussi la RAM.                    *
 *           Pour le STM32L1xx :                                             *
 *           - On s'assure que la RTC est bien active, sinon bonne-nuit!     *
 *           - On s'assure que l'oscillateur lent externe est bien active.   *
 *           - On leve le bit sleepdeep.                                     *
 *           - On baisse le bit WUP de PWR_CR. Ce bit indique qu'un reveil   *
 *             a eu lieu, et il empeche de passer en veille profonde.        *
 *           - On leve les bits FWU et ULP. Le second permet de desactiver   *
 *             le maintien de Vrefint pendant que l'on passe en mode basse-  *
 *             consommation. Le premier permet de sortir de basse consom-    *
 *             mation avant que Vrefint ne soit revenu a sa valeur.          *
 *           - On active le mode low power du regulateur : cela devrait      *
 *             reduire la consommation (mais on n'a pas la valeur de cette   *
 *             reduction), et augmenter le temps de remise en veille. Ce     *
 *             temps passe de 8 a 10us en moyenne ...                        *
 *           - Enfin, on utilise l'instruction WFI.                          *
 *****************************************************************************/
    case SLEEP_MOST  :                 /*                                    */

    if ((rcc->csr & 0x430300) NE       /* On verifie RTC, et LSE             */
                    0x410300   )       /*                                    */
      BEGIN                            /* En cas d'erreur, on vient voir,    */
        vhalt()                      ; /* mais on ne se met pas en veille.   */
        goto exit                    ; /* On sort a la place.                */
      END_IF                           /*                                    */

    scs->sc = 4                      ; /* About to deepsleep                 */
    pwr[0] &= ~0x602                 ; /* Clear FWU, ULP, PDDS               */
    pwr[0] |=      1                 ; /* Enables low power deepsleep        */

    asm("WFI")                       ; /* On se met en attente d'une IT.     */

    goto exit                        ; /*                                    */


    END                                /* ---------------------------------- */


    exit :                             /*                                    */

#ifdef debug                           /* Debut compilation conditionnelle   */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* Fin compilation conditionnelle     */

    return ret                       ; /* Code de retour de la procedure     */

  END_PROC                             /* Fin de la procedure                */
#endif
